package coco.codegen.coco;
import coco.utils.*;
import coco.parser.ast.*;
import coco.codegen.AbstractCodeGenerator;
import java.util.Enumeration;
import java.util.Vector;
/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author
 * @version 1.0
 */



public class CoCoGenerator extends AbstractASTVisitor implements AbstractCodeGenerator{
  private static CoCoGenerator instance = null;
  private StringBuffer result = null;

  static int indentLevel = 0;
  static final char indentChar=' ';


  public void init(){
    result = new StringBuffer();
  }

  public String getResult(){
    return result.toString();
  }


  private void incIndent(){
    indentLevel++;
  }

  private void decIndent(){
    indentLevel--;
    if(indentLevel < 0)
      indentLevel = 0;
  }


  private String indent(int level){
    Assert.require(level>=0);
    char[] res = new char[2*level];
    for(int i=0;i<2*level;i++)
      res[i] = indentChar;
    return new String(res);
  }

  public static CoCoGenerator getInstance(){
    if(instance == null)
      instance = new CoCoGenerator();
    Assert.require(instance);
    instance.init();
    return instance;
  }

  protected CoCoGenerator(){
    init();
  }


	public boolean preVisit(){
		return true;
	}
	public boolean postVisit(){
		return true;
	}

	public Object visitPre(ASTComponentList node){
		Assert.require(false);
		return null;
	}
	public Object visitIntraList(ASTComponentList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTComponentList node){
		Assert.require(false);
		return null;
	}




  public Object visitPre(ASTComponent node){
    ASTComments comments = node.getBeforeComment();
    if(comments != null)
      result.append(comments.toString());

    if(node.isActive())
      result.append("active ");
    result.append("component ");
    return null;
  }


  public Object visitPost(ASTComponent node){
    ASTComments comments = node.getAfterComment();
   if(comments != null)
      result.append(comments.toString());
		return null;
  }

  public Object visitPre(ASTInterfaceIdentifierList node){
    result.append(" is ");
    return null;
  }

  public Object visitIntraList(ASTInterfaceIdentifierList node){
    result.append(", ");
    return null;
  }


  public Object visitPost(ASTInterfaceIdentifierList node){
    // nothing to do here
    return null;
  }

  public Object visitPre(ASTInterfaceIdentifier node){
    result.append(node.getID());
    return null;
  }
  public Object visitPost(ASTInterfaceIdentifier node){
    // nothing to do here
    return null;
  }


  public Object visitPre(ASTComponentBody node){
    result.append("{\n");
    this.incIndent();
    return null;
  }
  public Object visitPost(ASTComponentBody node){
    result.append("}\n");
    this.decIndent();
    return null;
  }
  public Object visitPre(ASTPortList node){
    result.append("\n");
    result.append("/**ports*/\n");

    return null;
  }

  public Object visitIntraList(ASTPortList node){
    result.append("\n");
    return null;
  }


  public Object visitPost(ASTPortList node){
    result.append("\n");
    result.append("/**end ports*/\n");

    return null;
  }
  public Object visitPre(ASTData node){
    ASTComments comments = node.getBeforeComment();
    if(comments != null)
      result.append(comments.toString());

    // take care of indentation
    result.append(this.indent(this.indentLevel));
    ASTPortModifiers modifiers = node.getModifiers();
    if(modifiers != null)
      result.append(modifiers.getFirstModifierString()+" ");


    result.append(node.getDirection()+ " ");
    result.append("port ");
    return null;
  }
  public Object visitPost(ASTData node){
    ASTComments comments = node.getAfterComment();

    result.append(";");
    if(comments != null)
      result.append(comments.toString());
    return null;
  }


  public Object visitPre(ASTHandler node){
    ASTComments comments = node.getBeforeComment();
    if(comments != null)
      result.append(comments.toString());

    // take care of indentation
    result.append(this.indent(this.indentLevel));
    ASTPortModifiers modifiers = node.getModifiers();
    if(modifiers != null)
      result.append(modifiers.getFirstModifierString()+" ");


    result.append(node.getDirection()+ " ");
    result.append("port ");

          return null;
  }
  public Object visitPost(ASTHandler node){
    ASTComments comments = node.getAfterComment();

    result.append(";");
    if(comments != null)
      result.append(comments.toString());
          return null;
  }
	public Object visitPre(ASTPortModifiers node){
              result.append(node.getFirstModifierString());
              return null;
	}
	public Object visitPost(ASTPortModifiers node){
		return null;
	}

  public Object visitPre(ASTInvocationSignature node){
    result.append("(");
    return null;
  }
  public Object visitPost(ASTInvocationSignature node){
    result.append(")");
    return null;
  }

    public Object visitPre(ASTInvocationParameterList node){
      return null;
    }

    public Object visitIntraList(ASTInvocationParameterList node){
      result.append(", ");
      return null;
    }


    public Object visitPost(ASTInvocationParameterList node){
      return null;
    }


    public Object visitPre(ASTInvocationParameter node){
      return null;
    }
    public Object visitPost(ASTInvocationParameter node){
      return null;
    }

  public Object visitPre(ASTPortBody node){
    result.append("{\n");
    this.incIndent();
    return null;
  }
  public Object visitPost(ASTPortBody node){
    this.decIndent();
    result.append(this.indent(this.indentLevel) + "}");
    return null;
  }



  public Object visitPre(ASTPropertyList node){
    result.append("\n");
    result.append("/**properties*/\n");

    return null;
  }
  public Object visitIntraList(ASTPropertyList node){
    result.append("\n");
    return null;
  }
  public Object visitPost(ASTPropertyList node){
    result.append("\n");
    result.append("/**end properties*/\n");

    return null;
  }


  public Object visitPre(ASTProperty node){
    ASTComments comments = node.getBeforeComment();
    if(comments != null)
      result.append(comments.toString());
    result.append(this.indent(this.indentLevel)+"property ");
    result.append(node.getVisibilityString()+ " ");
    return null;
  }
  public Object visitPost(ASTProperty node){
    result.append(";");
    ASTComments comments = node.getAfterComment();
    if(comments != null)
      result.append(comments.toString());

    return null;
  }

	public Object visitPre(ASTInstanceList node){
            result.append("\n");
            result.append("/**instances*/\n");
            this.incIndent();
            return null;
	}

	public Object visitIntraList(ASTInstanceList node){
		result.append("\n");
		return null;
	}



	public Object visitPost(ASTInstanceList node){
		this.decIndent();
		result.append("\n");
                result.append("/**end instances*/\n");
                return null;
	}

	public Object visitPre(ASTInstance node){
	  result.append(this.indent(this.indentLevel));
          return null;
	}
	public Object visitPost(ASTInstance node){
	  result.append(";");
          return null;
	}





  public Object visitPre(ASTComponentPropertySet node){
    result.append("\ncomponent properties");
    this.incIndent();
    return null;
  }
  public Object visitPost(ASTComponentPropertySet node){
    this.decIndent();
    result.append("\n}");
    return null;
  }

  public Object visitPre(ASTConnectorPropertySet node){
    result.append("\nconnector properties");
    this.incIndent();
    return null;
  }
  public Object visitPost(ASTConnectorPropertySet node){
    this.decIndent();
    result.append("\n}");

    return null;
  }

  public Object visitPre(ASTPortPropertySet node){
    result.append("\nport properties{\n");
    this.incIndent();

    return null;
  }
  public Object visitPost(ASTPortPropertySet node){
    this.decIndent();
    result.append("\n}");
    return null;
  }


	public Object visitPre(ASTComment node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTComment node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTComments node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTComments node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTInterfaceBody node){
		result.append("{\n");
                this.incIndent();
		return null;
	}
	public Object visitPost(ASTInterfaceBody node){
		this.decIndent();
		result.append("\n");
		return null;
	}


  public Object visitPre(ASTInterface node){
    result.append("abstract component ");
    return null;
  }
  public Object visitPost(ASTInterface node){
    return null;
  }




	public Object visitPre(ASTLiteral node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTLiteral node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTNumericLiteral node){
		   result.append(node.getValue());
		return null;
	}
	public Object visitPost(ASTNumericLiteral node){
		return null;
	}

	public Object visitPre(ASTInvocationTypeReference node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTInvocationTypeReference node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTField node){
          result.append(indent(indentLevel));
		return null;
	}
	public Object visitPost(ASTField node){
		result.append(";");
		return null;
	}

	public Object visitPre(ASTHandlerTypeReference node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTHandlerTypeReference node){
		Assert.require(false);
		return null;
	}



        public Object visitPre(ASTPortAccess node){
          // nothing to do (is handled by designator)
		return null;
	}
	public Object visitPost(ASTPortAccess node){
		return null;
	}



/*	public Object visitPre(ASTContainer node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTContainer node){
		Assert.require(false);
		return null;
	}
*/

	public Object visitPre(ASTPortTypeList node){
		Assert.require(false);
		return null;
	}

	public Object visitIntraList(ASTPortTypeList node){
		Assert.require(false);
		return null;
	}


	public Object visitPost(ASTPortTypeList node){
		Assert.require(false);
		return null;
	}




	public Object visitPre(ASTPropertyContainer node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTPropertyContainer node){
		Assert.require(false);
		return null;
	}


  public Object visitPre(ASTIdentifierList node){
    return null;
  }

  public Object visitIntraList(ASTIdentifierList node){
    result.append(", ");
    return null;
  }


  public Object visitPost(ASTIdentifierList node){
    // nothing to do here
    return null;
  }

  public Object visitPre(ASTIdentifier node){
    ASTComments comments = node.getBeforeComment();
    if(comments != null)
      result.append(comments.toString());
    result.append(node.getID());
    return null;
  }
  public Object visitPost(ASTIdentifier node){
    ASTComments comments = node.getAfterComment();
    if(comments != null)
      result.append(comments.toString());
    return null;
  }


  public Object visitPre(ASTCharLiteral node){
    result.append(node.getValue());
    return null;
  }
  public Object visitPost(ASTCharLiteral node){
          return null;
  }


	public Object visitPre(ASTPortTypeReferenceList node){
		Assert.require(false);
		return null;
	}

	public Object visitIntraList(ASTPortTypeReferenceList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTPortTypeReferenceList node){
		Assert.require(false);
		return null;
	}




  public Object visitPre(ASTDataTypeIdentifier node){


    return null;
  }
  public Object visitPost(ASTDataTypeIdentifier node){
    result.append("\n");
          return null;
  }


  public Object visitPre(ASTConstExpression node){
    return null;
  }
  public Object visitPost(ASTConstExpression node){
    return null;
  }


  public Object visitPre(ASTBoolLiteral node){
    result.append(node.getValue());
    return null;
  }
  public Object visitPost(ASTBoolLiteral node){
    return null;
  }


	public Object visitPre(ASTConnector node){
		result.append(this.indent(indentLevel)+ "connector ");
		return null;
	}
	public Object visitPost(ASTConnector node){
		result.append(";");
		return null;
	}



	public Object visitPre(ASTConnectorBody node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTConnectorBody node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTFloatLiteral node){
	   result.append(node.getValue());
           return null;
	}
	public Object visitPost(ASTFloatLiteral node){
		return null;
	}




	public Object visitPre(ASTDesignator node){

		return null;
	}


	public Object visitIntraList(ASTDesignator node){
		result.append(".");
		return null;
	}

	public Object visitPost(ASTDesignator node){
		return null;
	}


	public Object visitPre(ASTConnectorList node){
          result.append("\n");
          result.append("/**connectors*/\n");

		return null;
	}

	public Object visitIntraList(ASTConnectorList node){
              result.append("\n");
		return null;
	}



	public Object visitPost(ASTConnectorList node){
          result.append("\n");
          result.append("/**end connectors*/\n");

		return null;
	}


	public Object visitPre(ASTPortAccessList node){
          result.append("(");
		return null;
	}

	public Object visitIntraList(ASTPortAccessList node){
              result.append(", ");
		return null;
	}



	public Object visitPost(ASTPortAccessList node){
          result.append(")");
		return null;
	}


	public Object visitPre(ASTRoot node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTRoot node){
		Assert.require(false);
		return null;
	}


  public Object visitPre(ASTStringLiteral node){
   result.append(node.getValue());
   return null;
  }
  public Object visitPost(ASTStringLiteral node){
          return null;
  }


	public Object visitPre(ASTConstExpressionList node){
		Assert.require(false);
		return null;
	}
	public Object visitIntraList(ASTConstExpressionList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTConstExpressionList node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTPortTypeReference node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTPortTypeReference node){
		Assert.require(false);
		return null;
	}





	public Object visitPre(ASTFieldList node){
          result.append("{\n");
          this.incIndent();
          return null;
	}

	public Object visitIntraList(ASTFieldList node){
          result.append("\n");
        	return null;
	}


	public Object visitPost(ASTFieldList node){
          result.append("\n}");
          this.decIndent();

		return null;
	}





	public Object visitPre(ASTCompilationUnit node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTCompilationUnit node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTIntLiteral node){
	   result.append(node.getValue());
           return null;
	}
	public Object visitPost(ASTIntLiteral node){
		return null;
	}



	public Object visitPre(ASTPort node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTPort node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTDataTypeReference node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTDataTypeReference node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTPortType node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTPortType node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTDataTypeList node){
          result.append("\n");
          result.append("/**data type list*/\n");
          return null;
	}

	public Object visitIntraList(ASTDataTypeList node){
          result.append("\n");
          return null;
	}



	public Object visitPost(ASTDataTypeList node){
          result.append("\n/** end of datatype list*/");
          result.append("\n");
          return null;
	}




	public Object visitPre(ASTGenericTypeIdentifierList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTGenericTypeIdentifierList node){
		Assert.require(false);
		return null;
	}



	public Object visitPre(ASTDataTypeIdentifierList node){
		Assert.require(false);
		return null;
	}
	public Object visitIntraList(ASTDataTypeIdentifierList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTDataTypeIdentifierList node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTEntititySpec node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTEntititySpec node){
		Assert.require(false);
		return null;
	}

  public Object visitPre(ASTRoleList node){
    result.append("\n");
    result.append("/**roles*/\n");
    return null;
  }
  public Object visitIntraList(ASTRoleList node){

    result.append("\n");
    return null;
  }

  public Object visitPost(ASTRoleList node){
    result.append("\n");
    result.append("/**end roles*/\n");

    return null;
  }

  public Object visitPre(ASTRole node){
    result.append(this.indent(indentLevel) + "role ");
    return null;
  }
  public Object visitPost(ASTRole node){
    result.append(";");
    return null;
  }

	public Object visitPre(ASTDataType node){
		result.append("\ntype ");
		return null;
	}
	public Object visitPost(ASTDataType node){
		result.append(";\n");
		return null;
	}


	public Object visitPre(ASTGenericTypeParameter node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTGenericTypeParameter node){
		result.append("{");
		return null;
	}

	public Object visitPre(ASTMappingLiteral node){
          result.append("{");
		return null;
	}
	public Object visitIntraList(ASTMappingLiteral node){
		result.append(", ");
		return null;
	}



	public Object visitPost(ASTMappingLiteral node){
		result.append("}");
		return null;
	}

	public Object visitPre(ASTMapping node){
		result.append("(");
		return null;
	}

	public Object visitPost(ASTMapping node){
		result.append(")");
		return null;
	}

	public Object visitPre(ASTExternalTypeName node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTExternalTypeName node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTGenericTypeParameterList node){
		Assert.require(false);
		return null;
	}

	public Object visitIntraList(ASTGenericTypeParameterList node){
		Assert.require(false);
		return null;
	}


	public Object visitPost(ASTGenericTypeParameterList node){
		Assert.require(false);
		return null;
	}

  public Object visitPre(ASTPropertyIdentifier node){
    result.append(node.getID());
    return null;
  }
  public Object visitPost(ASTPropertyIdentifier node){
    result.append(" = ");
    return null;
  }
  public Object visitPre(ASTComponentTypeIdentifier node){
    result.append(node.getID() + " ");
    return null;
  }
  public Object visitPost(ASTComponentTypeIdentifier node){
    return null;
  }

  public Object visitPre(ASTRoleIdentifier node){
    result.append(" is " + node.getID() + " ");
    return null;
  }
  public Object visitPost(ASTRoleIdentifier node){
    return null;

  }




  public Object visitPre(ASTUserTypeIdentifier node){
    if(node.isExternType())
      result.append("extern ");
    return null;
  }
  public Object visitPost(ASTUserTypeIdentifier node){
    if(node.isArrayType()){
      // take care of array types
      Vector dims= node.getDimensions();
      Enumeration enum = dims.elements();
      while(enum.hasMoreElements()){
        Integer dim = (Integer) enum.nextElement();
        result.append("["+dim.toString()+"]");
      }
    }
    result.append(" ");

    return null;
  }

  public Object visitPre(ASTBaseTypeIdentifier node){

    return null;
  }
  public Object visitPost(ASTBaseTypeIdentifier node){

    if(node.isArrayType()){
      // take care of array types
      Vector dims= node.getDimensions();
      Enumeration enum = dims.elements();
      while(enum.hasMoreElements()){
        Integer dim = (Integer) enum.nextElement();
        result.append("["+dim.toString()+"]");
      }
    }
    result.append(" ");

    return null;
  }
    public Object visitPre(ASTBaseTypeReference node){
    result.append(" extends ");
    return null;

  }

  public Object visitPost(ASTBaseTypeReference node){
    return null;
  }

}