package coco.codegen.cpp;
import java.io.OutputStream;
import coco.utils.*;
import coco.parser.ast.*;
import coco.parser.ast.enums.PortEnumeration;
import coco.codegen.AbstractCodeGenerator;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author
 * @version 1.0
 */



public class CppGenerator extends AbstractASTVisitor implements AbstractCodeGenerator{




  private StringBuffer neededDeclarations = null;
  private Hashtable neededDeclarationsList = null;

  private void addNeededDeclaration(String componentName, String type){
    Assert.require(neededDeclarations);
    Assert.require(componentName);
    Assert.require(type);
    Assert.require(neededDeclarationsList);
    if(neededDeclarationsList.containsKey(componentName))
      return;

    neededDeclarationsList.put(componentName, componentName);

    neededDeclarations.append("\n/* including declarations for "+ type + "  " + componentName + " */");
    neededDeclarations.append("\n#include \""+componentName+".h\"");
  }


  private String createExecuteMethod(ASTPortList portList){
    Assert.require(portList);
    StringBuffer tmp = new StringBuffer();
    tmp.append(this.indent(indentLevel)+ "public void* execute(void* data){\n");
    this.incIndent();
    Enumeration enum = portList.getNodes();
    while(enum.hasMoreElements()){
      ASTPort elem = (ASTPort) enum.nextElement();
      if(elem instanceof ASTData){
        ASTData data = (ASTData) elem;
        Assert.require(data);
        ASTIdentifier id = data.getPortId();
        Assert.require(id);
        String name = id.getID();
        Assert.require(name);
        ASTDataTypeIdentifier ti = data.getDataType();
        Assert.require(ti);
        ASTIdentifier tId = ti.getTypeID();
        Assert.require(tId);
        String tName = tId.getID();
        Assert.require(tName);

        if((data.getDirection() == data.IN_DIR) || (data.getDirection() == data.INOUT_DIR)){
          // fetch the value from the OM
          tmp.append(this.indent(indentLevel)+ name);
          tmp.append(" = (" + tName +") om.fetch(IDX_");
          tmp.append(name+ ");");
        }
        if((data.getDirection() == data.OUT_DIR) || (data.getDirection() == data.INOUT_DIR)){
          tmp.append("\n");
          tmp.append(this.indent(indentLevel)+ "om.store(IDX_");
          tmp.append(name+ ", ");
          tmp.append(name+ ");\n");
        }
      }
    }
    this.decIndent();
    tmp.append(this.indent(indentLevel)+ "}\n");
    return tmp.toString();
  }


  private StringBuffer result = null;
  static int indentLevel = 0;
  static final char indentChar='\t';
  boolean dealtWithByParent = false;

  private void incIndent(){
    indentLevel++;
  }

  private void decIndent(){
    indentLevel--;
    if(indentLevel < 0)
      indentLevel = 0;
  }


  private String indent(int level){
    Assert.require(level>=0);
    char[] res = new char[level];
    for(int i=0;i<level;i++)
      res[i] = indentChar;
    return new String(res);
  }

  public boolean preVisit(){
          return true;
  }
  public boolean postVisit(){
          return true;
  }

  public CppGenerator(){
    init();
  }


  public void init(){
    result = new StringBuffer();
    neededDeclarations = new StringBuffer();
    neededDeclarationsList = new Hashtable(10);
  }

  public String getResult(){
    return result.toString();
  }


  public Object visitPre(ASTComponentList node){
          return null;
  }

  public Object visitIntraList(ASTComponentList node){
    return null;
  }


  public Object visitPost(ASTComponentList node){
          return null;
  }


  public Object visitPre(ASTComponent node){
    String name = node.getName();
    String description = null;
    Assert.require(name);

    this.addNeededDeclaration("ObjectManager", "infrastructure");


    ASTComponentBody body = (ASTComponentBody)node.getBody();

    result.append("/*********************************************************************/\n");
    result.append("/* This code has been generated by the PECOS composition environment */\n");
    result.append("/*********************************************************************/\n");
    result.append("/* Name of the component: " + name +"\n");
    result.append("   Description (if provided):\n");
    if(description != null)
      result.append(description + "\n");
    else
      result.append("(no description available)\n");
    result.append("*/\n");
    if(node.isActive()){
      result.append("__ACTIVE__ ");
      this.addNeededDeclaration("Thread", "infrastructure");
    }
    result.append("class ");
    return null;
  }
  public Object visitPost(ASTComponent node){
    String name = node.getName();
    Assert.require(name);
    addNeededDeclaration(name, node.getUnitType());



    result.append(";\n");
    // add the includes for all needed declarations
    StringBuffer tmp = result;
    result = neededDeclarations;
    result.append("\n" + tmp.toString());

    return null;
  }



  public Object visitPre(ASTInterfaceIdentifierList node){
    result.append(": ");
    return null;
  }

  public Object visitIntraList(ASTInterfaceIdentifierList node){
      result.append(", ");
      return null;
  }


  public Object visitPost(ASTInterfaceIdentifierList node){
    return null;
  }

  public Object visitPre(ASTInterfaceIdentifier node){
    result.append("public " + node.getID());
    return null;
  }

  public Object visitPost(ASTInterfaceIdentifier node){
    return null;
  }


  public Object visitPre(ASTComponentBody node){
    incIndent();
    result.append("{\n");
    result.append(this.indent(indentLevel)+ "private ObjectManager &om = ObjectManager.getInstance();\n");
    return null;
  }
  public Object visitPost(ASTComponentBody node){
    decIndent();
    result.append("\n}");
    return null;
  }




  public Object visitPre(ASTPortList node){

    result.append(indent(indentLevel) + "/***************************/\n");
    result.append(indent(indentLevel) + "/* ports of this component */\n");
    result.append(indent(indentLevel) + "/***************************/\n");
    return null;
  }

  public Object visitIntraList(ASTPortList node){
          return null;
  }


  public Object visitPost(ASTPortList node){
    result.append("\n");
    result.append(createExecuteMethod(node)+"\n");
    result.append(indent(indentLevel) + "/********************/\n");
    result.append(indent(indentLevel) + "/* end of port list */\n");
    result.append(indent(indentLevel) + "/********************/\n");
    return null;
  }



  public Object visitPre(ASTHandler node){
    // so at this time we don't have to do anything, because visit*(ASTInvocationSignature) will
    // take care of it
    result.append("\n" + indent(indentLevel));
    return null;
  }
  public Object visitPost(ASTHandler node){
    if(node.getBody() == null){
      result.append("{\n"+ indent(indentLevel) + "}");
    }
    result.append(";\n");
    return null;
  }


  public Object visitPre(ASTData node){

    result.append("\n" +indent(indentLevel));
    result.append("/*actual data port rep., not seen from outside*/");


    // now start with the accessor method
    result.append("\n" +indent(indentLevel));
    result.append("private ");
    return null;
  }


  public Object visitPost(ASTData node){

    result.append("\n" +indent(indentLevel));
    result.append("/* accessor methods for this data port follow */");


    ASTDataTypeIdentifier ti = node.getDataType();
    Assert.require(ti);
    ASTIdentifier typeId = ti.getTypeID();
    Assert.require(typeId);
    ASTIdentifier portId = node.getPortId();
    Assert.require(portId);


    if((node.getDirection() == node.IN_DIR) || (node.getDirection() == node.INOUT_DIR)){
      // generate the setter method
      result.append("\n" +indent(indentLevel));
      result.append("public void setPort_");
      result.append(portId.getID() + "(");
      result.append(typeId.getID() + " value){\n");
      this.incIndent();
      result.append(indent(indentLevel) + portId.getID() + " = value;\n");
      this.decIndent();
      result.append(indent(indentLevel) + "}");
    }

    if((node.getDirection() == node.OUT_DIR) || (node.getDirection() == node.INOUT_DIR)){
    // generate the getter method
      result.append("\n" +indent(indentLevel));
      result.append("public " + typeId.getID() + " getPort_");
      result.append(portId.getID() + "(){\n");
      this.incIndent();
      result.append(indent(indentLevel) + "return " + portId.getID() + ";\n");
      this.decIndent();
      result.append(indent(indentLevel) + "}");

    }



    result.append(";\n");
    return null;

  }


  public Object visitPre(ASTPortModifiers node){
    // only public/private modifiers can be mapped into C++
/*    if((node.getFirst() == ASTPortModifiers.PUBLIC)||(node.getFirst() == ASTPortModifiers.PRIVATE))
      result.append(node.getFirstModifierString()+ " ");
    else if((node.getSecond() == ASTPortModifiers.PUBLIC)||(node.getSecond() == ASTPortModifiers.PRIVATE))
      result.append(node.getFirstModifierString()+ " ");
*/

    return null;
  }

  public Object visitPost(ASTPortModifiers node){
    // nothing to be done here
    return null;
  }



  public Object visitPre(ASTInvocationSignature node){
    result.append("void ");
    return null;
  }
  public Object visitPost(ASTInvocationSignature node){

    if(node.getInvocationParams() == null)
      result.append("()");
    return null;
  }



  public Object visitPre(ASTInvocationParameterList node){
    result.append("(");
    return null;
  }

  public Object visitIntraList(ASTInvocationParameterList node){
    result.append(", ");
    return null;
  }


  public Object visitPost(ASTInvocationParameterList node){
    result.append(")");
    return null;
  }


  public Object visitPre(ASTDataTypeIdentifier node){
    result.append(" ");
    return null;
  }
  public Object visitPost(ASTDataTypeIdentifier node){
    result.append(" ");
    return null;
  }





  public Object visitPre(ASTPortBody node){
    Node n = node.getParent();
    if((n != null) && (n instanceof ASTData))
      // if parent is of type ASTData, the port body is dealt with separately
      return null;

    //otherwise take care of the port body contents;
    result.append("{\n");
    incIndent();
    return null;
  }
  public Object visitPost(ASTPortBody node){

    Node n = node.getParent();
    if((n != null) && (n instanceof ASTData))
      // if parent is of type ASTData, the port body is dealt with separately
      return null;

    decIndent();
    result.append("\n" + indent(indentLevel) + "}");
    return null;
  }


  public Object visitPre(ASTPropertyList node){
    node.getParent();
    result.append("\n");
    result.append(indent(indentLevel) + "/********************************/\n");
    result.append(indent(indentLevel) + "/* properties of this component */\n");
    result.append(indent(indentLevel) + "/********************************/");
    return null;
  }
  public Object visitIntraList(ASTPropertyList node){

    return null;
  }



  public Object visitPost(ASTPropertyList node){
    result.append("\n");
    result.append(indent(indentLevel) + "/*********************/\n");
    result.append(indent(indentLevel) + "/* end of properties */\n");
    result.append(indent(indentLevel) + "/*********************/\n");

    return null;
  }



  public Object visitPre(ASTProperty node){
    result.append("\n" + indent(indentLevel)+ "char* ");
    return null;
  }
  public Object visitPost(ASTProperty node){
    result.append(";");
    return null;
  }


  public Object visitPre(ASTInstanceList node){
    result.append("\n");
    result.append(indent(indentLevel) + "/***********************************************/\n");
    result.append(indent(indentLevel) + "/* subcomponents (instances) of this component */\n");
    result.append(indent(indentLevel) + "/***********************************************/\n");
    return null;
  }

  public Object visitIntraList(ASTInstanceList node){
    return null;
  }



  public Object visitPost(ASTInstanceList node){
    result.append("\n");
    result.append(indent(indentLevel) + "/********************************************/\n");
    result.append(indent(indentLevel) + "/* end of instances list for this component */\n");
    result.append(indent(indentLevel) + "/********************************************/\n");
    return null;
  }


  public Object visitPre(ASTInstance node){
    result.append(indent(indentLevel) + " private ");
    return null;
  }
  public Object visitPost(ASTInstance node){
    result.append(";\n");
    ASTIdentifier id = node.getTypeId();
    Assert.require(id);
    String compId = id.getID();
    this.addNeededDeclaration(compId, "component");
    return null;
  }


  public Object visitPre(ASTRoleList node){
    return null;
  }
  public Object visitIntraList(ASTRoleList node){
    return null;
  }

  public Object visitPost(ASTRoleList node){
    return null;
  }

  public Object visitPre(ASTRole node){
    return null;
  }
  public Object visitPost(ASTRole node){
    return null;
  }





// data types
	public Object visitPre(ASTDataTypeList node){
		return null;
	}

	public Object visitIntraList(ASTDataTypeList node){
		return null;
	}



	public Object visitPost(ASTDataTypeList node){
		return null;
	}


  public Object visitPre(ASTDataType node){
    result.append("typedef struct t_");
    return null;
  }
  public Object visitPost(ASTDataType node){
    result.append(" " + node.getDataTypeIdentifier().getID()+ ";\n");
          return null;
  }
  public Object visitPre(ASTBaseTypeReference node){
    result.append(" : public ");
    return null;

  }

  public Object visitPost(ASTBaseTypeReference node){
    return null;
  }



    public Object visitPre(ASTFieldList node){
      result.append("{");
      this.incIndent();
      return null;
    }

    public Object visitIntraList(ASTFieldList node){
      return null;
    }


    public Object visitPost(ASTFieldList node){
      this.decIndent();
      result.append("\n" + this.indent(indentLevel) + "}");
      return null;
    }




  public Object visitPre(ASTField node){
    result.append("\n" + this.indent(indentLevel));
    return null;
  }
  public Object visitPost(ASTField node){
    ASTDataTypeIdentifier ti = node.getTypeId();
    Assert.require(ti);
    if(ti.isArrayType()){
      Vector dimensions = ti.getDimensions();
      Enumeration enum = dimensions.elements();
      while(enum.hasMoreElements()){
        Integer i = (Integer) enum.nextElement();
        result.append("[" + i.toString() + "]");
      }
    }

    result.append(";");
    return null;
  }





	public Object visitPre(ASTComment node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTComment node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTComments node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTComments node){
		Assert.require(false);
		return null;
	}





  public Object visitPre(ASTInterface node){
    String name = node.getName();
    String description = null;
    Assert.require(name);

    ASTInterfaceBody body = (ASTInterfaceBody)node.getBody();

    result.append("/*********************************************************************/\n");
    result.append("/* This code has been generated by the PECOS composition environment */\n");
    result.append("/*********************************************************************/\n");
    result.append("/* Name of the abstract component: " + name +"\n");
    result.append("   Description (if provided):\n");
    if(description != null)
      result.append(description + "\n");
    else
      result.append("(no description available)\n");
    result.append("*/\n");
    result.append("/*abstract*/ class ");
    return null;
  }

  public Object visitPost(ASTInterface node){
    result.append(";\n");
    // add the includes for all needed declarations
    StringBuffer tmp = result;
    result = neededDeclarations;
    result.append("\n" + tmp.toString());

    return null;
  }




  public Object visitPre(ASTInterfaceBody node){
    result.append("{\n");
    return null;
  }
  public Object visitPost(ASTInterfaceBody node){
    result.append("\n}");
    return null;
  }








  public Object visitPre(ASTComponentPropertySet node){
    Assert.require(null);
    return null;
  }
  public Object visitPost(ASTComponentPropertySet node){
    Assert.require(null);
    return null;
  }

  public Object visitPre(ASTConnectorPropertySet node){
    Assert.require(null);
    return null;
  }
  public Object visitPost(ASTConnectorPropertySet node){
    Assert.require(null);
    return null;
  }

  public Object visitPre(ASTPortPropertySet node){
    Assert.require(null);
    return null;
  }
  public Object visitPost(ASTPortPropertySet node){
    Assert.require(null);
    return null;
  }








	public Object visitPre(ASTInvocationTypeReference node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTInvocationTypeReference node){
		Assert.require(false);
		return null;
	}



	public Object visitPre(ASTHandlerTypeReference node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTHandlerTypeReference node){
		Assert.require(false);
		return null;
	}



        public Object visitPre(ASTPortAccess node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTPortAccess node){
		Assert.require(false);
		return null;
	}



	public Object visitPre(ASTPortTypeList node){
		Assert.require(false);
		return null;
	}

	public Object visitIntraList(ASTPortTypeList node){
		Assert.require(false);
		return null;
	}


	public Object visitPost(ASTPortTypeList node){
		Assert.require(false);
		return null;
	}








	public Object visitPre(ASTPropertyContainer node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTPropertyContainer node){
		Assert.require(false);
		return null;
	}





	public Object visitPre(ASTPortTypeReferenceList node){
		Assert.require(false);
		return null;
	}

	public Object visitIntraList(ASTPortTypeReferenceList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTPortTypeReferenceList node){
		Assert.require(false);
		return null;
	}




	public Object visitPre(ASTConnector node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTConnector node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTConnectorBody node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTConnectorBody node){
		Assert.require(false);
		return null;
	}





	public Object visitPre(ASTDesignator node){
		Assert.require(false);
		return null;
	}


	public Object visitIntraList(ASTDesignator node){
		Assert.require(false);
		return null;
	}

	public Object visitPost(ASTDesignator node){
		Assert.require(false);
		return null;
	}




	public Object visitPre(ASTConnectorList node){
		Assert.require(false);
		return null;
	}

	public Object visitIntraList(ASTConnectorList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTConnectorList node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTPortAccessList node){
		Assert.require(false);
		return null;
	}

	public Object visitIntraList(ASTPortAccessList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTPortAccessList node){
		Assert.require(false);
		return null;
	}



	public Object visitPre(ASTConstExpressionList node){
		Assert.require(false);
		return null;
	}
	public Object visitIntraList(ASTConstExpressionList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTConstExpressionList node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTPortTypeReference node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTPortTypeReference node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTInvocationParameter node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTInvocationParameter node){
		Assert.require(false);
		return null;
	}







	public Object visitPre(ASTCompilationUnit node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTCompilationUnit node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTPort node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTPort node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTDataTypeReference node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTDataTypeReference node){
		Assert.require(false);
		return null;
	}





	public Object visitPre(ASTPortType node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTPortType node){
		Assert.require(false);
		return null;
	}





	public Object visitPre(ASTGenericTypeIdentifierList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTGenericTypeIdentifierList node){
		Assert.require(false);
		return null;
	}



	public Object visitPre(ASTDataTypeIdentifierList node){
		Assert.require(false);
		return null;
	}
	public Object visitIntraList(ASTDataTypeIdentifierList node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTDataTypeIdentifierList node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTEntititySpec node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTEntititySpec node){
		Assert.require(false);
		return null;
	}



	public Object visitPre(ASTGenericTypeParameter node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTGenericTypeParameter node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTMappingLiteral node){
		Assert.require(false);
		return null;
	}
	public Object visitIntraList(ASTMappingLiteral node){
		Assert.require(false);
		return null;
	}



	public Object visitPost(ASTMappingLiteral node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTMapping node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTMapping node){
		Assert.require(false);
		return null;
	}


	public Object visitPre(ASTExternalTypeName node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTExternalTypeName node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTRoot node){
		Assert.require(false);
		return null;
	}
	public Object visitPost(ASTRoot node){
		Assert.require(false);
		return null;
	}

	public Object visitPre(ASTGenericTypeParameterList node){
		Assert.require(false);
		return null;
	}

	public Object visitIntraList(ASTGenericTypeParameterList node){
		Assert.require(false);
		return null;
	}


	public Object visitPost(ASTGenericTypeParameterList node){
		Assert.require(false);
		return null;
	}



  public Object visitPre(ASTConstExpression node){
    return null;
  }
  public Object visitPost(ASTConstExpression node){
          return null;
  }


  // terminal symbols

  public Object visitPre(ASTCharLiteral node){
    result.append("\"" +  node.getValue()+ "\"");
    return null;
  }
  public Object visitPost(ASTCharLiteral node){
    return null;
  }

  public Object visitPre(ASTBoolLiteral node){
    result.append("\"" +  node.getValue()+ "\"");
    return null;
  }
  public Object visitPost(ASTBoolLiteral node){
    return null;
  }

  public Object visitPre(ASTFloatLiteral node){
    result.append("\"" +  node.getValue()+ "\"");
    return null;
  }
  public Object visitPost(ASTFloatLiteral node){
    return null;
  }



  public Object visitPre(ASTIntLiteral node){
    result.append("\"" +  node.getValue()+ "\"");
    return null;
  }
  public Object visitPost(ASTIntLiteral node){
    return null;
  }

  public Object visitPre(ASTNumericLiteral node){
    result.append("\"" +  node.getValue()+ "\"");
    return null;
  }
  public Object visitPost(ASTNumericLiteral node){
    return null;
  }

  public Object visitPre(ASTStringLiteral node){
    result.append(node.getValue());
    return null;
  }
  public Object visitPost(ASTStringLiteral node){
    return null;
  }


  public Object visitPre(ASTIdentifierList node){
          return null;
  }

  public Object visitIntraList(ASTIdentifierList node){
          result.append(", ");
          return null;
  }

  public Object visitPre(ASTComponentTypeIdentifier node){
    result.append(node.getID() + " ");
    return null;
  }
  public Object visitPost(ASTComponentTypeIdentifier node){
    return null;
  }

  public Object visitPre(ASTRoleIdentifier node){
    result.append(node.getID() + " ");
    return null;
  }
  public Object visitPost(ASTRoleIdentifier node){
    return null;

  }



  public Object visitPre(ASTUserTypeIdentifier node){
    // the repective header file for this user data type has to be included
    ASTIdentifier id = node.getTypeID();
    Assert.require(id);
    this.addNeededDeclaration(id.toString(), "data type");
    return null;
  }
  public Object visitPost(ASTUserTypeIdentifier node){

    result.append(" ");
    return null;
  }

  public Object visitPre(ASTBaseTypeIdentifier node){
    return null;
  }
  public Object visitPost(ASTBaseTypeIdentifier node){
    result.append(" ");
    return null;
  }




  public Object visitPre(ASTPropertyIdentifier node){
    result.append(node.getID() + " = " );
    return null;
  }
  public Object visitPost(ASTPropertyIdentifier node){
    return null;
  }

  public Object visitPost(ASTIdentifierList node){
          return null;
  }



  public Object visitPre(ASTIdentifier node){
    result.append(node.getID());
    return null;
  }
  public Object visitPost(ASTIdentifier node){
    return null;
  }



}