package coco.config;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Properties;
import java.util.Enumeration;
import java.io.FileInputStream;
import java.util.Vector;
import coco.utils.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class GlobalConfig {

  static public final String OPTION_BASEDIR = "system.basedir";
  static public final String OPTION_LASTDIR = "system.lastdir";
  static public final String OPTION_BROWSER_PROG = "system.extern.browser";
  static public final String OPTION_RECENT_FILES = "system.recent-files";
  static public final String OPTION_STORE_BASEDIR = "coco.store-basedir";
  static public final String OPTION_COCO_REPOSITORY = "coco.repository";
  static public final String OPTION_GENERATED_JAVACODE_DIR = "coco.codegen-java.outdir";
  static public final String OPTION_COCO_REPOSITORY_SAVE = "coco.repository.tempsave";
  static public final String OPTION_EDITOR_TABSIZE = "editor.tabsize";
  static public final String OPTION_PROJECT_LAST = "project.last";

  static private GlobalConfig instance = null;
  private String propFile = null;
  private Properties props = new Properties();

  private GlobalConfig() {
  }



  private void createStandardProps(){
   props.setProperty(OPTION_BASEDIR, "");
   props.setProperty(OPTION_BROWSER_PROG, "");
   props.setProperty(OPTION_RECENT_FILES, "3");
   props.setProperty(OPTION_STORE_BASEDIR, "");
   props.setProperty(OPTION_COCO_REPOSITORY, "");
   props.setProperty(OPTION_LASTDIR, "");
   props.setProperty(OPTION_GENERATED_JAVACODE_DIR,"");
   props.setProperty(OPTION_COCO_REPOSITORY_SAVE,"");
   props.setProperty(OPTION_EDITOR_TABSIZE,"2");
   props.setProperty(OPTION_PROJECT_LAST, "");
  }


  private String checkForConfigFile() throws java.io.IOException{
    // first we search in the users home directory
    String res = createPropertyFileName(System.getProperty("user.home"));
    File f = new File(res);
    // if the file exists, return the name
    if(f.isFile() && f.canRead())
      return res;

    // otherwise search or create it in the current directory
    res = createPropertyFileName(System.getProperty("user.dir"));
    f = new File(res);
    // if the file still doesn't exist, create a new one
    if(!f.isFile()){
      f.createNewFile();
    }
    return res;
  }


  private String createPropertyFileName(String basePath){
    Assert.require(basePath);
    String path = basePath;
    String pathDelim = System.getProperty("file.separator");
    Assert.require(pathDelim);
    path += pathDelim + "coco.ini";
    return path;
  }

  public String getProperty(String key){
    Assert.require(key);
    return props.getProperty(key);
  }

  public void setProperty(String key, String value){
    Assert.require(key);
    Assert.require(value);
    props.setProperty(key, value);
  }

  public void setProperties(Properties p){
    Assert.require(p);
    props = p;
  }
  public Properties getProperties(){
   return props;
  }
  static public GlobalConfig getInstance(){
    if(instance == null)
      instance = new GlobalConfig();
    return instance;
  }

  public String getConfigFilename(){
    return propFile;
  }

  public void readConfig()
    throws java.io.IOException, java.lang.SecurityException{
    // search file in current directory
    readConfig(checkForConfigFile());
   props.setProperty(OPTION_LASTDIR, "");
  }

  public void readConfig(String path)
    throws java.io.IOException, java.lang.SecurityException{
      // there must be a file name and its length must not be zero
      Assert.require(path);
      Assert.require(path.length());
      try{
        FileInputStream reader = new FileInputStream(path);
        props = new Properties();
        props.load(reader);
        System.out.println("Read from file " + path);
      }catch(java.io.IOException e){
        // something went wrong -> dismiss the properties
        props = new Properties();
        propFile = null;
        // re-throw the exception for further processing
        throw e;
      }
      propFile = path;
      if(props.isEmpty()){
        // create at least the standard entries
        createStandardProps();
      }
  }

  public void writeConfig()
    throws java.io.IOException, java.lang.SecurityException{
    if(propFile == null){


      // generate the default name
      propFile = checkForConfigFile();
    }
    writeConfig(propFile);
  }
  public void writeConfig(String path)
    throws java.io.IOException, java.lang.SecurityException{
      // there must be a file name and its length must not be zero
      Assert.require(path);
      Assert.require(path.length());

      FileOutputStream writer = new FileOutputStream(path);
      props.store(writer,"PECOS Composition Environment properties");
      System.out.println("Wrote to file " + path);
      props.list(System.out);
      propFile = path;
  }

}
