package coco.filehandling;

import java.io.*;
import coco.utils.*;

/**
 * Title:        PECOS Composition Environment 2
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      FZI
 * @author
 * @version 1.0
 */

 /**
  * This class represents an entitiy file belonging to a CoCoProject.
  * It is used to keep track of the original file and the modified text
  * of it.
  */
public class CoCoFile {

  /**
   * Name of the File without Path, but with suffix.
   */
  private String fileName = null;

  /**
   * The originial Text of the file, as read, when the CoCoFile was
   * created.
   */
  private String originalText = null;

  /**
   * Modified Text of the File, if any. Normally set before Save by
   * the controller through the project. So saving can be done by the
   * project and CoCoFile classes.
   */
  private String modifiedText = null;

  /**
   * True, if modified Text was set, which has not yet been saved.
   */
  private boolean isModified = false;

  private String FS = System.getProperty("file.separator");

  // ------------------------------------------------------ CONSTRUCTOR

  /**
   * Create new CocoFile.
   */
  public CoCoFile(String fileName, String projectPath, String projectName) {
    //System.out.println("Creating new CoCoFile: "+fileName);
    this.fileName = fileName;
    loadFile(projectPath, projectName);
  }

  public CoCoFile(String fileName) {
    //System.out.println("Creating new CoCoFile: "+fileName);
    this.fileName = fileName;
    this.originalText = "";
    this.modifiedText = "";
  }

  // ------------------------------------------------------ PUBLIC

  public void setModifiedText() {
    this.isModified = true;
  }

  public String getActualText() {
    if (!this.isModified()) {
      return this.originalText;
    } else {
      return this.modifiedText;
    }
  }

  /**
   * Get the kind of coco file :
   *
   *   cm = component
   *   acm = abstract component
   *   rl = rule
   */
  public String getTypeOf() {
    return this.fileName.substring(this.fileName.lastIndexOf('.')+1, this.fileName.length());
  }

  /**
   * get the FileName
   */
  public String getFilename() {
    return fileName;
  }

  /**
   * set a modified Text
   */
   public void setModifiedText(String text) {
    this.modifiedText = text;
    this.isModified = true;
  }

  /**
   * get the original text
   */
   public String getOriginalText() {
    Assert.require(this.originalText);
    return this.originalText;
   }

  /**
   * returns true, if modified, not saved text is present
   */
  public boolean isModified() {
    return this.isModified;
  }

  /**
   *  save the coco file
   */
   public void save(String projectPath, String projectName) {
    File cFile = new File(projectPath+projectName+FS+this.fileName);
    try {
      FileOutputStream out = new FileOutputStream(cFile);

      byte[] data;
      // switch the buffers -> what has been modified is now the original text
      // if necessary
      if (this.isModified)
        this.originalText = this.modifiedText;

      data = this.originalText.getBytes();
      out.write(data);
      out.close();
      this.isModified = false;


    } catch (Exception ex) {
      System.out.println("Error: during saving of: " + this.getFilename());
    }
   }

  // ------------------------------------------------------ PRIVATE

  private void loadFile(String projectPath, String projectName) {
    //System.out.println("Loading CoCoFile: "+projectPath+projectName+FS+fileName);
    File file = new File(projectPath+FS+projectName+FS+fileName);
    try {
      if (file != null) {
        if (file.exists()) {
          if(file.canRead()) {
          StringBuffer buf = new StringBuffer();
          // read file
          BufferedReader in = new BufferedReader(new FileReader(file));
          String line;
          while ((line = in.readLine()) != null) {
            buf.append(line);
            buf.append("\n");
          }
          in.close();
          // file content
          this.originalText = buf.toString();
        }}
      }
    } catch (Exception ex) {
      System.err.println("Fehler beim ffnen !");
    }
  }
}