package coco.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;

import java.io.*;

import coco.utils.TextAreaOutputStream;

import coco.utils.Assert;

/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author T. Genssler (FZI)
 * @version 1.0
 */

public class CombinedTextOutputPane extends JTabbedPane {


  private PrintStream stdout = null;
  private PrintStream stderr = null;

  CardLayout cardLayout = new CardLayout();
  JTextArea stderrText = new JTextArea();
  JScrollPane stdoutScrollPane = new JScrollPane();
  JTextArea stdoutText = new JTextArea();
  JScrollPane stderrScrollPane = new JScrollPane();


  public InternalPopupMenu popup = new InternalPopupMenu();


class MyDocumentListener implements DocumentListener{
    private JTabbedPane paneToSwitch = null;
    private Component componentInFront = null;

    public MyDocumentListener(JTabbedPane paneToSwitch, Component inFront){
     Assert.require(paneToSwitch);
     Assert.require(inFront);
     this.paneToSwitch = paneToSwitch;
     this.componentInFront = inFront;
    }
    public void changedUpdate(DocumentEvent e){
      getInFront();
    }

    public void insertUpdate(DocumentEvent e){
      getInFront();
    }

    public void removeUpdate(DocumentEvent e){
      getInFront();
    }
    void getInFront(){
      // bring the attached component in front
      paneToSwitch.setSelectedComponent(this.componentInFront);
    }
 }




   public class InternalPopupMenu extends JPopupMenu{

    JMenuItem jMenuItemClearWindow= new JMenuItem("Clear output");
    JMenuItem jMenuItemSaveToFile= new JMenuItem("Save to file...");
    private JTextArea currentWindow = null;
    public InternalPopupMenu(){
      super();
      this.add(jMenuItemClearWindow);
      this.addSeparator();
      this.add(jMenuItemSaveToFile);

      jMenuItemClearWindow.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  clearWindow();
              }
          });
      jMenuItemSaveToFile.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  saveWindowToFile();
              }
          });
      jMenuItemSaveToFile.disable();
    }

    public void setTextArea(JTextArea ta){
     Assert.require(ta);
     currentWindow = ta;
    }
    private void clearWindow(){
     Assert.require(currentWindow);
     currentWindow.setText("");
    }

    private void saveWindowToFile(){
     Assert.require(currentWindow);
     JFileChooser fileDlg = new JFileChooser();
     //fileDlg.setFileFilter(filter);
     // no file selected.
     if( JFileChooser.APPROVE_OPTION!=fileDlg.showOpenDialog(null)){
	System.out.println("No file selected");
	return;
     }
     File f = fileDlg.getSelectedFile();

     if(!f.canWrite())
      System.err.println("Can't write to file " + f.getAbsolutePath());
    try{
      PrintStream os = new PrintStream(new FileOutputStream(f));
      os.println(currentWindow.getText());
      clearWindow();
    }catch(FileNotFoundException e){
      System.err.println("Can't write to file " + f.getAbsolutePath());
    }
  }

}




  public CombinedTextOutputPane() {
	super();


    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }


  private void jbInit() throws Exception {

    //re-direct stderr and stdout
    redirect();

    stderrText.setForeground(Color.red);
    stderrText.setBackground(Color.lightGray);
    stderrText.setEditable(false);
    stderrText.addMouseListener(new java.awt.event.MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        stderrText_mouseClicked(e);
      }
    });
//    stdoutText.setForeground(SystemColor.textHighlight);
    stdoutText.setEditable(false);
    stdoutText.addMouseListener(new java.awt.event.MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        stdoutText_mouseClicked(e);
      }
    });
    setTabPlacement(JTabbedPane.BOTTOM);
/*    setMinimumSize(new Dimension(400, 400));
    setPreferredSize(new Dimension(400, 400));
*/

    add(stdoutScrollPane, "Standard out");
    add(stderrScrollPane, "Standard error");
    stderrScrollPane.getViewport().add(stderrText, null);
    stdoutScrollPane.getViewport().add(stdoutText, null);

    // make sure the last updated log is always in front
    stdoutText.getDocument().addDocumentListener(
      new MyDocumentListener(this, stdoutScrollPane));

    stderrText.getDocument().addDocumentListener(
      new MyDocumentListener(this, stderrScrollPane));

//    this.pack();


   }

  private void redirect(){

    stdout = System.out;
    stderr = System.err;

    System.setOut(new TextAreaOutputStream(stdoutText, stdout, true));
//    System.out.println("Standard out has been re-directed");
    System.setErr(new TextAreaOutputStream(stderrText, stderr, true));
//    System.err.println("Standard error has been re-directed");
  }

  void stdoutText_mouseClicked(MouseEvent e) {
    if(e.getModifiers() == e.META_MASK){
        popup.setTextArea(stdoutText);
        popup.show(this.stdoutText, e.getX(), e.getY());
    }

  }

  void stderrText_mouseClicked(MouseEvent e) {
    if(e.getModifiers() == e.META_MASK){
        popup.setTextArea(stderrText);
        popup.show(this.stderrText, e.getX(), e.getY());
    }

  }

}




