package coco.gui.editor;

import org.gjt.sp.jedit.textarea.*;
import org.gjt.sp.jedit.syntax.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * This class implements a simple file editor, using the JEditTextArea component.
 * It is a JPanel, containing the editor area. To open a file for editing, use the constuctor.
 * To save the file, call 'saveFile()'. Via 'set/getFile(...)' you can change the name of the file (i.e. for storing
 * under a different name). The file, which is displayed in this editor is not locked in the file system!
 */

public class FileEditor extends JPanel {

    File file;
    String title;

    JEditTextArea textArea;
    JLabel statusLine;

    boolean dirty;

    public FileEditor(File file, String title) {
	super();

	this.file = file;
	this.title = title;

	init();

	openFile();
    }

    void init() {
	setLayout(new BorderLayout());

	textArea = new JEditTextArea();
	textArea.setTokenMarker(new JavaTokenMarker());

	InputHandler handler = new DefaultInputHandler();
	handler.addDefaultKeyBindings();

        // key bindings
	handler.addKeyBinding("C+s", new ActionListener() {
		public void actionPerformed(ActionEvent e) {
                  System.out.println("File saved");
                  saveFile();
		}
	    });





	textArea.setInputHandler(handler);

	add(textArea, BorderLayout.CENTER);

	statusLine = new JLabel("Position");
	add(statusLine, BorderLayout.SOUTH);

	textArea.addCaretListener(new CaretListener() {
		public void caretUpdate(CaretEvent e) {update();}

		private void update() {
		    int line = textArea.getCaretLine();
		    int col = textArea.getCaretPosition() - textArea.getLineStartOffset(line);
		    statusLine.setText("Position " + line + ":" + col);
		}
	    });




	textArea.getDocument().addDocumentListener(new DocumentListener() {
		public void changedUpdate(DocumentEvent e) {update();}
		public void insertUpdate(DocumentEvent e) {update();}
		public void removeUpdate(DocumentEvent e) {update();}

		private void update() {
		    dirty = true;
		}

	    });
    }

    /**
     * Open the file named in the constructor.
     */
    void openFile() {
	try {
	    if (file != null) {
		if (file.exists() && file.canRead()) {

		    StringBuffer buf = new StringBuffer();

		    // read file
		    BufferedReader in = new BufferedReader(new FileReader(file));

		    String line;
		    while ((line = in.readLine()) != null) {
			buf.append(line);
			buf.append("\n");
		    }
		    in.close();

		    // file content
		    String content = buf.toString();

		    textArea.setText(content);

		    dirty = false;

		    goToLine(0);
		}
	    }
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    /**
     * Save the file.
     */
    void saveFile() {
	try {
	    if (file != null) {
		if (file.exists() && file.canWrite()) {

		    // open the file for writing
		    FileOutputStream out = new FileOutputStream(file);

		    // get the content of the editor as bytes
		    String content = textArea.getText();
		    byte[] data = content.getBytes();

		    // write data
		    out.write(data);
		    out.close();

		    dirty = false;
		}
	    }
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    public void setFile(File f) {
	file = f;
    }

    public File getFile() {
	return(file);
    }

    public String getTitle() {
	return(title);
    }

    public void setTitle(String s) {
	title = s;
    }

    public boolean hasChanged() {
	return(dirty);
    }

    public void setEditable(boolean e) {
	textArea.setEditable(e);
    }


    public String getText() {
	return(textArea.getText());
    }

    public void setText(String s) {
	textArea.setText(s);
    }

    public void goToLine(int line) {
	textArea.scrollTo(line, 0);
	int pos = textArea.getLineStartOffset(line);
	textArea.setCaretPosition(pos);
    }

    public int getLineCount() {
	return(textArea.getLineCount());
    }

}
