package coco.model;

import javax.swing.tree.*;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeModelListener;

import java.util.Enumeration;

import coco.project.*;
import coco.model.nodes.*;
import coco.utils.*;

/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

 /**
  * The CoCoTreeModel is the model for the JTree. It encapsulates
  * all modifications of the tree structure.
  */
public class CoCoTreeModel extends DefaultTreeModel{

 /**
  * The compSpecs attribute points to the component list tree node of the
  * current project. It just simplifies its access.
  */
  private ComponentListTreeNode compSpecs = null;

 /**
  * The abstcompSpecs attribute points to the abstract component list tree
  * node of the current project. It just simplifies its access.
  */
  private AbstractComponentListTreeNode abstcompSpecs = null;

  private PropertySetListTreeNode propSpecs = null;
  private DatatypeListTreeNode typeSpecs = null;

  int[] intarray = null;
  TreeNode[] childarray = null;
  int[] intarray2 = null;
  TreeNode[] childarray2 = null;


  // ------------------------------------------ Constructors

  /**
   * This Constructor creates a new treemodel for the given project.
   * It inserts the project tree node and an empty componentlist node.
   */
  public CoCoTreeModel(CoCoProject proj) {
    super(new ProjectTreeNode(""));
    // set the name of the project tree node
    String name = proj.getProjectName();
    if((name == null) || (name.length() == 0))
      name = "Unnamed";
    // insert the Project Tree Node
    setRoot(new ProjectTreeNode(name));
    // insert the component list tree node
    compSpecs = new ComponentListTreeNode("Components");
    abstcompSpecs = new AbstractComponentListTreeNode("Abstract Components");
    propSpecs = new PropertySetListTreeNode("Property Sets");
    typeSpecs = new DatatypeListTreeNode("Data Types");
    Object o = getRoot();
    Assert.require(o instanceof CoCoTreeNode);
    CoCoTreeNode root = (CoCoTreeNode) o;
    insertNodeInto(compSpecs, root, (int)0);
    insertNodeInto(abstcompSpecs, root, (int)0);
    insertNodeInto(propSpecs, root, (int)0);
    insertNodeInto(typeSpecs, root, (int)0);
  }

  // ------------------------------------------ Public Methods

  public void changeProjectNameTo(String s) {
    ((ProjectTreeNode)this.root).setName(s);
    // fire node removed event
    intarray = new int[0];
    childarray = new TreeNode[0];
    this.fireTreeNodesChanged(this, this.getPathToRoot(this.root), intarray, childarray);
  }

  public synchronized void remove(ComponentTreeNode n) {
    Assert.require(n);
    CoCoTreeNode node = this.getCoCoTreeNode(n.getSourceFileName());
    int index = this.compSpecs.getIndex(node);
    this.compSpecs.remove(index);

    // fire node removed event
    intarray = new int[1];
    intarray[0] = index;
    childarray = new TreeNode[1];
    childarray[0] = node;
    this.fireTreeNodesRemoved(this, this.getPathToRoot(this.compSpecs), intarray, childarray);
  }

  public synchronized void remove(PropertySetTreeNode n) {
    Assert.require(n);
    CoCoTreeNode node = this.getCoCoTreeNode(n.getSourceFileName());
    int index = this.propSpecs.getIndex(node);
    this.propSpecs.remove(index);

    // fire node removed event
    intarray = new int[1];
    intarray[0] = index;
    childarray = new TreeNode[1];
    childarray[0] = node;
    this.fireTreeNodesRemoved(this, this.getPathToRoot(this.propSpecs), intarray, childarray);
  }

  public synchronized void remove(DatatypeTreeNode n) {
    Assert.require(n);
    CoCoTreeNode node = this.getCoCoTreeNode(n.getSourceFileName());
    int index = this.typeSpecs.getIndex(node);
    this.typeSpecs.remove(index);

    // fire node removed event
    intarray = new int[1];
    intarray[0] = index;
    childarray = new TreeNode[1];
    childarray[0] = node;
    this.fireTreeNodesRemoved(this, this.getPathToRoot(this.typeSpecs), intarray, childarray);
  }

  public synchronized void remove(AbstractComponentTreeNode n) {
    Assert.require(n);
    CoCoTreeNode node = this.getCoCoTreeNode(n.getSourceFileName());
    int index = this.abstcompSpecs.getIndex(node);
    this.abstcompSpecs.remove(index);

    // fire node removed event
    intarray = new int[1];
    intarray[0] = index;
    childarray = new TreeNode[1];
    childarray[0] = node;
    this.fireTreeNodesRemoved(this, this.getPathToRoot(this.abstcompSpecs), intarray, childarray);
  }

  /**
   * Get the component tree node for with this name;
   */
   public synchronized CoCoTreeNode getCoCoTreeNode(String name) {
    for (int i=0; i<this.compSpecs.getChildCount(); i++) {
      CoCoTreeNode child = (CoCoTreeNode)compSpecs.getChildAt(i);
      if (child.getSourceFileName().equals(name)) {
        return child;
      }
    }
    for (int i=0; i<this.abstcompSpecs.getChildCount(); i++) {
      CoCoTreeNode child = (CoCoTreeNode)abstcompSpecs.getChildAt(i);
      if (child.getSourceFileName().equals(name)) {
      return child;
     }
    }

    for (int i=0; i<this.propSpecs.getChildCount(); i++) {
      CoCoTreeNode child = (CoCoTreeNode)propSpecs.getChildAt(i);
      if (child.getSourceFileName().equals(name)) {
      return child;
     }
    }

    for (int i=0; i<this.typeSpecs.getChildCount(); i++) {
      CoCoTreeNode child = (CoCoTreeNode)typeSpecs.getChildAt(i);
      if (child.getSourceFileName().equals(name)) {
      return child;
     }
    }

    return null;
   }

   public synchronized boolean isInCompSpecs(ComponentTreeNode n) {
    Enumeration e = this.compSpecs.children();
    while (e.hasMoreElements()) {
      if (((ComponentTreeNode)e.nextElement()) == n) return true;
    }
    return false;
   }

   /**
    * Exchange the old with the new Component tree subnode.
    */
   public synchronized void exchangeNodes(CoCoTreeNode neu, CoCoTreeNode old) {
   if (neu instanceof ComponentTreeNode) {
    int index = this.compSpecs.getIndex(old);
    this.compSpecs.remove(index);
//    this.compSpecs.remove(old);

    // insert new
    this.compSpecs.insert(neu, index);
    this.nodeStructureChanged(compSpecs);
    }

   if (neu instanceof AbstractComponentTreeNode) {
    int index = this.abstcompSpecs.getIndex(old);
    this.abstcompSpecs.remove(index);

    // insert new
    this.abstcompSpecs.insert(neu, index);
    this.nodeStructureChanged(abstcompSpecs);
    }

   if (neu instanceof PropertySetTreeNode) {
    int index = this.propSpecs.getIndex(old);
    this.propSpecs.remove(index);

    // insert new
    this.propSpecs.insert(neu, index);
    this.nodeStructureChanged(propSpecs);
    }

   if (neu instanceof DatatypeTreeNode) {
    int index = this.typeSpecs.getIndex(old);
    this.typeSpecs.remove(index);

    // insert new
    this.typeSpecs.insert(neu, index);
    this.nodeStructureChanged(typeSpecs);
    }
   }

  /*
     * @param source the Object responsible for generating the event (typically
     *               the creator of the event object passes <code>this</code>
     *               for its value)
     * @param path   an array of Object identifying the path to the
     *               parent of the modified item(s), where the first element
     *               of the array is the Object stored at the root node and
     *               the last element is the Object stored at the parent node
     * @param childIndices an array of <code>int</code> that specifies the
     *               index values of the removed items. The indices must be
     *               in sorted order, from lowest to highest
     * @param children an array of Object containing the inserted, removed, or
     *                 changed objects
     * @see TreePath
  */

  /**
   * Add a component tree node to the JTree.
   */
  public synchronized CoCoTreeNode add(ComponentTreeNode n){
    Assert.require(n);
    ComponentTreeNode childNode = new ComponentTreeNode();
    int idx = compSpecs.getChildCount();
    this.insertNodeInto(n, compSpecs, idx);
    this.nodeStructureChanged(compSpecs);
    return childNode;
  }

  public synchronized CoCoTreeNode add(PropertySetTreeNode n){
    Assert.require(n);
    PropertySetTreeNode childNode = new PropertySetTreeNode();
    int idx = propSpecs.getChildCount();
    this.insertNodeInto(n, propSpecs, idx);
    this.nodeStructureChanged(propSpecs);
    return childNode;
  }

  /**
   * Add an abstract component tree node to the JTree.
   */
  public synchronized CoCoTreeNode add(AbstractComponentTreeNode n){
    Assert.require(n);
    AbstractComponentTreeNode childNode = new AbstractComponentTreeNode();
    int idx = abstcompSpecs.getChildCount();
    this.insertNodeInto(n, abstcompSpecs, idx);
    this.nodeStructureChanged(abstcompSpecs);
    return childNode;
  }

  public synchronized CoCoTreeNode add(DatatypeTreeNode n){
    Assert.require(n);
    DatatypeTreeNode childNode = new DatatypeTreeNode();
    int idx = typeSpecs.getChildCount();
    this.insertNodeInto(n, typeSpecs, idx);
    this.nodeStructureChanged(typeSpecs);

    return childNode;
  }

 /**
  * Add an error list tree node to the Tree.
  */
  public synchronized CoCoTreeNode add(ErrorListTreeNode n){
    Assert.require(n);
    ErrorListTreeNode childNode = new ErrorListTreeNode();
    int idx = compSpecs.getChildCount();
    this.insertNodeInto(n, compSpecs, idx);
    this.nodeStructureChanged(compSpecs);
    return childNode;
  }
}
