package coco.model;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.JScrollPane;
import javax.swing.JEditorPane;
import coco.gui.MessageBox;

import java.io.File;
import java.io.StringReader;
import java.io.FileReader;
import java.io.Reader;
import java.util.*;

import coco.fuzzyparser.*;
import coco.model.*;
import coco.model.nodes.*;
import coco.project.*;
import coco.utils.*;
import coco.config.*;
import coco.filehandling.*;

import coco.parser.*;
import coco.parser.ast.*;
import coco.parser.symtab.*;

//import org.gjt.sp.jedit.textarea.*;
import coco.gui.editor.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author
 * @version 1.0
 */

/**
 * The Controller class manages the dependencies between the JTree,
 * the CoCoProject and the JScrollPane of the CoCo Composition
 * Environment.
 */
public class Controller {

    // serves as semaphore for tree access
    Object semaphore = new Object();

    boolean portsWasThere = false;
    boolean instancesWasThere = false;
    boolean propertiesWasThere = false;
    boolean connectorsWasThere = false;


//  private Enumeration expandedPaths = null;

  /**
   * The MultiEditor of the CoCoFrame.
   */
  MultiEdit multiEdit;

   /**
   * The current project beeing edited. There is only one open project for
   * instance.
   */
  private CoCoProject currentProject = null;

  /**
   * The JTree on the left which displays the project structure.
   */
  private CoCoJTree tree = null;

  /**
   * The TreeModel used to manage the JTree.
   */
  private CoCoTreeModel model = null;

  // ------------------------------------------ Constructors

  public static Controller getInstance() {
    Assert.require(instance);
    return instance;
  }
  private static Controller instance;

  /**
   * This Constructor initializes the Controller, given a JTree and a
   * JScrollPane by creating a new empty CoCoProject and a CoCoTreeModel
   * for the JTree.
   */
  public Controller(CoCoJTree tr, MultiEdit me) {
    Assert.require(tr);
    Assert.require(me);

    // provide static instance variable
    this.instance = this;

    // store the tree
    tree = tr;
    // create a new, empty CoCoProject
    currentProject = new CoCoProject();
    // initialize model and register it with the tree
    model = new CoCoTreeModel(currentProject);
    tree.setModel(model);
    // store the detailpane
    multiEdit = me;

    // start thread which initiates reparse every 2 seconds
    Thread t = new Thread(new Runnable() {
      public void run() {
        while (true) {
          try {
            //  System.out.println("Reparsing ...");
            reparseTree();
          } catch (Exception e) {
            e.printStackTrace();
          }

          try {
            Thread.currentThread().sleep(2000);
          } catch (Exception ex) {}
        }
      }
    });
   t.start();
  }

  // ------------------------------------------ Public Methods


  /**TG
   * Invokes the coco parser and performs checks
   * @todo implement a tool abstraction layer in order to plug in other tools (like the OTI parser)
   * --> Michael
   */
  public void check(){
//    this.storeChangedTextsInCoCoProject();
    this.saveProject();
    Enumeration files = this.currentProject.getCoCoFiles();

    CoCoParser parser = null;
    StringReader reader;
    String txt;
    CoCoFile cfile;
    ASTCompilationUnit root;

    SymtabService.removeAll();
    /*
     * parse the entire project
     */
    while (files.hasMoreElements()) {
      cfile = (CoCoFile)files.nextElement();
      txt = cfile.getActualText();
      reader = new StringReader(txt);
      parser = ParserService.getCoCoParser(reader);

      try {
        root = parser.compilationUnit();
        System.out.println(cfile.getFilename()+" ... OK");
      } catch (Exception e) {
        System.out.println(cfile.getFilename()+" ... There have been errors:");
        System.out.println("Last error message was: \n" + e.getMessage());
        // there have been syntax errors, so skip the rest
        return;
      }

      /**TG
       * @todo add semantic checker (still needs to be updated to the new coco semantic)
       * --> Alex
       */

      /**TG
       * @todo add calls to code generators
       * --> T. Fischer
       */



    }
  }


  public void closeTab(String fileName) {
      StringEditor edit = this.multiEdit.getEditorByTitle(fileName);
      if ((edit != null) || (edit.hasChanged())) {
        String text = edit.getText();
        this.currentProject.setChangedTextForFile(fileName, text);
        this.multiEdit.removeEditorTab(fileName);
        String name = this.multiEdit.getSelectedEditor();
        if(name != null)
          this.multiEdit.selectEditor(name);
    }
  }

  public void saveFile(String fileName) {
      StringEditor edit = this.multiEdit.getEditorByTitle(fileName);
      if ((edit != null) || (edit.hasChanged())) {
        String text = edit.getText();
        this.multiEdit.resetChanged(edit.getTitle());
        this.currentProject.setChangedTextForFile(fileName, text);
        if (this.projectCanSave()) {
          this.currentProject.saveFile(fileName);
        } else {
          // +++++++
          MessageBox msg =
            new MessageBox("Message",
              "No project file set. Please select one.",
              MessageBox.ANSWER_OK);
            msg.show();

          CoCoProjectFileFilter filter = new CoCoProjectFileFilter();
          File f = FileService.doFileDialog(filter);
          if (f != null){
            this.currentProject.saveProjectAs(f);
        }
      }
    }
  }

  public void changeProjectNameTo(String s) {
    this.model.changeProjectNameTo(s);
  }


  public synchronized void removePropertyset(PropertySetTreeNode node) {
    // remove from editor;
    multiEdit.removeEditorTab(node.getSourceFileName());
    // remove form model;
    model.remove(node);
    // remove form project;
    this.currentProject.removeCoCoFile(node.getSourceFileName());
  }

  public synchronized void removeDatatype(DatatypeTreeNode node) {
    // remove from editor;
    multiEdit.removeEditorTab(node.getSourceFileName());
    // remove form model;
    model.remove(node);
    // remove form project;
    this.currentProject.removeCoCoFile(node.getSourceFileName());
  }

  public synchronized void removeComponent(ComponentTreeNode node) {
    // remove from editor;
    multiEdit.removeEditorTab(node.getSourceFileName());
    // remove form model;
    model.remove(node);
    // remove form project;
    this.currentProject.removeCoCoFile(node.getSourceFileName());
  }

  public synchronized void removeAbstractComponent(AbstractComponentTreeNode node) {
    // remove from editor;
    multiEdit.removeEditorTab(node.getSourceFileName());
    // remove form model;
    model.remove(node);
    // remove form project;
    this.currentProject.removeCoCoFile(node.getSourceFileName());
  }

  /**
   * Adds a new, empty component with name compName to the
   * current project.
   */
  public synchronized void addNewComponentToProject(String compName)  {
    if (this.currentProject.isInProject(compName+".cm")) {
      System.out.println("Error: A component with this name does already exist!");
    } else {
      CoCoFile cFile = this.currentProject.addNewComponentToProject(compName);
      String text = cFile.getActualText();

      StringReader sReader = new StringReader(text);
      CoCoTreeNode cu = parseComponentFile(sReader, cFile.getFilename());
      if (cu instanceof ComponentTreeNode) {
        addToTreeView((ComponentTreeNode)cu);
      } else {
        if (cu instanceof ErrorListTreeNode) {
          addToTreeView((ErrorListTreeNode)cu);
        }
      }
    }
  }

  public synchronized void addNewPropertySetToProject(String propName)  {
    if (this.currentProject.isInProject(propName+".prop")) {
      System.out.println("Error: A property set with this name does already exist!");
    } else {
      CoCoFile cFile = this.currentProject.addNewPropertySetToProject(propName);
      String text = cFile.getActualText();

      StringReader sReader = new StringReader(text);
      CoCoTreeNode cu = parsePropertyFile(sReader, cFile.getFilename());
      if (cu instanceof PropertySetTreeNode) {
        addToTreeView((PropertySetTreeNode)cu);
      } else {
        if (cu instanceof ErrorListTreeNode) {
          addToTreeView((ErrorListTreeNode)cu);
        }
      }
    }
  }


  public synchronized void addNewDataTypeToProject(String typeName)  {
    if (this.currentProject.isInProject(typeName+".type")) {
      System.out.println("Error: A data type with this name does already exist!");
    } else {
      CoCoFile cFile = this.currentProject.addNewDataTypeToProject(typeName);
      String text = cFile.getActualText();

      StringReader sReader = new StringReader(text);
      CoCoTreeNode cu = parseDatatypeFile(sReader, cFile.getFilename());
      if (cu instanceof DatatypeTreeNode) {
        addToTreeView((DatatypeTreeNode)cu);
      } else {
        if (cu instanceof ErrorListTreeNode) {
          addToTreeView((ErrorListTreeNode)cu);
        }
      }
    }
  }

  /**
   * Adds a new, empty abstract component with name compName to the
   * current project.
   */
  public synchronized void addNewAbstractComponentToProject(String compName)  {
    if (this.currentProject.isInProject(compName+".acm")) {
      System.out.println("Error: An abstract component with this name does already exist!");
    } else {
      CoCoFile cFile = this.currentProject.addNewAbstractComponentToProject(compName);
      String text = cFile.getActualText();

      StringReader sReader = new StringReader(text);
      CoCoTreeNode cu = parseAbstractComponentFile(sReader, cFile.getFilename());
      if (cu instanceof AbstractComponentTreeNode) {
        addToTreeView((AbstractComponentTreeNode)cu);
      } else {
        if (cu instanceof ErrorListTreeNode) {
          addToTreeView((ErrorListTreeNode)cu);
        }
      }
    }
  }

  /**
   * Adds a component file to the project.
   */
  public synchronized void addComponentFileToProject(File f) {
    CoCoFile cFile = currentProject.addEntityFileToProject(f);
    if (cFile != null) {
      String text = cFile.getOriginalText();
      StringReader sReader = new StringReader(text);
      CoCoTreeNode cu = parseComponentFile(sReader, cFile.getFilename());
      if (cu instanceof ComponentTreeNode) {
        addToTreeView((ComponentTreeNode)cu);
      } else {
        if (cu instanceof ErrorListTreeNode) {
          addToTreeView((ErrorListTreeNode)cu);
        }
      }
    }
  }

  /**
   * Adds a component file to the project.
   */
  public synchronized void addDataTypeFileToProject(File f) {
    CoCoFile cFile = currentProject.addEntityFileToProject(f);
    if (cFile != null) {
      String text = cFile.getOriginalText();
      StringReader sReader = new StringReader(text);
      CoCoTreeNode cu = parseDatatypeFile(sReader, cFile.getFilename());
      if (cu instanceof DatatypeTreeNode) {
        addToTreeView((DatatypeTreeNode)cu);
      } else {
        if (cu instanceof ErrorListTreeNode) {
          addToTreeView((ErrorListTreeNode)cu);
        }
      }
    }
  }


  /**
   * Adds an abstract component file to the project.
   */
  public synchronized void  addAbstractComponentFileToProject(File f) {
    CoCoFile cFile = currentProject.addEntityFileToProject(f);
    if (cFile != null) {
      String text = cFile.getOriginalText();
      StringReader sReader = new StringReader(text);
      CoCoTreeNode cu = parseAbstractComponentFile(sReader, cFile.getFilename());
      if (cu instanceof AbstractComponentTreeNode) {
        addToTreeView((AbstractComponentTreeNode)cu);
      } else {
        if (cu instanceof ErrorListTreeNode) {
          addToTreeView((ErrorListTreeNode)cu);
        }
      }
    }
  }

  /**
   * Adds an abstract component file to the project.
   */
  public synchronized void  addPropertySetFileToProject(File f) {
    CoCoFile cFile = currentProject.addEntityFileToProject(f);
    if (cFile != null) {
      String text = cFile.getOriginalText();
      StringReader sReader = new StringReader(text);
      CoCoTreeNode cu = parsePropertyFile(sReader, cFile.getFilename());
      if (cu instanceof PropertySetTreeNode) {
        addToTreeView((PropertySetTreeNode)cu);
      } else {
        if (cu instanceof ErrorListTreeNode) {
          addToTreeView((ErrorListTreeNode)cu);
        }
      }
    }
  }

  /**
   * Closes the project. Does not save changed files !!!
   */
  public void closeProject() {
    // remove the open editor tabs from the multiedit
    Enumeration enum = this.multiEdit.getEditors();
    while (enum.hasMoreElements()) {
      String edit = ((String)enum.nextElement());
      System.out.println("Remove Editor :"+edit);
      this.multiEdit.removeEditorTab(edit);
    }
    // set new project
    this.setProject(new CoCoProject());
    GlobalConfig.getInstance().setProperty(GlobalConfig.OPTION_PROJECT_LAST,"");
  }

  /**
   * Return true, if the files opended in the editor have not been edited,
   * false otherwise.
   */
  public boolean projectHasChanged() {
    // did project change ? -> new components ...
    if (this.currentProject.hasChanged()) {
      System.out.println("CoCo Files changed.");
      return true;
    }

    // did user edit files in the editor ?
    Enumeration files = this.multiEdit.getEditors();
    while (files.hasMoreElements()) {
      String edit = (String)files.nextElement();
      if (this.multiEdit.getEditorByTitle(edit).hasChanged()) {
        System.out.println("Editor Files changed.");
        return true;
      }
    }
    return false;
  }

  /**
   * A project can be saved, if it has a project file attached. Used by
   * the gui in order to find out, if a FileDialog has to pop up to select
   * a project file.
   */
  public boolean projectCanSave() {
    return this.currentProject.hasProjetFile();
  }

  /**
   * Saves the current project.
   */
  public void saveProject() {
    Assert.require(this.currentProject);
    storeChangedTextsInCoCoProject();
    currentProject.saveProject();
  }

  /**
   * Saves the current project in project file f.
   */
  public void saveProjectAs(File f) {
    Assert.require(this.currentProject);
    Assert.require(f);
    storeChangedTextsInCoCoProject();
    currentProject.saveProjectAs(f);
    this.model.changeProjectNameTo(this.currentProject.getProjectName());
  }

  /**
   * Writes back all changed texts from the string editors to the
   * coco project. This has to be done before saving the project in
   * order to save the changes.
   */
  private void storeChangedTextsInCoCoProject() {
    Enumeration enum = this.multiEdit.getEditors();
    while (enum.hasMoreElements()) {
      String edit = (String)enum.nextElement();
      String text = this.multiEdit.getEditorByTitle(edit).getText();
      if (this.multiEdit.getEditorByTitle(edit).hasChanged()) {
        this.currentProject.setChangedTextForFile(edit, text);
        this.multiEdit.resetChanged(edit);
      }
    }
  }


  /**
   * Sets the Controller to the passed project. It creates a new CoCo
   * TreeModel for the JTree.
   */
  public void setProject(CoCoProject proj) {
   // set currentProject
   currentProject = proj;
   // create a new model...
   model = new CoCoTreeModel(proj);
   // ... and connect with the tree
   tree.setModel(model);
   // put the compilation units into the treemodel
   //ParserService.reset();
   Enumeration elems = currentProject.getCoCoFiles();
   while (elems.hasMoreElements()) {
     CoCoFile cFile = (CoCoFile)elems.nextElement();
     String text = cFile.getOriginalText();
     StringReader sReader = new StringReader(text);

     // distinguish between coco file types ...
     if (cFile.getTypeOf().equals("cm")) {
       // ... component
       CoCoTreeNode cu = parseComponentFile(sReader, cFile.getFilename());
       if (cu instanceof ComponentTreeNode) {
         addToTreeView((ComponentTreeNode)cu);
       } else {
         if (cu instanceof ErrorListTreeNode) {
        addToTreeView((ErrorListTreeNode)cu);
        }
      }
     }

     // distinguish between coco file types ...
     if (cFile.getTypeOf().equals("prop")) {
       // ... component
       CoCoTreeNode cu = parsePropertyFile(sReader, cFile.getFilename());
       if (cu instanceof PropertySetTreeNode) {
         addToTreeView((PropertySetTreeNode)cu);
       } else {
         if (cu instanceof ErrorListTreeNode) {
        addToTreeView((ErrorListTreeNode)cu);
        }
      }
     }


     if (cFile.getTypeOf().equals("acm")) {
      // ... abstract component
      CoCoTreeNode cu = parseAbstractComponentFile(sReader, cFile.getFilename());
      if (cu instanceof AbstractComponentTreeNode) {
         addToTreeView((AbstractComponentTreeNode)cu);
       } else {
         if (cu instanceof ErrorListTreeNode) {
        addToTreeView((ErrorListTreeNode)cu);
       }
     }
   }

     if (cFile.getTypeOf().equals("type")) {
      // ... abstract component
      CoCoTreeNode cu = parseDatatypeFile(sReader, cFile.getFilename());
      if (cu instanceof DatatypeTreeNode) {
         addToTreeView((DatatypeTreeNode)cu);
       } else {
         if (cu instanceof ErrorListTreeNode) {
        addToTreeView((ErrorListTreeNode)cu);
       }
     }
   }
  }}


    /**
     * Retrieves the current project.
     */
    public CoCoProject getProject() {
	return(currentProject);
    }

  /**
   * Shows the file corresponding to some tree node in the editor,
   * and jumps to the lines of the construct, if possible.
   */
  public void showComponent(CoCoTreeNode node) {
    //System.out.println("Bring node "+node.getName()+" to the front.");
    //System.out.println("Lines: "+node.getBeginLine()+"-"+node.getEndLine());

    // get containing file and node
    CoCoTreeNode parent = getFileNameParentForNode(node);
    if (parent == null){
      return;
    } else {
      if (parent.getSourceFileName() == null) return;

      String parentFileName = parent.getSourceFileName();
      // test if file already opened
      if (!isAllreadyOpen(parentFileName)) {
        // open it
        String text = this.currentProject.getTextForFile(parentFileName);
        multiEdit.addEditorTab(text, parentFileName);
      }

      // select the display tab
      multiEdit.selectEditor(parentFileName);

      // jump to begin line of construct, if possible
      if (node.getBeginLine() != -1) {
        //System.out.println("going to line " + (node.getBeginLine()-1));
        try {
          multiEdit.goToLine(parentFileName, node.getBeginLine()-1);
        } catch (Exception ex) {
          // do nothing
        }
      }
    }
  }

  // ------------------------------------------ Private Methods

  /**
   * Is called every ... seconds to reparse the selected open file
   * in order to adapt the project structure tree.
   */
  private synchronized void reparseTree(){
    // ---- get corresponding File, but give acutal text to reader !
    String editName = this.multiEdit.getSelectedEditor();
    StringEditor edit = this.multiEdit.getEditorByTitle(editName);
    if (edit == null) return;
    if (!edit.hasFurtherChanged()) return;  // setzt selbst zurck !
    String text = edit.getText();
    StringReader sReader = new StringReader(text);

    // ---- parse it
    FuzzyParserService.reset();
    CoCoTreeNode node = null;
    if (editName.endsWith("acm")) {
      // ---- acm
      node = this.parseAbstractComponentFile(sReader, editName);
    } else {
      // ---- cm
      if (editName.endsWith("cm")) {
        node = this.parseComponentFile(sReader, editName);
      } else {
        if (editName.endsWith("prop")) {
          node = this.parsePropertyFile(sReader, editName);
        } else {
          if (editName.endsWith("type")) {
            node = this.parseDatatypeFile(sReader, editName);
          }
        }
      }
    }
    // get old node
    CoCoTreeNode oldNode = model.getCoCoTreeNode(node.getSourceFileName());
    if (oldNode == null) {
      System.out.println("oldNode == null !");
      return;
    }  else {
      exchangeTreeNode(node, oldNode);
    }
  }

  private void exchangeTreeNode(CoCoTreeNode node, CoCoTreeNode oldNode) {
    boolean portsExpanded = false;
    boolean instancesExpanded = false;
    boolean propertiesExpanded = false;
    boolean connectorsExpanded = false;
    boolean componentExpanded = false;
    boolean errorsExpanded = false;
    boolean rolesExpanded = false;

      // ----- figure out what has been expanded in the original node
      TreePath cpath = new TreePath(model.getPathToRoot(oldNode));
      componentExpanded = !tree.isCollapsed(cpath);
      for (int i=0; i<oldNode.getChildCount(); i++) {
        // get child
        CoCoTreeNode childNode = (CoCoTreeNode)oldNode.getChildAt(i);
        // expanded ?
        TreePath path = new TreePath(model.getPathToRoot(childNode));
        boolean expanded = !tree.isCollapsed(path);
        if (expanded) {
          if (childNode instanceof PortListTreeNode) {portsExpanded = true;}
          if (childNode instanceof InstanceListTreeNode) {instancesExpanded = true;}
          if (childNode instanceof PropertyListTreeNode) {propertiesExpanded = true;}
          if (childNode instanceof ErrorListTreeNode) {errorsExpanded = true;}
          if (childNode instanceof ConnectorListTreeNode) {connectorsExpanded = true;}
          if (childNode instanceof RoleListTreeNode) {rolesExpanded = true;}
        }
      }

      // ---- avoid repainting during changes are performed
      this.tree.noPaint();

      // ---- exchange node
      model.exchangeNodes(node, oldNode);

      // ---- expand if necessary
      if (componentExpanded) {
        tree.expandPath(new TreePath(model.getPathToRoot(node)));
        for (int i=0; i<node.getChildCount(); i++) {
          CoCoTreeNode childNode = (CoCoTreeNode)node.getChildAt(i);
          TreePath path = new TreePath(model.getPathToRoot(childNode));
           if ((childNode instanceof PortListTreeNode) && (portsExpanded)) {tree.expandPath(path);}
           if ((childNode instanceof InstanceListTreeNode) && (instancesExpanded)) {tree.expandPath(path);}
           if ((childNode instanceof PropertyListTreeNode) && (propertiesExpanded)) {tree.expandPath(path);}
           if ((childNode instanceof ErrorListTreeNode)) {tree.expandPath(path);}
           if ((childNode instanceof ConnectorListTreeNode) && (connectorsExpanded)) {tree.expandPath(path);}
           if ((childNode instanceof RoleListTreeNode) && (rolesExpanded)) {tree.expandPath(path);}
         }
        }

      // ---- repaint JTree
      this.tree.doPaint();
  }

  /**
   * Return the file, the node points to. For this, one may need to
   * climb up in the tree.
   */
   private CoCoTreeNode getFileNameParentForNode(CoCoTreeNode node) {
    while((node.getSourceFileName() == null) && (node.getParent() != null)) {
      node = (CoCoTreeNode)node.getParent();
    }
    return node;
   }

  /**
   * Returns true, if file corresponding to node with name name is
   * already opened in the multi editor, flase otherwise.
   */
  private boolean isAllreadyOpen(String fileName) {
    Enumeration enum = this.multiEdit.getEditors();
    while (enum.hasMoreElements()) {
      String eName = (String)enum.nextElement();
      if (eName.equals(fileName)) return true;
    }
    return false;
  }

  private CoCoTreeNode parseAbstractComponentFile(Reader r, String fileName) {
    Assert.require(r);

    // get a FuzzyParser
    FuzzyParserService.reset();
    FuzzyParser parser = null;
    parser = FuzzyParserService.getfuzzyParser(r, fileName);

    // parse the entity file
    CoCoTreeNode root= null;
    try {
      // parse component file
      root = parser.abstractComponent();
    } catch (coco.fuzzyparser.ParseException e) {
    }

    // set file reference in the component tree node
    if (root == null) {
      root = new AbstractComponentTreeNode();
      root.setName("?");
    }
    root.setSourceFileName(fileName);

    // add Error node
    ErrorListTreeNode errors = FuzzyParserService.getErrors();
    if (errors.hasChildren()) {
      if (root != null) {
        root.add(errors);
      }
    }
    return root;
  }


  private CoCoTreeNode parseComponentFile(Reader r, String fileName) {
    Assert.require(r);

    // get a FuzzyParser
    FuzzyParserService.reset();
    FuzzyParser parser = null;
    parser = FuzzyParserService.getfuzzyParser(r, fileName);

    // parse the entity file
    CoCoTreeNode root= null;
    try {
      // parse component file
      root = parser.component();
    } catch (coco.fuzzyparser.ParseException e) {
    }

    // set file reference in the component tree node
    if (root == null) {
      root = new ComponentTreeNode();
      root.setName("?");
    }
    root.setSourceFileName(fileName);

    // add Error node
    ErrorListTreeNode errors = FuzzyParserService.getErrors();
    if (errors.hasChildren()) {
      if (root != null) {
        root.add(errors);
      }
    }
    return root;
  }

private CoCoTreeNode parsePropertyFile(Reader r, String fileName) {
    Assert.require(r);

    // get a FuzzyParser
    FuzzyParserService.reset();
    FuzzyParser parser = null;
    parser = FuzzyParserService.getfuzzyParser(r, fileName);

    // parse the entity file
    CoCoTreeNode root= null;
    try {
      // parse component file
      root = parser.propertyset();
    } catch (coco.fuzzyparser.ParseException e) {
    }

    // set file reference in the component tree node
    if (root == null) {
      root = new PropertySetTreeNode();
      root.setName("?");
    }
    root.setSourceFileName(fileName);

    // add Error node
    ErrorListTreeNode errors = FuzzyParserService.getErrors();
    if (errors.hasChildren()) {
      if (root != null) {
        root.add(errors);
      }
    }
    return root;
  }

private CoCoTreeNode parseDatatypeFile(Reader r, String fileName) {
    Assert.require(r);

    // get a FuzzyParser
    FuzzyParserService.reset();
    FuzzyParser parser = null;
    parser = FuzzyParserService.getfuzzyParser(r, fileName);

    // parse the entity file
    CoCoTreeNode root= null;
    try {
      // parse component file
      root = parser.datatype();
    } catch (coco.fuzzyparser.ParseException e) {
    }

    // set file reference in the component tree node
    if (root == null) {
      root = new DatatypeTreeNode();
      root.setName("?");
    }
    root.setSourceFileName(fileName);

    // add Error node
    ErrorListTreeNode errors = FuzzyParserService.getErrors();
    if (errors.hasChildren()) {
      if (root != null) {
        root.add(errors);
      }
    }
    return root;
  }


  /**
   * Adds a ComponentTreeNode to the JTree.
   * @param cu   the component which is to be added
   */
    private void addToTreeView(ComponentTreeNode c){
      Assert.require(model);
      Assert.require(c);

      // add the component c to the model
      CoCoTreeNode node = model.add(c);
      // make the node visible within the tree view
      tree.scrollPathToVisible(new TreePath(node.getPath()));
    }

    private void addToTreeView(PropertySetTreeNode c){
      Assert.require(model);
      Assert.require(c);

      // add the component c to the model
      CoCoTreeNode node = model.add(c);
      // make the node visible within the tree view
      tree.scrollPathToVisible(new TreePath(node.getPath()));
    }


  /**
   * Adds an AbstractComponentTreeNode to the JTree.
   * @param cu   the component which is to be added
   */
    private void addToTreeView(AbstractComponentTreeNode c){
      Assert.require(model);
      Assert.require(c);

      // add the component c to the model
      CoCoTreeNode node = model.add(c);
      // make the node visible within the tree view
      tree.scrollPathToVisible(new TreePath(node.getPath()));
    }

    private void addToTreeView(DatatypeTreeNode c){
      Assert.require(model);
      Assert.require(c);

      // add the component c to the model
      CoCoTreeNode node = model.add(c);
      // make the node visible within the tree view
      tree.scrollPathToVisible(new TreePath(node.getPath()));
    }


    /**
     * Adds an ErrorListTreeNode to the JTree.
     */
    private void addToTreeView(ErrorListTreeNode c){
      Assert.require(model);
      Assert.require(c);

      // add the component c to the model
      CoCoTreeNode node = model.add(c);
      // make the node visible within the tree view
      tree.scrollPathToVisible(new TreePath(node.getPath()));
    }
}
