package coco.model.nodes;

import javax.swing.tree.*;
import java.io.File;
import javax.swing.ImageIcon;

import coco.fuzzyparser.Node;
import coco.model.commandsets.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:  Base Class of the Tree
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author M. Winter
 * @version 1.0
 */

 /**
  * The CoCoTreeNode is the base class for the TreeNodes used to model
  * the project structure tree.
  */
public class CoCoTreeNode extends DefaultMutableTreeNode{
  // the attributed File
  private String sourceFileName = null;

  public void setSourceFileName (String f) {sourceFileName = f;}
  public String getSourceFileName() {return sourceFileName;}

  // begin and end lines of within the text file of this parse unit
  private int beginLine = -1;
  private int endLine = -1;

  public void setBeginLine(int l) {beginLine = l;}
  public void setEndLine(int l) {endLine   = l;}
  public int getBeginLine() {return beginLine;}
  public int getEndLine() {return endLine;}

  // the name of the parse unit
  private String name = null;

  public void setName(String n) {name = n;}
  public String getName() {return name;}

  /**
   * Returns the icon wghich is to be shown for the respective node type.
   * @return the icon ressource. when null, the standard icon, according to the current L&F will be shown
   */
  public ImageIcon getIcon(){
    return null;
  }
  // Constructor 1
  public CoCoTreeNode() {
    setUserObject(this);
  }

  // Constructor 2
  public CoCoTreeNode(java.lang.String name) {
    this.name = name;
    //System.out.println("Constructor : " +name);
    setUserObject(this);
  }

  // toString Method
  public String toString() {
  //System.out.println("toString called; name = + " + name);
    if (name == null) {
      return "---";
    } else {
      return name;
    };
  }

  /**
   * Returns true, if the node has got children, false otherwise.
   */
  public boolean hasChildren() {
    if (this.children == null) {
      return false;
    } else {
      return !(this.children.isEmpty());
    }
  }

  public CommandSet getCommandSet() {
    return null;
  }
}