package coco.parser.ast;
import coco.utils.*;
import coco.parser.ast.enums.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTComponentBody extends GenericNode implements ASTComponentTypeBody{
  private ASTPortList portList = null;
  private ASTPropertyList propList = null;
  private ASTComponent parent = null;

  private ASTInstanceList instances = null;
  private ASTConnectorList connectors = null;
  public ASTComponentBody(ASTPortList ports, ASTPropertyList pl, ASTInstanceList il, ASTConnectorList cl) {

    portList = ports;
    propList = pl;
    instances = il;
    connectors = cl;
  }


  public void setParent(Node n){
    Assert.require(n instanceof ASTComponent);
    parent = (ASTComponent)n;
    if(portList != null)
      portList.setParent(n);
    if(propList != null){
      propList.setParent(n);
    }
    if(instances != null){
     instances.setParent(n);
    }
  }

  public ASTInstanceList getInstances(){
    return instances;
  }

  public void setInstances(ASTInstanceList il){
    instances = il;
  }
  public ASTPortList getPortList(){
    return portList;
  }

  public void setConnectors(ASTConnectorList cl){
    connectors = cl;
  }
  public ASTConnectorList getConnectors(){
    return connectors;
  }

  /**
   * setPortList sets the portlist of the astComponentBody. If param is null, then portlist is set to null
   * @param portList instance of ASTPortList
   */

  public void setPortList(ASTPortList pl) {
    this.portList = pl;
    Assert.require(parent);
    if(portList != null)
      portList.setParent(parent);
  }

  public ASTPropertyList getPropertyList(){
    return propList;
  }

  /**
   * setPropertyList sets the propertylist of the astComponentBody. If param is null, then propertylist is set to null
   * @param propertyList instance of ASTPropertyList
   */

  public void setPropertyList(ASTPropertyList pl) {

    this.propList = pl;
    Assert.require(parent);
    if(propList != null)
      propList.setParent(parent);
  }


  public String toString(){
    String res = "{\n";
    if(portList != null)
      res += portList.toString();
    if(propList !=null)
      res += propList.toString();
    if(instances != null)
      res += instances.toString();
    if(connectors != null)
      res += connectors.toString();
    res += "\n}";
    return res;
  }
  public String toPrologString(){
    String res = "";

    // translate the ports of this component to Prolog facts
    if(portList != null)
      res += portList.toPrologString();

    // if there are properties, translate them into Prolog facts
    if(propList != null){
      res += propList.toPrologString();
    }
    if(instances != null)
      res += instances.toPrologString();

    return res;
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    if(portList != null)
      portList.accept(v,data);


    if(propList != null){
      propList.accept(v,data);
    }
    if(instances != null)
      instances.accept(v, data);
    if(connectors != null)
      connectors.accept(v, data);

      if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}