package coco.parser.ast;
import java.util.*;
import coco.parser.ast.enums.*;
import coco.utils.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTComponentList extends GenericContainerNode {

  public ASTComponentList(ASTComponentType c){
    super();
    addNode(c);
  }

  public void addNode(Node n){
    Assert.require(n instanceof ASTComponentType);
    container.add(n);
  }

  public Enumeration getNodes(){
    return new ComponentEnumeration(container);
  }

  public String toString(){
    ComponentEnumeration enum = (ComponentEnumeration) getNodes();
    String res = "";
    ASTComponentType elem;
    while(enum.hasMoreElements()){
      elem = (ASTComponentType) enum.nextElement();
      res+= elem.toString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }

  public String toPrologString(){
    ComponentEnumeration enum = (ComponentEnumeration) getNodes();
    String res = "";
    ASTComponentType elem;
    while(enum.hasMoreElements()){
      elem = (ASTComponentType) enum.nextElement();
      res+= elem.toPrologString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }

  protected void setParents(Node n){
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    ComponentEnumeration enum = (ComponentEnumeration) getNodes();
    ASTComponentType elem;
    while(enum.hasMoreElements()){
      elem = (ASTComponentType) enum.nextElement();
      elem.accept(v, data);

      if(enum.hasMoreElements())
        v.visitIntraList(this);


    }
    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}
