package coco.parser.ast;
import coco.parser.symtab.SymtabDuplicateEntryException;
/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public interface ASTComponentType extends Node, ASTCompilationUnit{
  public ASTComponentTypeBody getBody();
  public ASTIdentifier getID();
  public ASTIdentifierList getInterfaceList();
  public String getName();

  public void setInterfaceList(ASTIdentifierList identifierList);
  public void setId(ASTIdentifier identifier);
  public void setBody(ASTComponentTypeBody body);
  public Object accept(AbstractASTVisitor v, Object data);


    /**
   * Adds a port to a port container. The port is registered with the symtab for
   * local symbols of the respective port container.
   * If a port with the same signature already exists in this container,
   * a SymtabDuplicateEntryException is thrown.
   *
   * @param p the port which is to be added
   */
  public void addPort(ASTPort p) throws SymtabDuplicateEntryException;

  /**
   * Removes a port from this port container. The port is de-registered from the
   * local symbols symtab of this port container.If the port is not in this port
   * container, the port container remains unchanged
   *
   * @param p the port which is to be removed
   */
  public void removePort(ASTPort p);


}