package coco.parser.ast;
import coco.parser.ParseException;
import coco.utils.*;
import coco.parser.symtab.*;
import java.util.Enumeration;
/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public class ASTConnector extends GenericNode implements ASTPropertyContainer{


  private ASTIdentifier connId;
  private ASTConnectorBody body = null;;
  private ASTPortAccessList ports;
  private ASTPropertySetReferenceList propSets = null;

  public ASTConnector(ASTIdentifier id, ASTPortAccessList p, ASTPropertySetReferenceList ps, ASTConnectorBody b){
    Assert.require(id);
    connId = id;
    Assert.require(p);
    ports = p;
    ports.setParent(this);


    body = b;
    if(body != null)
      body.setParent(this);
    propSets = ps;
    if(propSets != null)
      propSets.setParent(this);
  }



  public ASTIdentifier getConnectorID(){
    return connId;
  }

  public ASTIdentifier getContainerID(){
    return getConnectorID();
  }

 public ASTPortAccessList getPorts(){
  return ports;
 }


 /**
  * setPorts sets the portslist of the astConnector.
   * @param p list of port references
  */


 public void  setPorts(ASTPortAccessList p){
  Assert.require(ports);
  this.ports = ports;
  ports.setParent(this);
 }




 public ASTConnectorBody getBody() {
 return body;
 }

  /**
  *  setBody sets the ConnectorBody of the astConnector.
   * @param body instance of ASTConnectorBody.
  */


 public void setBody(ASTConnectorBody body) {

 this.body = body;
 if(body != null)
  body.setParent(this);
 }


 public String toString(){
    String res = "\nconnector " + connId.toString() + "(";
    res+= ports.toString() +")";
    if(propSets != null)
      res += " has " + propSets.toString();
    if(body != null)
      res += body.toString() +"\n";
    res += ";";
    return res;
  }
 public String toPrologString(){
  String res = "connector(" + connId.toPrologString() + ").\n";
  res += "connected_ports(" + connId.toPrologString() + ", " + ports.toPrologString() + ").\n";
  if(body != null)
    res += body.toPrologString();
  return res;
 }
 public void addProperty(ASTProperty p) throws SymtabDuplicateEntryException{
    Assert.require(p);
    Assert.require(body);
    ASTPropertyList pl = body.getPropertyList();
    if(pl != null)
      pl.addNode(p);
    else{
      pl = new ASTPropertyList(p);
      body.setPropertyList(pl);
    }
  }

  public void removeProperty(ASTProperty p){

    Assert.require(body);
    // there must be a property list, if somethings is gonna be removed.
    ASTPropertyList pl = body.getPropertyList();
    Assert.require(pl);


    pl.removeNode(p);

  }


   public ASTPropertyList getPropertyList(){
    Assert.require(body);
    return body.getPropertyList();
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the sub nodes
    Assert.require(connId);
    connId.accept(v, data);

    Assert.require(ports);
    ports.accept(v, data);
    if(propSets != null)
      propSets.accept(v, data);
    if(body != null)
      body.accept(v, data);

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }


}
