package coco.parser.ast;
import java.util.*;
import coco.parser.ast.enums.*;
import coco.utils.*;


/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTConnectorList extends GenericContainerNode {

  public ASTConnectorList(ASTConnector c) {
    super();
    addNode(c);
  }

  public void addNode(Node n){
    Assert.require(n);
    Assert.require(n instanceof ASTConnector);
    container.add(n);
  }

  public Enumeration getNodes(){
    return new ConnectorEnumeration(container);
  }

  public String toString(){
    ConnectorEnumeration enum = (ConnectorEnumeration) getNodes();
    String res = "";
    ASTConnector elem;
    while(enum.hasMoreElements()){
      elem = (ASTConnector) enum.nextElement();
      res+= elem.toString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }

  public String toPrologString(){
    ConnectorEnumeration enum = (ConnectorEnumeration) getNodes();
    String res = "";
    ASTConnector elem;
    while(enum.hasMoreElements()){
      elem = (ASTConnector) enum.nextElement();
      res+= elem.toPrologString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }

  protected void setParents(Node n){
  }


  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit subnodes
    ConnectorEnumeration enum = (ConnectorEnumeration) getNodes();
    ASTConnector elem;
    while(enum.hasMoreElements()){
      elem = (ASTConnector) enum.nextElement();
      elem.accept(v, data);
      if(enum.hasMoreElements())
        v.visitIntraList(this);



    }

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}
