package coco.parser.ast;
import coco.utils.*;

import coco.parser.symtab.*;

/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public class ASTData extends ASTPort{


  public static ASTData createNew(String name) {
    ASTIdentifier id = null;
    if((name == null) || (name.length()== 0))
      id = new ASTIdentifier("newDataPort");
    else
      id = new ASTIdentifier(name);

    ASTDataTypeIdentifier t  = new ASTBaseTypeIdentifier("int");
    ASTData res = new ASTData(t, id, null, null);
    res.setDataType(t);
    res.setDirection(ASTPort.IN_DIR);
    return res;
  }

  public ASTData(ASTDataTypeIdentifier t, ASTIdentifier i, ASTPropertySetReferenceList ps, ASTPortBody b){
    Assert.require(t);
    Assert.require(i);

    this.dataType = t;

    this.portId = i;
    propSets = ps;
    if(propSets != null)
      propSets.setParent(this);

    if(b != null)
      body = b;
    else
      body = new ASTPortBody(null);

    Assert.require(body);

    body.setParent(this);


  }

/**
   * Copy all data from another generic node into this generic node
   * resync the syntax tree. The parameter src must not be null and must be of type ASTData
   *
   * @param src   the source node
   */
  public void copyFrom(GenericNode src) throws SymtabDuplicateEntryException{
    super.copyFrom(src);
    Assert.require(src instanceof ASTData);
    ASTData srcData = (ASTData)src;
    String origName = this.getPortId().getID();
    String newName = srcData.getPortId().getID();

    // copy the actual content
    this.setPortId(srcData.getPortId());
    this.setDataType(srcData.getDataType());
    this.setDirection(srcData.getDirection());
    this.setBody(srcData.getBody());

    // check if the name of the port has changed
    // if so, re-register with the symtab
    if (!origName.equals(newName))
     {
        if(this.parent instanceof ASTComponent){
          SymtabService.remove((ASTComponent)parent, SymtabService.S_PORT, origName);
          SymtabService.registerLocal((ASTComponent)parent, SymtabService.S_PORT, newName, this);
        }else if(this.parent instanceof ASTInterface){
          SymtabService.remove((ASTInterface)parent, SymtabService.S_PORT, origName);
          SymtabService.registerLocal((ASTInterface)parent, SymtabService.S_PORT, newName, this);
        }
      }
}






  public String toString(){
    String res = "";
    if(beforeComments != null)
      res = beforeComments.toString();

    if(modifiers!=null)
      res = modifiers.toString();
    res += direction + dataType.toString() + " ";
    res += portId.toString();
   if(propSets != null)
      res +=" has " + propSets.toString();

    if(body!=null){

      res +=body.toString();
    }

    res+=";\n";

    if(afterComments != null)
      res += afterComments.toString();

    return res;

  }

  public String toPrologString(){
    Assert.require(parent);
    String res = "data(";
    res += parent.getID().toPrologString() + ", ";
    if(modifiers != null)
      res += modifiers.toPrologString() + ", ";

    res += "'" + direction + "', ";

    // probably needed, currently ommitted
    //    res += dataType.toPrologString() +", ";

    res += portId.toPrologString() + ").";
    return res;
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }

    // visit the subnodes
    if(modifiers != null)
      modifiers.accept(v, data);
    Assert.require(dataType);
    dataType.accept(v, data);

    Assert.require(portId);
    portId.accept(v, data);
   if(propSets != null) 
      propSets.accept(v, data);

    if(body != null)
      body.accept(v, data);

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }


}
