package coco.parser.ast;
import java.util.Enumeration;

import coco.parser.symtab.*;
import coco.parser.ParseException;
import coco.utils.*;
import coco.parser.ast.enums.IdentifierEnumeration;
/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public class ASTDataType extends GenericNode {
  private ASTIdentifier packetId;
  private ASTGenericTypeIdentifierList genTypeList = null;
  private ASTFieldList fieldList = null;
//  private ASTExternalTypeName targetType = null;
  private ASTBaseTypeReference baseTypeRef = null;
  private boolean baseType = false;


  public ASTDataType(ASTIdentifier id, ASTBaseTypeReference btr, ASTGenericTypeIdentifierList tlist, ASTFieldList fields) {
    Assert.require(id);
    packetId = id;
    baseTypeRef = btr;
    fieldList = fields;
    genTypeList = tlist;
  /*  if(tType != null){
      targetType = tType;
      baseType = true;
    }
    */
  }

  public void register() throws SymtabDuplicateEntryException {

      SymtabService.register(this);

      if (fieldList != null) {
        for (Enumeration enum = fieldList.getNodes(); enum.hasMoreElements(); ) {
          ASTField f = (ASTField)enum.nextElement();
          SymtabService.registerLocal(this, SymtabService.S_FIELD, f.getFieldId().getID(), f);
        }
      }
  }

  public void unregister() {
  SymtabService.remove(this);
  }

 public static void modifyASTDataType(ASTDataType original,ASTDataType modified) {
/*  original.getDataTypeIdentifier().setID(modified.getDataTypeIdentifier().getID());
  if (modified.isBaseType()) {
   original.setTargetTypeIdentifier(modified.getTargetTypeIdentifier());
   original.setBaseType(true);
   original.setBasetypeIdentifier(null);
   original.setFieldList(null);
   }
  else
   if (!modified.isBaseType()) {
       if (modified.getBasetypeIdentifier()!= null) {
            if (original.getBasetypeIdentifier()!= null)
                original.getBasetypeIdentifier().setID(modified.getBasetypeIdentifier().getID());
            else
                original.setBasetypeIdentifier(new ASTIdentifier(modified.getBasetypeIdentifier().getID()));
        }
        else {
              original.setBasetypeIdentifier(null);
          }
       original.setFieldList(modified.getFieldList());
       original.setBaseType(false);
       original.setTargetTypeIdentifier(null);
   }
   System.out.println("in astpacket"+original.toString());*/
  }



  public ASTIdentifier getDataTypeIdentifier(){
    return packetId;
  }

  public ASTIdentifier getBasetypeIdentifier(){
   if (baseTypeRef != null) {
     return baseTypeRef.getTypeId();
   } else {
     return(null);
   }
  }

  public void setBasetypeIdentifier(ASTIdentifier baseTypeId){
   baseTypeRef.setTypeId(baseTypeId);
  }

/*  public boolean isBaseType(){
   return baseType;
  }

  public void setBaseType(boolean baseType){
   this.baseType = baseType;
  }


  public ASTExternalTypeName getTargetTypeIdentifier(){
   return  targetType;
  }

   public void setTargetTypeIdentifier(ASTExternalTypeName ttype){
   targetType = ttype;
  }
*/
  public ASTFieldList getFieldList() {
    return(fieldList);
  }

  public void setFieldList(ASTFieldList fieldList) {
    this.fieldList = fieldList;
  }

  public ASTGenericTypeIdentifierList getGenTypeList() {
    return(genTypeList);
  }

  public String toString(){
    String res = "packet " + packetId.toString() ;
    if(genTypeList != null)
      res += "[ " + genTypeList.toString() + " ]";
    if(baseTypeRef != null){
     res+=" extends " + baseTypeRef.toString();
    }
    if(fieldList != null){
      res += "{\n" + fieldList.toString() + "\n}";
    }
    /*else if(targetType != null){
     res += " := " + targetType.toString();
    }*/
    res += ";\n";
    return res;
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit subnodes
    Assert.require(packetId);
    packetId.accept(v, data);
/*    if(genTypeList != null)
      genTypeList.accept(v, data);
*/
    if(baseTypeRef != null)
      baseTypeRef.accept(v, data);

    if(fieldList != null)
      fieldList.accept(v, data);
    /*else if (targetType != null)
      targetType.accept(v, data);
    */

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}
