package coco.parser.ast;
import java.util.*;
import coco.parser.ast.enums.*;
import coco.utils.*;
import coco.parser.ParseException;
import coco.parser.symtab.*;


/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTDataTypeList extends GenericContainerNode implements ASTCompilationUnit{

  public static final String UNIT_TYPE = "packets";

  private java.io.File sourceFile = null;
  public ASTDataTypeList(ASTDataType p){
    super();
    addNode(p);
  }


  /**
   * Creates a template for a compilation unit with the given name
   *
   * @param name   the name of the unit
   * @return the newly created compilation unit template
   */
  public static ASTCompilationUnit createTemplateFromName(String name) throws coco.parser.ParseException{
    String text = "packet " + name + " := test; ";
    coco.parser.CoCoParser parser = coco.parser.ParserService.getCoCoParser(new java.io.StringReader(text));
    return parser.compilationUnit();
  }


  public void addNode(Node n){
    Assert.require(n instanceof ASTDataType);
    container.add(n);
  }

  public String getUnitName(){

    Assert.require(sourceFile);

    return sourceFile.getName();
  }

  public String getUnitType(){
      return(UNIT_TYPE);
  }



  public void register() throws SymtabDuplicateEntryException{
    DataTypeEnumeration enum = (DataTypeEnumeration)getNodes();
    ASTDataType elem;
    while(enum.hasMoreElements()){
      elem = (ASTDataType)enum.nextElement();
      try {
        elem.register();
      } catch(SymtabDuplicateEntryException e){
        unregister();
        System.out.println(e.toString());
        throw(e);
      }
    }
  }

  public void unregister(){
    DataTypeEnumeration enum = (DataTypeEnumeration)getNodes();
    ASTDataType elem;
    while(enum.hasMoreElements()){
      elem = (ASTDataType)enum.nextElement();
      SymtabService.remove(elem);
    }
  }




  public Enumeration getNodes(){
    return new DataTypeEnumeration(container);
  }

  public void setSourceFile(java.io.File f){
    Assert.require(f);
    sourceFile = f;
  }
  public java.io.File getSourceFile(){
    return sourceFile;
  }





  public String toString(){
    DataTypeEnumeration enum = (DataTypeEnumeration) getNodes();
    String res = "";
    ASTDataType  p;
    while(enum.hasMoreElements()){
      p = (ASTDataType) enum.nextElement();
      res+= p.toString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }
  protected void setParents(Node n){
    Assert.require(n);
// todo: possibly type check needed

    DataTypeEnumeration enum = (DataTypeEnumeration) getNodes();
    ASTDataType elem;
    while(enum.hasMoreElements()){
      elem = (ASTDataType) enum.nextElement();
      elem.setParent(n);
    }
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the sub nodes
    DataTypeEnumeration enum = (DataTypeEnumeration) getNodes();
    ASTDataType elem;
    while(enum.hasMoreElements()){
      elem = (ASTDataType) enum.nextElement();
      elem.accept(v, data);

      if(enum.hasMoreElements())
        v.visitIntraList(this);



    }

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}
