package coco.parser.ast;
import coco.utils.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTGenericTypeParameter extends GenericNode {
  private ASTIdentifier typeId = null;
  private ASTIdentifier paramId = null;

  private ASTComponentType parent = null;

  public ASTGenericTypeParameter(ASTIdentifier t_id, ASTIdentifier p_id) {
    Assert.require(t_id);
    Assert.require(p_id);
    typeId = t_id;
    paramId = p_id;
  }

  public ASTIdentifier getTypeID(){
    return typeId;
  }

  public ASTIdentifier getParamID(){
   return paramId;
  }


  public void setParent(Node p){
   Assert.require(p);
   Assert.require(p instanceof ASTComponentType);
   parent = (ASTComponentType)p;
  }

  public Node getParent() {
    return parent;
  }



  public String toString(){
    return typeId.toString() + " " + paramId.toString();
  }

  public String toPrologString(){
    String res = "genericTypeParameter(";
    // bye bye, law of demeter
    res += parent.getID().toPrologString() +", ";
    res += typeId.toPrologString() +", ";
    res += paramId.toPrologString() + ").";
    return res;
  }


  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the subnodes
    paramId.accept(v, data);
    typeId.accept(v, data);

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }



}