package coco.parser.ast;
import java.util.*;
import coco.parser.ast.enums.*;
import coco.utils.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTGenericTypeParameterList extends GenericContainerNode {

  public ASTGenericTypeParameterList(ASTGenericTypeParameter p){
    super();
    addNode(p);
  }

  public void addNode(Node n){
    Assert.require(n instanceof ASTGenericTypeParameter);
    container.add(n);
  }

  public Enumeration getNodes(){
    return new GenericTypeParameterEnumeration(container);
  }

  public String toString(){
    GenericTypeParameterEnumeration enum = (GenericTypeParameterEnumeration) getNodes();
    String res = "[";
    ASTGenericTypeParameter elem;
    while(enum.hasMoreElements()){
      elem = (ASTGenericTypeParameter) enum.nextElement();
      res+= elem.toString();
      if(enum.hasMoreElements())
        res+=", ";
    }
    res += "]";
    return res;
  }

  public String toPrologString(){
    GenericTypeParameterEnumeration enum = (GenericTypeParameterEnumeration) getNodes();
    String res = "";
    ASTGenericTypeParameter elem;
    while(enum.hasMoreElements()){
      elem = (ASTGenericTypeParameter) enum.nextElement();
      res+= elem.toPrologString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }


  public void setParent(Node n){
   Assert.require(n);
   Assert.require(n instanceof ASTComponentType);
   setParents(n);
  }

  protected void setParents(Node n) {
     Assert.require(n);
    // only ASTComponents can be parent of ports
    Assert.require(n instanceof ASTComponentType);
    GenericTypeParameterEnumeration enum = (GenericTypeParameterEnumeration) getNodes();
    ASTGenericTypeParameter p;
    while(enum.hasMoreElements()){
      p = (ASTGenericTypeParameter) enum.nextElement();
      p.setParent(n);
    }

  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the elements in the list
    GenericTypeParameterEnumeration enum = (GenericTypeParameterEnumeration) getNodes();
    ASTGenericTypeParameter elem;
    while(enum.hasMoreElements()){
      elem = (ASTGenericTypeParameter) enum.nextElement();
      elem.accept(v, data);
    }
    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}