package coco.parser.ast;
import coco.utils.*;
import coco.parser.symtab.*;

/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public class ASTHandler extends ASTPort{
  private ASTInvocationSignature handlerSig;


  public ASTHandler(ASTDataTypeIdentifier t, ASTIdentifier i, ASTInvocationSignature sig, ASTPropertySetReferenceList ps, ASTPortBody b) {
    Assert.require(t);
    this.dataType = t;
    Assert.require(i);
    portId = i;

    Assert.require(sig);
    handlerSig = sig;

    propSets = ps;
    if(propSets != null)
      propSets.setParent(this);

    if(b != null)
      body = b;
    else
      body = new ASTPortBody(null);

    Assert.require(body);
    body.setParent(this);
  }


 /**
   * Copy all data from another generic node into this generic node
   * resync the syntax tree. The parameter src must not be null and must be of type ASTHandler
   *
   * @param src   the source node
   */
  public void copyFrom(GenericNode src) throws SymtabDuplicateEntryException{
    super.copyFrom(src);
    Assert.require(src instanceof ASTHandler);
    ASTHandler srcHandler = (ASTHandler)src;
    String origName = getPortId().getID();
    String newName = srcHandler.getPortId().getID();

    // copy the actual content
    this.setHandlerSig(srcHandler.getHandlerSig());
    this.setBody(srcHandler.getBody());



  // check if the name of the port has changed
  // if so, re-register with the symtab
  if (!origName.equals(newName))
   {
      if(this.parent instanceof ASTComponent){
        SymtabService.remove((ASTComponent)parent, SymtabService.S_PORT, origName);
        SymtabService.registerLocal((ASTComponent)parent, SymtabService.S_PORT, newName, this);
      }else if(this.parent instanceof ASTInterface){
        SymtabService.remove((ASTInterface)parent, SymtabService.S_PORT, origName);
        SymtabService.registerLocal((ASTInterface)parent, SymtabService.S_PORT, newName, this);
      }
    }
}




  public ASTInvocationSignature getHandlerSig() {
    return (handlerSig);
  }

  public void setHandlerSig(ASTInvocationSignature handlerSig) {
   Assert.require(handlerSig);
   this.handlerSig = handlerSig;
  }

  public String toString(){
    String res = "";
    if(beforeComments != null)
      res = beforeComments.toString();
    if(modifiers != null)
      res += modifiers.toString() +" ";
    res += ASTPort.IN_DIR;

    if(dataType != null)
      res += " " + dataType.toString()+ " ";
    else
      res += " void ";
    res += portId.toString();
    res += handlerSig.toString();
    if(propSets != null)
      res +=" has " + propSets.toString();

    if(body!=null)
      res +=body.toString();
    res += "\n";

    if(afterComments != null)
      res += afterComments.toString();

    return res;
  }

  public String toPrologString(){
    String res = "handler(";
    Assert.require(parent);

    res += parent.getID().toPrologString() + ", ";
    if(modifiers != null)
      res += modifiers.toPrologString() +", ";
    res += handlerSig.toPrologString();
    res += ").";
    return res;
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }

    // visit the subnodes
    if(modifiers != null)
      modifiers.accept(v, data);
    Assert.require(dataType);
    dataType.accept(v, data);

    Assert.require(portId);
    portId.accept(v, data);

    Assert.require(handlerSig);
    handlerSig.accept(v, data);

   if(propSets != null)
      propSets.accept(v, data);

    if(body != null)
      body.accept(v, data);

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }


}
