package coco.parser.ast;
import coco.utils.*;


/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTInstance extends GenericNode {
  private ASTIdentifier varId;
  private ASTComponentTypeIdentifier typeId;
  private ASTRoleIdentifier roleId = null;
  private ASTComponentType parent = null;

  public ASTInstance(ASTIdentifier v, ASTComponentTypeIdentifier t, ASTRoleIdentifier r) {
    Assert.require(v);
    Assert.require(t);

    varId = v;
    typeId = t;
    roleId = r;
  }

  public ASTRoleIdentifier getRoleId(){
    return roleId;
  }

  public void setRoleId(ASTRoleIdentifier r){
    roleId = r;
  }
  public ASTIdentifier getVarId(){
    return varId;
  }
  public ASTIdentifier getTypeId(){
    return typeId;
  }

 public void setParent(Node n){
  Assert.require(n);
  Assert.require(n instanceof ASTComponentType);

  parent = (ASTComponentType) n;
 }

 public Node getParent(){
  return parent;
 }

  public String toString(){
    String res = "";
    if(beforeComments != null)
      res +=beforeComments.toString();
    res +=   typeId.toString() + " "+ varId.toString();
    if(roleId != null)
      res += " is " + roleId.toString();

    res += ";";

    if(afterComments != null)
      res +=afterComments.toString();

    return res;
  }


 public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the subnodes
    typeId.accept(v, data);
    varId.accept(v, data);
    if(roleId != null)
      roleId.accept(v, data);

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}