package coco.parser.ast;
import java.util.*;
import coco.parser.ast.enums.*;
import coco.utils.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTInstanceList extends GenericContainerNode {

  public ASTInstanceList(ASTInstance i){
    super();
    if(i != null)
      addNode(i);
  }

  public void addNode(Node n){
    Assert.require(n instanceof ASTInstance);
    container.add(n);
  }

  public Enumeration getNodes(){
    return new InstanceEnumeration(container);
  }

  public String toString(){
    InstanceEnumeration enum = (InstanceEnumeration) getNodes();
    String res = "\n\n";
    ASTInstance i;
    while(enum.hasMoreElements()){
      i = (ASTInstance) enum.nextElement();
      res+= i.toString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }

  protected void setParents(Node n){
    Assert.require(n);
    Assert.require(n instanceof ASTComponentType);
    InstanceEnumeration enum = (InstanceEnumeration) getNodes();
    ASTInstance elem;
    while(enum.hasMoreElements()){
      elem = (ASTInstance) enum.nextElement();
      elem.setParent(n);
    }
  }


 public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the subnodes
    InstanceEnumeration enum = (InstanceEnumeration) getNodes();
    ASTInstance i;
    while(enum.hasMoreElements()){
      i = (ASTInstance) enum.nextElement();
      i.accept(v, data);

      if(enum.hasMoreElements())
        v.visitIntraList(this);



    }
    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}
