
package coco.parser.ast;
import coco.utils.*;
import coco.parser.ast.enums.*;
/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTInterfaceBody extends GenericNode implements ASTComponentTypeBody{
  private ASTPortList portList;
  private ASTAbstractPropertyList propList = null;
  private ASTInterface parent = null;
  private ASTRoleList roles = null;
  private ASTInstanceList instances = null;
  private ASTConnectorList connectors = null;


  public ASTInterfaceBody(
    ASTPortList ports,
    ASTAbstractPropertyList pl,
    ASTRoleList rl,
    ASTInstanceList il,
    ASTConnectorList cl
    ) {
    portList = ports;
    propList = pl;
    roles = rl;
    instances = il;
    connectors = cl;
  }

  public void setParent(Node n){
    Assert.require(n instanceof ASTInterface);
    parent = (ASTInterface)n;
    if(portList != null)
      portList.setParent(n);
    if(propList != null){
      propList.setParent(n);
    }
    if(roles != null)
      roles.setParent(n);
    if(instances != null)
      instances.setParent(n);
    if(connectors != null)
      connectors.setParent(n);
  }


  public ASTPortList getPortList(){
    return portList;
  }

  public void setPortList(ASTPortList pl) {
    this.portList = pl;
    Assert.require(parent);
    if(portList != null)
      portList.setParent(parent);

  }

  public ASTInstanceList getInstances(){
    return instances;
  }

  public void setInstances(ASTInstanceList il){
    instances = il;
    if(instances != null)
      instances.setParent(parent);
  }

  public ASTConnectorList getConnectors(){
    return connectors;
  }

  public void setConnectors(ASTConnectorList cl){
    connectors = cl;
    if(connectors != null)
      connectors.setParent(parent);
  }

  public ASTRoleList getRoles(){
    return roles;
  }

  public void setRoles(ASTRoleList rl){
    roles = rl;
    roles.setParent(parent);
  }

  public void setPropertyList(ASTPropertyList pl) {
    propList = (ASTAbstractPropertyList) pl;
    Assert.require(parent);
    if(propList != null)
      propList.setParent(parent);

  }
    public void setPropertyList(ASTAbstractPropertyList propList) {
  this.propList = propList;
  }

  public ASTPropertyList getPropertyList(){
    return propList;
  }


  public String toString(){
    String res = "{\n";
    if(portList != null)
      res += portList.toString();
    if(propList !=null)
      res += propList.toString();
    if(roles != null)
      res += roles.toString();
    if(instances != null)
      res += instances.toString();
    if(connectors != null)
      res += connectors.toString();
     res += "\n}";
    return res;
  }
  public String toPrologString(){
    String res = "";

    // translate the ports of this component to Prolog facts
    if(portList != null)
      res += portList.toPrologString();

    // if there are properties, translate them into Prolog facts
    if(propList != null){
      res += propList.toPrologString();
    }

    return res;
  }


  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the sub nodes
    if(portList != null)
      portList.accept(v, data);
    if(propList != null)
      propList.accept(v, data);

     if(roles != null)
      roles.accept(v,data);
    if(instances != null)
      instances.accept(v,data);
    if(connectors != null)
      connectors.accept(v,data);

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }
}