package coco.parser.ast;

import coco.utils.*;

import coco.parser.symtab.*;

/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public abstract class ASTPort extends GenericNode{
  protected ASTPortBody body= null;
  protected String direction;
  // Constants for data direction
  public static final String OUT_DIR = "out";
  public static final String IN_DIR = "in";
  public static final String INOUT_DIR = "inout";
  protected ASTPropertySetReferenceList propSets = null;

  protected ASTComponentType parent = null;
  protected  ASTPortModifiers modifiers = null;
  protected ASTDataTypeIdentifier dataType = null;
  protected ASTIdentifier portId = null;
 /**
   * setModifiers sets the modifiers of a port
   * @param m an instance of ASTPortModifiers
   */
  public void setModifiers(ASTPortModifiers m){
    modifiers = m;
  }

 /**
   * setModifiers sets the modifiers of a port
   * @return the modifiers
   */
  public ASTPortModifiers getModifiers(){
    return modifiers;
  }

  public static GenericNode parse(String text) throws coco.parser.ParseException{
    Assert.require(text);

    coco.parser.CoCoParser parser =
      coco.parser.ParserService.getCoCoParser(new java.io.StringReader(text));

    ASTPort p = parser.port();
    return p;
  }

  public ASTIdentifier getPortId(){
    Assert.require(portId);
    return portId;

  }

 public void setPortId(ASTIdentifier id) {
    Assert.require(id);
    portId = id;
  }

  /**
   * Copy all data from another generic node into this generic node
   * resync the syntax tree. The parameter src must not be null and must be of type ASTPort
   *
   * @param src   the source node
   */
  public void copyFrom(GenericNode src) throws SymtabDuplicateEntryException{
    super.copyFrom(src);
    Assert.require(src instanceof ASTPort);
    ASTPort srcPort = (ASTPort)src;
    this.setModifiers(srcPort.getModifiers());
  }

  public String getDirection(){
   return direction;
  }
  public void setDirection(String dir){
    Assert.require(dir);
    direction = dir;
  }

 /**
   * setBody sets the body of a port
   * @param body an instance of ASTPortBody
   */


  public void setBody(ASTPortBody body) {
  Assert.require(body);
  this.body= body;
  }

/**
 * getBody returns the body of the port.
 * @return body an instance of ASTPortBody
 */
  public ASTPortBody getBody(){
    return body;
  }



  public void addProperty(ASTProperty p){
    Assert.require(p);
    Assert.require(body);
    ASTPropertyList pl = body.getPropertyList();
    if(pl != null)
      pl.addNode(p);
    else{
      pl = new ASTPropertyList(p);
      body.setPropertyList(pl);
    }
  }

  public ASTDataTypeIdentifier getDataType() {
    return(dataType);
  }

  public void setDataType(ASTDataTypeIdentifier type) {
    Assert.require(type);
    dataType = type;
  }

   public ASTPropertyList getPropertyList(){
    if(body != null)
      return body.getPropertyList();
    else
      return null;
  }

  public void setParent(Node n){
    Assert.require(n instanceof ASTComponentType);
    parent = (ASTComponentType) n;
  }
  // has to be implemented if needed
  public Node getParent(){
    return parent;
  }

  public abstract Object accept(AbstractASTVisitor v, Object data);
}