package coco.parser.ast;
import java.util.*;
import coco.parser.ast.enums.*;
import coco.utils.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTPortAccessList extends GenericContainerNode {
  ASTConnector parent = null;
  public ASTPortAccessList(ASTPortAccess acc){
    super();
    addNode(acc);
  }

  public void addNode(Node n){
    Assert.require(n instanceof ASTPortAccess);
    container.add(n);
  }

  public Enumeration getNodes(){
    return new PortAccessEnumeration(container);
  }

  public String toString(){
    PortAccessEnumeration enum = (PortAccessEnumeration) getNodes();
    String res = "";
    ASTPortAccess acc;
    while(enum.hasMoreElements()){
      acc = (ASTPortAccess) enum.nextElement();
      res+= acc.toString();
      if(enum.hasMoreElements())
        res+=", ";
    }
    return res;
  }
  public void setParent(Node n){
    Assert.require(n);
    Assert.require(n instanceof ASTConnector);
    parent = (ASTConnector)n;
    this.setParents(n);
  }

  public Node getParent(){
    return parent;
  }



  protected void setParents(Node n){
    Assert.require(n);
// todo: possibly type check needed

    PortAccessEnumeration enum = (PortAccessEnumeration) getNodes();
    ASTPortAccess elem;
    while(enum.hasMoreElements()){
      elem = (ASTPortAccess) enum.nextElement();
      elem.setParent(n);
    }
  }


  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    PortAccessEnumeration enum = (PortAccessEnumeration) getNodes();
    ASTPortAccess acc;
    while(enum.hasMoreElements()){
      acc = (ASTPortAccess) enum.nextElement();
      acc.accept(v, data);
      if(enum.hasMoreElements())
        v.visitIntraList(this);



    }
    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }



}
