package coco.parser.ast;
import java.util.*;
import coco.parser.ast.enums.*;
import coco.utils.*;


/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public class ASTPortList extends GenericContainerNode {

  private Node parent = null;
  public ASTPortList(ASTPort p) {
    super();

    if(p != null)
      addNode(p);
  }


  public void addNode(Node n){
    Assert.require(n instanceof ASTPort);
    container.add(n);
    if(parent != null)
      n.setParent(parent);
  }

  public Enumeration getNodes(){
    return new PortEnumeration(container);
  }

  public String toString(){
    PortEnumeration enum = (PortEnumeration) getNodes();
    String res = "";
    if(beforeComments != null)
      res += beforeComments.toString();

    res += "";
    ASTPort p;
    while(enum.hasMoreElements()){
      p = (ASTPort) enum.nextElement();
      res+= p.toString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }

  public void setParent(Node n){
    Assert.require(n);
    parent = n;

    // set parent for all child nodes
    setParents(n);
  }
  public Node getParent(){
    return parent;
  }


  protected void setParents(Node n){
    Assert.require(n);
    // only ASTComponents can be parent of ports
    Assert.require(n instanceof ASTComponentType);
    PortEnumeration enum = (PortEnumeration) getNodes();
    ASTPort p;
    while(enum.hasMoreElements()){
      p = (ASTPort) enum.nextElement();
      p.setParent(n);
    }
  }


  public String toPrologString(){
    PortEnumeration enum = (PortEnumeration) getNodes();
    ASTPort p;
    String res = "% ports\n";
    while(enum.hasMoreElements()){
      p = (ASTPort) enum.nextElement();
      res+= p.toPrologString();
      res+="\n";
    }
    return res;
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the subnodes
    PortEnumeration enum = (PortEnumeration) getNodes();
    ASTPort p;
    while(enum.hasMoreElements()){
      p = (ASTPort) enum.nextElement();
      p.accept(v, data);
       if(enum.hasMoreElements())
        v.visitIntraList(this);



    }

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }


}