package coco.parser.ast;
import coco.utils.*;

/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public class ASTPortModifiers extends GenericNode {

  private int modifier[] = new int[2];

  public final static int IMPLICITELY = 0;
  public final static int MANDATORY = 1;
  public final static int OPTIONAL = 2;


  public final static int PUBLIC = 3;
  public final static int PRIVATE = 4;
  public final static int UNKNOWN = -1;

  private static final String MODIFIERS[] =
    {"", "mandatory", "optional", "public", "private"};

  public ASTPortModifiers(int m1, int m2) {
    modifier[0] = m1;
    modifier[1] = m2;
  }


  private String mapModifier(int m){
    Assert.require(m < MODIFIERS.length);
    return MODIFIERS[m];
  }

  /**
 * setModifiers sets the modifiers of any ASTPort.
 * @param m1 a String
 * @param m2 a String (can be null)
 */

 public void setModifiers(int m1, int m2) {
  this.modifier[0] = m1;
  this.modifier[1] = m2;
 }

 /**
  * getFirst returns the First Modifier.
  * @return a String with value of modifier1
  */

  public int getFirst() {
  return modifier[0];
  }

   /**
  * getSecond returns the Second Modifier.
  * @return a String with value of modifier2
  */


  public int getSecond() {
  return modifier[1];
  }

 /**
  * getFirst returns the First Modifier.
  * @return a String with value of modifier1
  */

  public String getFirstModifierString() {
  return mapModifier(modifier[0]);
  }

   /**
  * getSecond returns the Second Modifier.
  * @return a String with value of modifier2
  */


  public String getSecondModifierString() {
  return mapModifier(modifier[1]);
  }



  public String toString(){
    String res = "";
    res += mapModifier(modifier[0]) + " ";
    res += mapModifier(modifier[1]) + " ";
    return res;
  }
  public String toPrologString(){
    String res = "[";
    res += "'" + mapModifier(modifier[0]) + "', ";
    res += "'" + mapModifier(modifier[1]) + "'" ;
    res += "]";
    return res;
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }

    // visit the subnodes
    // no subnodes
    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }


}