package coco.parser.ast;

import coco.utils.*;


/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public class ASTProperty extends GenericNode {

  public final static int IMPLICITELY_MANDATORY = 0;
  public final static int MANDATORY = 1;
  public final static int OPTIONAL = 2;

  private ASTPropertyIdentifier propId;
  private ASTConstExpression initExpr = null;

  private ASTPropertyContainer parent = null;

  private int mandatory = IMPLICITELY_MANDATORY;

  public ASTProperty(ASTPropertyIdentifier i, ASTConstExpression e, int m) {
    Assert.require(i);
    propId = i;

    mandatory = m;
    initExpr = e;
  }

  public static ASTProperty createNew(String name){
    ASTPropertyIdentifier id = null;

    ASTDesignator des = null;
    if((name == null) || (name.length()== 0))
      des = new ASTDesignator("newProperty");
    else
      des = new ASTDesignator(name);

    id = new ASTPropertyIdentifier(des);
    ASTConstExpression value  = new ASTConstExpression(new ASTStringLiteral("\"property value\""));
    return new ASTProperty(id, value, MANDATORY);
  }


  public ASTIdentifier getPropertyId(){
    return propId.getID();
  }

  /**
   * This method sets the ASTConstExpression to a specified expression. The value of the parameter
   * can be null also.
   * @param expr-  the ASTConstExpression
   */
  public void setInitExpr(ASTConstExpression expr) {
  initExpr = expr;
  }

  public ASTConstExpression getInitExpr(){
    return(initExpr);
  }

  public int getVisibility() {
  return mandatory;
  }
  public String getVisibilityString(){
    return (mandatory == this.OPTIONAL? "optional": "mandatory");

  }
  /**
   * This method sets the visibility. The value of the parameter
   * should be 0 or 1 or 2.
   * @param m-  an Integer
   */

  public void setVisibility(int m) {
  mandatory = m;
  }

  public void setParent(Node n){
    Assert.require(n instanceof ASTPropertyContainer);
    parent= (ASTPropertyContainer) n;
  }

  public Node getParent(){
    return parent;
  }

  public String toString(){
    String res = "";
    if(beforeComments != null)
      res += beforeComments.toString();
    res += "property ";
    if (mandatory == OPTIONAL)
      res += "optional ";
    else if(mandatory == MANDATORY)
      res += "mandatory ";
    res += propId.toString();

    if(initExpr != null){
      res += " = ";
      res += initExpr.toString();
    }
    res += ";";
    if(afterComments != null)
      res += afterComments.toString();


    return res;
  }

  public String toPrologString(){

    Assert.require(parent);
    String res = "property(";

    res += parent.getContainerID().toPrologString();
    if (mandatory == OPTIONAL)
      res += ", 'optional', ";
    else
      res += ", 'mandatory', ";

    res += propId.toPrologString() + ", ";

    // possibly needed,currently ommitted
    // res += propType.toString();

    if(initExpr != null){
      res += initExpr.toPrologString();
    }
    else
      res +="'void'";

    res += ").";
    return res;
  }

 public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the subnodes
    Assert.require(propId);
    propId.accept(v, data);

    if(initExpr != null)
      initExpr.accept(v, data);

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }


}