package coco.parser.ast;
import java.util.*;
import coco.utils.*;
import coco.parser.ast.enums.*;

/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public class ASTPropertyList extends GenericContainerNode {

  private ASTPropertyContainer parent = null;

  public ASTPropertyList(ASTProperty prop) {
    super();
    if(prop != null)
      addNode(prop);
  }


  public void addNode(Node n){
    Assert.require(n instanceof ASTProperty);
    container.add(n);
    if(parent != null)
      n.setParent(parent);

  }
  public Enumeration getNodes(){
    return new PropertyEnumeration(container);
  }

  public String toString(){
    PropertyEnumeration enum = (PropertyEnumeration) getNodes();
    String res = "";
    if(beforeComments != null)
      res += beforeComments.toString();

    res += "";
    ASTProperty prop;
    while(enum.hasMoreElements()){
      prop = (ASTProperty) enum.nextElement();
      res+= prop.toString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }

  public String toPrologString(){
    PropertyEnumeration enum = (PropertyEnumeration) getNodes();
    String res = "% properties \n";
    ASTProperty prop;
    while(enum.hasMoreElements()){
      prop = (ASTProperty) enum.nextElement();
      res+= prop.toPrologString();
      res+="\n";
    }
    return res;
  }

  public void setParent(Node n){
    Assert.require(n);
    Assert.require(n instanceof ASTPropertyContainer);
    parent = (ASTPropertyContainer)n;

    // set parent for all child nodes
    setParents(parent);
  }

  public Node getParent(){
    return parent;
  }


  protected void setParents(Node n){
    Assert.require(n);
// todo: possibly type check needed
    PropertyEnumeration enum = (PropertyEnumeration) getNodes();
    ASTProperty elem;
    while(enum.hasMoreElements()){
      elem = (ASTProperty) enum.nextElement();
      elem.setParent(n);
    }
  }

  public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the subnodes
    PropertyEnumeration enum = (PropertyEnumeration) getNodes();
    ASTProperty prop;
    while(enum.hasMoreElements()){
      prop = (ASTProperty) enum.nextElement();
      prop.accept(v, data);

      if(enum.hasMoreElements())
        v.visitIntraList(this);


    }

    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}