package coco.parser.ast;
import coco.utils.*;
import java.io.File;
import java.util.Enumeration;
import coco.parser.symtab.SymtabService;
import coco.parser.symtab.SymtabDuplicateEntryException;
/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author
 * @version 1.0
 */

public abstract class ASTPropertySet extends GenericNode implements ASTCompilationUnit, ASTPropertyContainer{


  protected ASTIdentifier ident = null;
  protected ASTAbstractPropertyList properties = null;
  protected File sourceFile = null;

  public ASTPropertySet(ASTIdentifier id, ASTAbstractPropertyList props) {
    Assert.require(id);
    Assert.require(props);

    ident = id;
    properties = props;
    properties.setParent(this);
  }

  public ASTIdentifier getId(){
    return ident;
  }

  public void setId(ASTIdentifier id){
    Assert.require(id);
    ident = id;
  }

  public ASTPropertyList getPropertyList(){
    return properties;
  }

  public void setPropertyList(ASTAbstractPropertyList props){
    Assert.require(props);
    properties = props;
    properties.setParent(this);
  }

 public void addProperty(ASTProperty p) throws SymtabDuplicateEntryException{
    Assert.require(this.properties);
    Assert.require(p);
    Assert.require(p instanceof ASTAbstractProperty);

    SymtabService.registerLocal(this, SymtabService.S_PROPERTY, p.getPropertyId().getID(), p);

    properties.addNode((ASTAbstractProperty)p);
  }

  public void removeProperty(ASTProperty p){
    // there must be a property list, if somethings is gonna be removed.
    Assert.require(properties);
    // remove the proeprty from the local symbol table
    if(SymtabService.lookupLocal(this, SymtabService.S_PROPERTY, p.getPropertyId().getID()) != null){
      SymtabService.remove(this, SymtabService.S_PROPERTY, p.getPropertyId().getID());
    }
    properties.removeNode(p);

  }

  public abstract Object accept(AbstractASTVisitor v, Object data);

  public void setSourceFile(File f){
    Assert.require(f);
    sourceFile = f;
  }
  public File getSourceFile(){
    return sourceFile;
  }
  public void register() throws coco.parser.symtab.SymtabDuplicateEntryException{

      SymtabService.register(this);

      Assert.require(properties);
      for (Enumeration enum = properties.getNodes(); enum.hasMoreElements(); ) {
          ASTAbstractProperty p = (ASTAbstractProperty)enum.nextElement();
          SymtabService.registerLocal(this, SymtabService.S_PROPERTY, p.getPropertyId().getID(), p);
      }
  }
  public void unregister(){
    SymtabService.remove(this);
  }

  /**
   * inherited from ASTPropertyContainer
   */
  public ASTIdentifier getContainerID(){
    return this.getId();
  }

  public String getUnitName(){
    return getId().getID()+".cm";
  }
  public abstract String getUnitType();


}