package coco.parser.ast;
import coco.utils.*;


/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTRole extends GenericNode {
  private ASTIdentifier varId;
  private ASTIdentifier typeId;
  private ASTInterface parent = null;

  public ASTRole(ASTIdentifier v, ASTIdentifier t) {
    Assert.require(v);
    Assert.require(t);

    varId = v;
    typeId = t;
  }

  public ASTIdentifier getVarId(){
    return varId;
  }
  public ASTIdentifier getTypeId(){
    return typeId;
  }

 public void setParent(Node n){
  Assert.require(n);
  Assert.require(n instanceof ASTInterface);

  parent = (ASTInterface) n;
 }

 public Node getParent(){
  return parent;
 }

  public String toString(){
    String res = "";
    if(beforeComments != null)
      res +=beforeComments.toString();
    res +=   "role " + typeId.toString() + " "+ varId.toString() + ";";

    if(afterComments != null)
      res +=afterComments.toString();

    return res;
  }


 public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the subnodes
    typeId.accept(v, data);
    varId.accept(v, data);


    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}