package coco.parser.ast;
import java.util.*;
import coco.parser.ast.enums.*;
import coco.utils.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ASTRoleList extends GenericContainerNode {

  public ASTRoleList(ASTRole r){
    super();
    if(r != null)
      addNode(r);
  }

  public void addNode(Node n){
    Assert.require(n instanceof ASTRole);
    container.add(n);
  }

  public Enumeration getNodes(){
    return new RoleEnumeration(container);
  }

  public String toString(){
    RoleEnumeration enum = (RoleEnumeration) getNodes();
    String res = "\n\n";
    ASTRole i;
    while(enum.hasMoreElements()){
      i = (ASTRole) enum.nextElement();
      res+= i.toString();
      if(enum.hasMoreElements())
        res+="\n";
    }
    return res;
  }

  protected void setParents(Node n){
    Assert.require(n);
    Assert.require(n instanceof ASTInterface);
    RoleEnumeration enum = (RoleEnumeration) getNodes();
    ASTRole elem;
    while(enum.hasMoreElements()){
      elem = (ASTRole) enum.nextElement();
      elem.setParent(n);
    }
  }


 public Object accept(AbstractASTVisitor v, Object data) {
    if(v.preVisit()){
      v.visitPre(this);
    }
    // visit the subnodes
    RoleEnumeration enum = (RoleEnumeration) getNodes();
    ASTRole r;
    while(enum.hasMoreElements()){
      r = (ASTRole) enum.nextElement();
      r.accept(v, data);

      if(enum.hasMoreElements())
        v.visitIntraList(this);



    }
    if(v.postVisit()){
      v.visitPost(this);
    }
    return null;
  }

}
