package coco.parser.ast;
import coco.utils.*;
import java.util.Vector;

import coco.parser.symtab.SymtabDuplicateEntryException;
/**
 * berschrift:
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:
 * @author
 * @version 1.0
 */

public abstract class GenericNode implements Node{

  protected ASTComments beforeComments = null;
  protected ASTComments afterComments = null;

  public int startLine = -1;
  public int endLine = -1;

  public GenericNode() {
  }


  /**
   * Returns the start line for the respective node in the source file.
   *
   * @ return then line number starting with 1. if no line information is
   * available, -1 is returned
   */
  public int getStartLine(){
    return startLine;
  }
  /**
   * Returns the end line for the respective node in the source file.
   *
   * @ return then line number starting with 1. if no line information is
   * available, -1 is returned.
   */
  public int getEndLine(){
    return endLine;
  }
  public void openNode(){}
  public void closeNode(){}

  // has to be implemented if needed
  public void setParent(Node n){
    // must not be called when not overridden
    Assert.require(null);
  }
  // has to be implemented if needed
  public Node getParent(){
    // must not be called when not overridden
    Assert.require(null);

    return null;
  }

  public static GenericNode parse(String text) throws coco.parser.ParseException{
   return null;
  }

  public int getNumChildren(){
    return 0;
  }

  public Node getChild(int i){
    return null;
  }


  public String toPrologString(){
    return toString();
  }


  /** Attaches before-comments if any to an AST node.
   *  @param comment the comment wich was found in front of the AST element
   */
   public void attachBeforeComment(ASTComment c){
    if(beforeComments == null)
      beforeComments = new ASTComments();
    beforeComments.add(c);
   }


  /** Attaches before-comments if any to an AST node.
   *  @param comment the comment wich was found in front of the AST element
   */
   public void attachBeforeComment(ASTComments c){
    Assert.require(c);
    if(beforeComments == null)
      beforeComments = new ASTComments();

    beforeComments.add(c);
   }



  /** Attaches after-comments if any to an AST node.
   *  @param comment the comment wich was found behind the AST element
   */
   public void attachAfterComment(ASTComment c){
    if(afterComments == null)
      afterComments = new ASTComments();
    afterComments.add(c);
   }

  /** Attaches after-comments if any to an AST node.
   *  @param comment the comment wich was found behind the AST element
   */
   public void attachAfterComment(ASTComments c){
    if(afterComments == null)
      afterComments = new ASTComments();
    afterComments.add(c);
   }


  /** Get before-comments of an AST node
   *  @return the comment wich was found in front of the AST element
   */
   public ASTComments getBeforeComment(){
    return beforeComments;
   }


  /** Get after-comments of an AST node
   *  @return the comment wich was found behind the AST element
   */
   public ASTComments getAfterComment(){
    return afterComments;
   }

  /**
   * Copy all data from another generic node into this generic node
   * resync the syntax tree. The parameter src must not be null.
   *
   * @param src   the source node
   */
  public void copyFrom(GenericNode src) throws SymtabDuplicateEntryException{
      Assert.require(src);
      this.beforeComments = src.beforeComments;
      this.afterComments = src.afterComments;
  }


}