package coco.parser.symtab;

import coco.parser.ast.*;
import java.util.Hashtable;
import java.util.Enumeration;
import coco.utils.*;
import java.io.PrintStream;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ComponentSymtab  {
  private Hashtable symtab;

  private Hashtable subjects;

  private static ComponentSymtab instance = null;
  protected ComponentSymtab() {
    symtab = new Hashtable(30);
    subjects = new Hashtable();
  }


  private void init() {
    symtab = new Hashtable(30);
    subjects = new Hashtable();
  }

  public static ComponentSymtab getInstance(){
    if(instance == null){
      instance = new ComponentSymtab();
    }
    return instance;
  }

  ASTComponent lookup(String key){
    Assert.require(key);
    return (ASTComponent) symtab.get(key);
  }

  boolean exists(String key){
    Assert.require(key);
    return symtab.containsKey(key);
  }

  void remove(String key){
    Assert.require(key);
    symtab.remove(key);
  }
  public void register(ASTComponent component) throws SymtabDuplicateEntryException {
    Assert.require(component);
    ASTIdentifier id = component.getID();
    if(symtab.containsKey(id.getID()))
      throw new SymtabDuplicateEntryException(id);
    symtab.put(id.getID(), component);
/*    try {
    System.out.println("Register----------");
    Enumeration enum = symtab.elements();
    while (enum.hasMoreElements()) {
    System.out.println(((ASTComponent)enum.nextElement()).getID().getID());
    }
    System.out.println("Register----------");
    }
    catch(Exception error) {error.printStackTrace();}
  */
  }

  public Enumeration elements() {
    return(symtab.elements());
  }

  // local symbols
  public void registerLocal(ASTComponent comp, String subject, String symbol, Object o) throws SymtabDuplicateEntryException {

    // is this subject already registered ?
    Hashtable syms = (Hashtable)subjects.get(subject);
    if (syms == null) {
      syms = new Hashtable();
      subjects.put(subject, syms);
    }

    Hashtable locals = (Hashtable)syms.get(comp);
    if (locals == null) {
      locals = new Hashtable();
      syms.put(comp, locals);
    }

    if (locals.containsKey(symbol)) {
      throw(new SymtabDuplicateEntryException(symbol));
    } else {
      locals.put(symbol, o);
    }
  }

  public boolean existsLocal(ASTComponent comp, String subject, String symbol) {
    boolean res = false;

    Hashtable syms = (Hashtable)subjects.get(subject);
    if (syms != null) {
      Hashtable locals = (Hashtable)syms.get(comp);

      if (locals != null) {
        res = locals.containsKey(symbol);
      }
    }
    return(res);
  }

  public Object lookupLocal(ASTComponent comp, String subject, String symbol) {
    Object res = null;
    if (existsLocal(comp, subject, symbol)) {
      res = ((Hashtable)((Hashtable)subjects.get(subject)).get(comp)).get(symbol);
    }
    return(res);
  }

  public void removeLocal(ASTComponent comp, String subject, String symbol) {

    if (existsLocal(comp, subject, symbol)) {
      ((Hashtable)((Hashtable)subjects.get(subject)).get(comp)).remove(symbol);
    }
/*
  try {
    System.out.println("While Removing -----------");
    Enumeration test = ((Hashtable)((Hashtable)subjects.get(subject)).get(comp)).elements();
    while (test.hasMoreElements())
    System.out.println(test.nextElement());
  }
  catch(Exception error) {error.printStackTrace();}
  System.out.println("While Removing -----------complete");
*/
   }

  public void remove(ASTComponent comp) {
  Assert.require(comp);
  remove(comp.getID().getID());
  ASTComponentTypeBody body = comp.getBody();
  Assert.require(body);
  ASTPortList pl = body.getPortList();
  if (pl != null) {
    for (Enumeration enum = pl.getNodes(); enum.hasMoreElements(); ) {
      ASTPort p = (ASTPort)enum.nextElement();
      removeLocal(comp,SymtabService.S_PORT,p.getPortId().getID());
    }
  }
  // properties
  ASTPropertyList properties = body.getPropertyList();
  if (properties != null) {

    for (Enumeration enum = properties.getNodes(); enum.hasMoreElements(); ) {
        ASTProperty p = (ASTProperty)enum.nextElement();
        removeLocal(comp, SymtabService.S_PROPERTY, p.getPropertyId().getID());
    }
  }


    /* try {
  System.out.println("While Removing Components-----------");
  Enumeration test = symtab.elements();
    while (test.hasMoreElements())
    System.out.println(test.nextElement());
  }
  catch(Exception error) {error.printStackTrace();}
  System.out.println("While Removing -----------complete");

*/
  }

  public Enumeration elementsLocal(ASTComponent comp, String subject) {
    Enumeration res = null;
    Hashtable syms = (Hashtable)subjects.get(subject);
    if (syms != null) {
      Hashtable locals = (Hashtable)syms.get(comp);

      if (locals != null) {
        res = locals.elements();
      }
    }
    return(res);
  }

  // dump

  public void dump(PrintStream os){
    Enumeration k = symtab.keys();
    os.println("components [");
    while(k.hasMoreElements()){
      os.println(k.nextElement().toString());
    }
    os.println("]");
  }
  /**
     *Removes all entries in the symbol table.
     */
    public void removeAll() {
	init();
    }
}