package coco.parser.symtab;
import coco.parser.ast.*;
import java.util.Hashtable;
import java.util.Enumeration;
import coco.utils.*;
import java.io.PrintStream;


/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class ConnectorSymtab {
    private Hashtable symtab;
    private Hashtable subjects;
    private static ConnectorSymtab instance = null;

    protected ConnectorSymtab() {
	init();
    }

    private void init() {
	symtab = new Hashtable(30);
	subjects = new Hashtable();
    }

    public static ConnectorSymtab getInstance(){
	if(instance == null){
	    instance = new ConnectorSymtab();
	}
	return instance;
    }

    ASTConnector lookup(String key){
	Assert.require(key);
	return (ASTConnector) symtab.get(key);
    }

    boolean exists(String key){
	Assert.require(key);
	return symtab.containsKey(key);
    }

    void remove(String key){
	Assert.require(key);
	symtab.remove(key);
    }

   public void remove(ASTConnector node){
	Assert.require(node);
	remove(node.getConnectorID().getID());
        ASTConnectorBody body = node.getBody();
        Assert.require(body);

        ASTPropertyList properties = body.getPropertyList();
        if (properties != null) {
          for (Enumeration enum = properties.getNodes(); enum.hasMoreElements(); ) {
              ASTProperty p = (ASTProperty)enum.nextElement();
              removeLocal(node, SymtabService.S_PROPERTY, p.getPropertyId().getID());
          }
      }

    }

    public void register(ASTConnector connector) throws SymtabDuplicateEntryException{
	Assert.require(connector);
	ASTIdentifier id = connector.getConnectorID();
	if(symtab.containsKey(id.getID()))
	    throw new SymtabDuplicateEntryException(id);
	symtab.put(id.getID(), connector);
    }

    public Enumeration elements() {
	return(symtab.elements());
    }

    // local symbols
    public void registerLocal(ASTConnector conn, String subject, String symbol, Object o) throws SymtabDuplicateEntryException {

	// is this subject already registered ?
	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms == null) {
	    syms = new Hashtable();
	    subjects.put(subject, syms);
	}

	Hashtable locals = (Hashtable)syms.get(conn);
	if (locals == null) {
	    locals = new Hashtable();
	    syms.put(conn, locals);
	}

	if (locals.containsKey(symbol)) {
	    throw(new SymtabDuplicateEntryException(symbol));
	} else {
	    locals.put(symbol, o);
	}
    }

    public boolean existsLocal(ASTConnector conn, String subject, String symbol) {
	boolean res = false;

	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms != null) {
	    Hashtable locals = (Hashtable)syms.get(conn);

	    if (locals != null) {
		res = locals.containsKey(symbol);
	    }
	}
	return(res);
    }

    public Object lookupLocal(ASTConnector conn, String subject, String symbol) {
	Object res = null;
	if (existsLocal(conn, subject, symbol)) {
	    res = ((Hashtable)((Hashtable)subjects.get(subject)).get(conn)).get(symbol);
	}
	return(res);
    }

    public void removeLocal(ASTConnector conn, String subject, String symbol) {
	if (existsLocal(conn, subject, symbol)) {
	    ((Hashtable)((Hashtable)subjects.get(subject)).get(conn)).remove(symbol);
	}
    }

    public Enumeration elementsLocal(ASTConnector conn, String subject) {
	Enumeration res = null;
	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms != null) {
	    Hashtable locals = (Hashtable)syms.get(conn);

	    if (locals != null) {
		res = locals.elements();
	    }
	}
	return(res);
    }

    // general
    public void dump(PrintStream os){
	Enumeration k = symtab.keys();
	os.println("connectors [");
	while(k.hasMoreElements()){
	    os.println(k.nextElement().toString());
	}
	os.println("]");
    }

    /**
     *Removes all entries in the symbol table.
     */
    public void removeAll() {
	init();
    }

}
