package coco.parser.symtab;

import coco.parser.ast.*;
import java.util.Hashtable;
import java.util.Enumeration;
import coco.utils.*;
import java.io.PrintStream;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class DataTypeSymtab {
    private Hashtable symtab;
    private Hashtable subjects;

    private static DataTypeSymtab instance = null;

    protected DataTypeSymtab() {
	init();
    }

    private void init() {
	symtab = new Hashtable(30);
	subjects = new Hashtable();
    }

    public static DataTypeSymtab getInstance(){
	if(instance == null){
	    instance = new DataTypeSymtab();
	}
	return instance;
    }

    ASTDataType lookup(String key){
	Assert.require(key);
	return (ASTDataType) symtab.get(key);
    }

    boolean exists(String key){
	Assert.require(key);
	return symtab.containsKey(key);
    }

    void remove(String key){
	Assert.require(key);
	symtab.remove(key);
    }

    public void remove(ASTDataType p){
	Assert.require(p);
        ASTIdentifier id = p.getDataTypeIdentifier();
	Assert.require(id);
	remove(id.getID());
	removeLocals(p);
    }


    public void register(ASTDataType packet) throws SymtabDuplicateEntryException{
	Assert.require(packet);

        ASTIdentifier id = packet.getDataTypeIdentifier();
	Assert.require(id);

	if(symtab.containsKey(id.getID()))
	    throw new SymtabDuplicateEntryException(id);
	symtab.put(id.getID(), packet);
    }

    public Enumeration elements() {
	return(symtab.elements());
    }

    // local symbols
    public void registerLocal(ASTDataType pack, String subject, String symbol, Object o) throws SymtabDuplicateEntryException {

	// is this subject already registered ?
	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms == null) {
	    syms = new Hashtable();
	    subjects.put(subject, syms);
	}

	Hashtable locals = (Hashtable)syms.get(pack);
	if (locals == null) {
	    locals = new Hashtable();
	    syms.put(pack, locals);
	}

	if (locals.containsKey(symbol)) {
	    throw(new SymtabDuplicateEntryException(symbol));
	} else {
	    locals.put(symbol, o);
	}
    }

    public void removeLocals(ASTDataType p) {
	for (Enumeration enum = subjects.elements(); enum.hasMoreElements(); ) {
	    Hashtable ht = (Hashtable)enum.nextElement();

	    if (ht.containsKey(p)) {
		ht.remove(p);
	    }
	}
    }

    public boolean existsLocal(ASTDataType pack, String subject, String symbol) {
	boolean res = false;

	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms != null) {
	    Hashtable locals = (Hashtable)syms.get(pack);

	    if (locals != null) {
		res = locals.containsKey(symbol);
	    }
	}
	return(res);
    }

    public Object lookupLocal(ASTDataType pack, String subject, String symbol) {
	Object res = null;
	if (existsLocal(pack, subject, symbol)) {
	    res = ((Hashtable)((Hashtable)subjects.get(subject)).get(pack)).get(symbol);
	}
	return(res);
    }

    public void removeLocal(ASTDataType pack, String subject, String symbol) {
	if (existsLocal(pack, subject, symbol)) {
	    ((Hashtable)((Hashtable)subjects.get(subject)).get(pack)).remove(symbol);
	}
    }

    public Enumeration elementsLocal(ASTDataType pack, String subject) {
	Enumeration res = null;
	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms != null) {
	    Hashtable locals = (Hashtable)syms.get(pack);

	    if (locals != null) {
		res = locals.elements();
	    }
	}
	return(res);
    }

    // general
    public void dump(PrintStream os){
	Enumeration k = symtab.keys();
	os.println("packets [");
	while(k.hasMoreElements()){
	    os.println(k.nextElement().toString());
	}
	os.println("]");
    }

    /**
     *Removes all entries in the symbol table.
     */
    public void removeAll() {
	init();
    }
}
