package coco.parser.symtab;

import coco.parser.ast.*;
import java.util.Hashtable;
import java.util.Enumeration;
import coco.utils.*;
import java.io.PrintStream;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class InterfaceSymtab  {
    private Hashtable symtab;

    private Hashtable subjects;

    private static InterfaceSymtab instance = null;

    protected InterfaceSymtab() {
	init();
    }

    private void init() {
	symtab = new Hashtable(30);
	subjects = new Hashtable();
    }

    public static InterfaceSymtab getInstance(){
	if(instance == null){
	    instance = new InterfaceSymtab();
	}
	return instance;
    }

    ASTInterface lookup(String key){
	Assert.require(key);
	return (ASTInterface) symtab.get(key);
    }

    boolean exists(String key){
	Assert.require(key);
	return symtab.containsKey(key);
    }


    public void remove(ASTInterface node) {
      Assert.require(node);
      remove(node.getID().getID());
      ASTComponentTypeBody body = node.getBody();
      Assert.require(body);
     ASTPortList pl = body.getPortList();
      if (pl != null) {
        for (Enumeration enum = pl.getNodes(); enum.hasMoreElements(); ) {
          ASTPort p = (ASTPort)enum.nextElement();
          removeLocal(node,SymtabService.S_PORT,p.getPortId().getID());
        }
      }
      ASTPropertyList properties = body.getPropertyList();
      if (properties != null) {
        for (Enumeration enum = properties.getNodes(); enum.hasMoreElements(); ) {
            ASTProperty p = (ASTProperty)enum.nextElement();
            removeLocal(node, SymtabService.S_PROPERTY, p.getPropertyId().getID());
        }
      }
  }
    void remove(String key){
	Assert.require(key);
	symtab.remove(key);
    }
    public void register(ASTInterface component) throws SymtabDuplicateEntryException {
	Assert.require(component);
	ASTIdentifier id = component.getID();
	if(symtab.containsKey(id.getID()))
	    throw new SymtabDuplicateEntryException(id);
	symtab.put(id.getID(), component);
    }

    public Enumeration elements() {
	return(symtab.elements());
    }

    // local symbols
    public void registerLocal(ASTInterface comp, String subject, String symbol, Object o) throws SymtabDuplicateEntryException {

	// is this subject already registered ?
	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms == null) {
	    syms = new Hashtable();
	    subjects.put(subject, syms);
	}

	Hashtable locals = (Hashtable)syms.get(comp);
	if (locals == null) {
	    locals = new Hashtable();
	    syms.put(comp, locals);
	}

	if (locals.containsKey(symbol)) {
	    throw(new SymtabDuplicateEntryException(symbol));
	} else {
	    locals.put(symbol, o);
	}
    }

    public boolean existsLocal(ASTInterface comp, String subject, String symbol) {
	boolean res = false;

	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms != null) {
	    Hashtable locals = (Hashtable)syms.get(comp);

	    if (locals != null) {
		res = locals.containsKey(symbol);
	    }
	}
	return(res);
    }

    public Object lookupLocal(ASTInterface comp, String subject, String symbol) {
	Object res = null;
	if (existsLocal(comp, subject, symbol)) {
	    res = ((Hashtable)((Hashtable)subjects.get(subject)).get(comp)).get(symbol);
	}
	return(res);
    }

    public void removeLocal(ASTInterface comp, String subject, String symbol) {
	if (existsLocal(comp, subject, symbol)) {
	    ((Hashtable)((Hashtable)subjects.get(subject)).get(comp)).remove(symbol);
	}
    }

    public Enumeration elementsLocal(ASTInterface comp, String subject) {
	Enumeration res = null;
	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms != null) {
	    Hashtable locals = (Hashtable)syms.get(comp);

	    if (locals != null) {
		res = locals.elements();
	    }
	}
	return(res);
    }

    // dump

    public void dump(PrintStream os){
	Enumeration k = symtab.keys();
	os.println("interfaces [");
	while(k.hasMoreElements()){
	    os.println(k.nextElement().toString());
	}
	os.println("]");
    }


    /**
     *Removes all entries in the symbol table.
     */
    public void removeAll() {
	init();
    }
}
