package coco.parser.symtab;
import coco.parser.ast.*;
import java.util.Hashtable;
import java.util.Enumeration;
import coco.utils.*;
import java.io.PrintStream;
/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author
 * @version 1.0
 */

public class PropertySetSymtab {

    private Hashtable symtab;
    private Hashtable subjects;

    private static PropertySetSymtab instance = null;

    public PropertySetSymtab() {
	init();
    }

    private void init() {
	symtab = new Hashtable(30);
	subjects = new Hashtable(30);
    }

    public static PropertySetSymtab getInstance(){
	if(instance == null){
	    instance = new PropertySetSymtab();
	}
	return instance;
    }

    ASTPropertySet lookup(String key){
	Assert.require(key);
	return (ASTPropertySet) symtab.get(key);
    }

    boolean exists(String key){
	Assert.require(key);
	return symtab.containsKey(key);
    }

    void remove(String key){
	Assert.require(key);
	symtab.remove(key);
    }

    public void remove(ASTPropertySet p){
	Assert.require(p);
        ASTIdentifier id = p.getId();
	Assert.require(id);
	remove(id.getID());
	removeLocals(p);
    }


    public void register(ASTPropertySet propSet) throws SymtabDuplicateEntryException{
	Assert.require(propSet);

        ASTIdentifier id = propSet.getId();
	Assert.require(id);

	if(symtab.containsKey(id.getID()))
	    throw new SymtabDuplicateEntryException(id);
	symtab.put(id.getID(), propSet);
    }

    public Enumeration elements() {
	return(symtab.elements());
    }

    // local symbols
    public void registerLocal(ASTPropertySet propSet, String subject, String symbol, Object o) throws SymtabDuplicateEntryException {

	// is this subject already registered ?
	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms == null) {
	    syms = new Hashtable();
	    subjects.put(subject, syms);
	}

	Hashtable locals = (Hashtable)syms.get(propSet);
	if (locals == null) {
	    locals = new Hashtable(10);
	    syms.put(propSet, locals);
	}

	if (locals.containsKey(symbol)) {
	    throw(new SymtabDuplicateEntryException(symbol));
	} else {
	    locals.put(symbol, o);
	}
    }

    public void removeLocals(ASTPropertySet p) {
	for (Enumeration enum = subjects.elements(); enum.hasMoreElements(); ) {
	    Hashtable ht = (Hashtable)enum.nextElement();

	    if (ht.containsKey(p)) {
		ht.remove(p);
	    }
	}
    }

    public boolean existsLocal(ASTPropertySet propSet, String subject, String symbol) {
	boolean res = false;

	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms != null) {
	    Hashtable locals = (Hashtable)syms.get(propSet);

	    if (locals != null) {
		res = locals.containsKey(symbol);
	    }
	}
	return(res);
    }

    public Object lookupLocal(ASTPropertySet propSet, String subject, String symbol) {
	Object res = null;
	if (existsLocal(propSet, subject, symbol)) {
	    res = ((Hashtable)((Hashtable)subjects.get(subject)).get(propSet)).get(symbol);
	}
	return(res);
    }

    public void removeLocal(ASTPropertySet propSet, String subject, String symbol) {
	if (existsLocal(propSet, subject, symbol)) {
	    ((Hashtable)((Hashtable)subjects.get(subject)).get(propSet)).remove(symbol);
	}
    }

    public Enumeration elementsLocal(ASTPropertySet propSet, String subject) {
	Enumeration res = null;
	Hashtable syms = (Hashtable)subjects.get(subject);
	if (syms != null) {
	    Hashtable locals = (Hashtable)syms.get(propSet);

	    if (locals != null) {
		res = locals.elements();
	    }
	}
	return(res);
    }

    // general
    public void dump(PrintStream os){
	Enumeration k = symtab.keys();
	os.println("PropertySets [");
	while(k.hasMoreElements()){
	    os.println(k.nextElement().toString());
	}
	os.println("]");
    }

    /**
     *Removes all entries in the symbol table.
     */
    public void removeAll() {
	init();
    }

}