package coco.parser.symtab;
import coco.parser.ast.*;

import java.io.PrintStream;
import java.util.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author T. Genssler, A. Christoph
 * @version 1.0
 */

public class SymtabService {

  // some common subjects for local symbols
  public static final String S_PORT = "port";
  public static final String S_PLUG = "plug";
  public static final String S_FIELD = "field";
  public static final String S_TYPEPARAM = "typeparam";
  public static final String S_INTERFACE = "interface";
  public static final String S_PROPERTY = "property";
  public static final String S_INSTANCES = "instances";
  public static final String S_PORTACCESS = "portaccess";
  public static final String S_ROLE = "role";
  public static final String S_CONNECTOR = "connector";

  public SymtabService() {}


  // Components ************************************
  public static ASTComponent lookupComponent(String key){
    return ComponentSymtab.getInstance().lookup(key);
  }

  public static void register(ASTComponent c) throws SymtabDuplicateEntryException{
    ComponentSymtab.getInstance().register(c);
  }

  public static void dumpComponentSymtab(PrintStream os){
    ComponentSymtab.getInstance().dump(os);
  }

  public static Enumeration getComponents() {
    return(ComponentSymtab.getInstance().elements());
  }
  public static void remove(ASTComponent c) {
    ComponentSymtab.getInstance().remove(c);
  }

  public static void remove(ASTComponent comp, String subject, String symbol) {
    ComponentSymtab.getInstance().removeLocal(comp,subject,symbol);
 }

  // locals
  public static Object lookupLocal(ASTComponent c, String subject, String key) {
    return ComponentSymtab.getInstance().lookupLocal(c, subject, key);
  }

  public static void registerLocal(ASTComponent c, String subject, String key, Object o) throws SymtabDuplicateEntryException{
    ComponentSymtab.getInstance().registerLocal(c, subject, key, o);
  }

  public static Enumeration getLocals(ASTComponent c, String subject) {
    return(ComponentSymtab.getInstance().elementsLocal(c, subject));
  }
  // end Components ************************************

  // Interfaces ************************************
  public static ASTInterface lookupInterface(String key){
    return InterfaceSymtab.getInstance().lookup(key);
  }

  public static void register(ASTInterface c) throws SymtabDuplicateEntryException{
    InterfaceSymtab.getInstance().register(c);
  }

  public static void dumpInterfaceSymtab(PrintStream os){
    InterfaceSymtab.getInstance().dump(os);
  }

  public static Enumeration getInterfaces() {
    return(InterfaceSymtab.getInstance().elements());
  }

  // locals
  public static Object lookupLocal(ASTInterface c, String subject, String key) {
    return InterfaceSymtab.getInstance().lookupLocal(c, subject, key);
  }

  public static void registerLocal(ASTInterface c, String subject, String key, Object o) throws SymtabDuplicateEntryException{
    InterfaceSymtab.getInstance().registerLocal(c, subject, key, o);
  }

 public static void remove(ASTInterface c, String subject, String symbol) {
    InterfaceSymtab.getInstance().removeLocal(c,subject,symbol);
 }

 public static void remove(ASTInterface c) {
    InterfaceSymtab.getInstance().remove(c);
 }

  public static Enumeration getLocals(ASTInterface c, String subject) {
    return(InterfaceSymtab.getInstance().elementsLocal(c, subject));
  }
  // end Interfaces ************************************

  // Connectors ************************************
  public static ASTConnector lookupConnector(String key){
    return ConnectorSymtab.getInstance().lookup(key);
  }

  public static void register(ASTConnector c) throws SymtabDuplicateEntryException{
    ConnectorSymtab.getInstance().register(c);
  }

  public static void remove(ASTConnector r){
    ConnectorSymtab.getInstance().remove(r);
  }

  public static void dumpConnectorSymtab(PrintStream os){
    ConnectorSymtab.getInstance().dump(os);
  }

  public static Enumeration getConnectors() {
    return(ConnectorSymtab.getInstance().elements());
  }

  // locals
  public static Object lookupLocal(ASTConnector c, String subject, String key) {
    return ConnectorSymtab.getInstance().lookupLocal(c, subject, key);
  }

 public static void remove(ASTConnector c, String subject, String symbol) {
    ConnectorSymtab.getInstance().removeLocal(c,subject,symbol);
 }

  public static void registerLocal(ASTConnector c, String subject, String key, Object o) throws SymtabDuplicateEntryException{
    ConnectorSymtab.getInstance().registerLocal(c, subject, key, o);
  }

  public static Enumeration getLocals(ASTConnector c, String subject) {
    return(ConnectorSymtab.getInstance().elementsLocal(c, subject));
  }
  // end Connectors ************************************


  // property sets ************************************
  public static ASTPropertySet lookupPropertySet(String key){
    return PropertySetSymtab.getInstance().lookup(key);
  }

  public static void register(ASTPropertySet c) throws SymtabDuplicateEntryException{
    PropertySetSymtab.getInstance().register(c);
  }

  public static void remove(ASTPropertySet p){
    PropertySetSymtab.getInstance().remove(p);
  }

  public static void dumpPropertySetSymtab(PrintStream os){
    PropertySetSymtab.getInstance().dump(os);
  }

  public static Enumeration getPropertySets() {
    return(PropertySetSymtab.getInstance().elements());
  }


  // locals
  public static Object lookupLocal(ASTPropertySet p, String subject, String key) {
    return PropertySetSymtab.getInstance().lookupLocal(p, subject, key);
  }

  public static void registerLocal(ASTPropertySet p, String subject, String key, Object o) throws SymtabDuplicateEntryException{
    PropertySetSymtab.getInstance().registerLocal(p, subject, key, o);
  }
 public static void remove(ASTPropertySet c, String subject, String symbol) {
    PropertySetSymtab.getInstance().removeLocal(c,subject,symbol);
 }

  public static Enumeration getLocals(ASTPropertySet p, String subject) {
    return(PropertySetSymtab.getInstance().elementsLocal(p, subject));
  }

  // end property sets ************************************


  // DataTypes ************************************
  public static ASTDataType lookupDataType(String key){
    return DataTypeSymtab.getInstance().lookup(key);
  }

  public static void register(ASTDataType c) throws SymtabDuplicateEntryException{
    DataTypeSymtab.getInstance().register(c);
  }

  public static void remove(ASTDataType p){
    DataTypeSymtab.getInstance().remove(p);
  }


  public static void dumpDataTypeSymtab(PrintStream os){
    DataTypeSymtab.getInstance().dump(os);
  }

  public static Enumeration getDataTypes() {
    return(DataTypeSymtab.getInstance().elements());
  }

  // locals
  public static Object lookupLocal(ASTDataType c, String subject, String key) {
    return DataTypeSymtab.getInstance().lookupLocal(c, subject, key);
  }

  public static void registerLocal(ASTDataType c, String subject, String key, Object o) throws SymtabDuplicateEntryException{
    DataTypeSymtab.getInstance().registerLocal(c, subject, key, o);
  }

  public static Enumeration getLocals(ASTDataType c, String subject) {
    return(DataTypeSymtab.getInstance().elementsLocal(c, subject));
  }
  // end DataTypes ************************************
  // all
  void dumpAll(PrintStream os){
    dumpComponentSymtab(os);
    dumpInterfaceSymtab(os);
    dumpDataTypeSymtab(os);
    dumpPropertySetSymtab(os);
    dumpConnectorSymtab(os);

  }

    /**
     * Deletes all symbol tables
     */
    public static void removeAll() {
			ComponentSymtab.getInstance().removeAll();
			ConnectorSymtab.getInstance().removeAll();
			InterfaceSymtab.getInstance().removeAll();
			DataTypeSymtab.getInstance().removeAll();
                        PropertySetSymtab.getInstance().removeAll();
    }


}
