package coco.project;

import java.util.*;
import java.io.*;
import coco.utils.*;
import coco.filehandling.*;
import coco.fuzzyparser.*;
import coco.model.*;
import coco.model.nodes.*;
import coco.config.*;

/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author
 * @version 1.0
 */


 /**
  * The CoCoProject class encapsulates the access to the different
  * files of project.
  */
public class CoCoProject {

  // Entity Files of the Project - Rule Files not yet there!!!
  private Hashtable cocoFiles = null;

  // The Project File name without path
  private String projectFileName = null;
  // The Project path without filename
  private String projectPath = null;
  // The project's name - for instance related to the filename
  private String projectName = null;

  private boolean isModified = false;

  // File Seperator
  private final String FS = System.getProperty("file.separator");

  // --------------------------------------------------------- CONSTRUCTORS

  // Constructor for new project
  public CoCoProject(){
    projectName = "Unnamed";
    cocoFiles = new Hashtable(20);
    //GlobalConfig.getInstance().setProperty(GlobalConfig.OPTION_PROJECT_LAST,"");
  }

  // Constructor from project file
  public CoCoProject(File f){
    Assert.require(f);
    this.cocoFiles = new Hashtable(20);
    if (f.exists()) {
      this.setProjectFileAndPath(f);
      loadProject();
      GlobalConfig.getInstance().setProperty(GlobalConfig.OPTION_PROJECT_LAST,this.projectPath+this.projectFileName);
    } else {
      System.out.println("No such file: "+f.getAbsoluteFile());
    }
  }

  // --------------------------------------------------------- PUBLIC

  public void saveFile(String fileName) {
    CoCoFile cf = this.getCoCoFile(fileName);
    if (cf != null) {
      cf.save(this.projectPath, this.projectName);
    }
  }

  public CoCoFile getCoCoFile(String s) {
    Enumeration enum = this.getCoCoFiles();
    CoCoFile f;
    while (enum.hasMoreElements()) {
      f = ((CoCoFile)enum.nextElement());
      if (f.getFilename().equals(s)) {
        return f;
      }
    }
    return null;
  }

  public void removeCoCoFile(String fileName) {
    CoCoFile file = getCoCoFile(fileName);
    if (file != null) {
      this.cocoFiles.remove(file.getFilename());
      //System.out.println("-> Removing :"+file.getFilename());
    }
    this.isModified = true;
  }

  public boolean hasChanged() {
    Enumeration enum = this.getCoCoFiles();
    while (enum.hasMoreElements()) {
      if (((CoCoFile)enum.nextElement()).isModified()) return true;
    }
    return this.isModified;
  }

  /**
   * Change the project name and path to file f and save the project.
   */
   public void saveProjectAs(File f) {
    setProjectFileAndPath(f);
    saveProject();
   }

  /**
   * Save the current project. Save ALL files for instance, not only the
   * changed ones. So reuse is possible, if saved to a new project file.
   */
  public void saveProject() {
    // write out the project file !!!
    Properties props = new Properties();
    Enumeration enum = this.getCoCoFiles();
    CoCoFile f;
    int i = 0;
    while (enum.hasMoreElements()) {
      f = (CoCoFile)enum.nextElement();
      //System.out.println("saving:"+f.getFilename());
      if (f.getTypeOf().equals("acm")) {
        props.setProperty("abstractcomponent_" + Integer.toString(i), f.getFilename());
      }
      if (f.getTypeOf().equals("cm")) {
        props.setProperty("component_" + Integer.toString(i), f.getFilename());
      }
      if (f.getTypeOf().equals("prop")) {
        props.setProperty("propertyset_" + Integer.toString(i), f.getFilename());
      }
      if (f.getTypeOf().equals("type")) {
        props.setProperty("datatype_" + Integer.toString(i), f.getFilename());
      }
      i++;
    }
    try {
      FileOutputStream writer = new FileOutputStream(this.projectPath+this.projectFileName);
      props.store(writer, "CoCo-Project");
      writer.close();
    } catch (Exception ex) {
      ex.printStackTrace();
    }

    // make sure that path to file exists
    File dir = new File(projectPath+FS+this.projectName);
    if (!dir.exists()) {
      dir.mkdir();
    }

    // save all coco files
    enum = this.getCoCoFiles();
    if(enum != null)
      while (enum.hasMoreElements()) {
        ((CoCoFile)enum.nextElement()).save(projectPath, projectName);
      }
    GlobalConfig.getInstance().setProperty(GlobalConfig.OPTION_PROJECT_LAST,this.projectPath+this.projectFileName);
    this.isModified = false;
  }

  public String getProjectName() {
    return this.projectName;
  }

  public Enumeration getCoCoFiles() {
    return this.cocoFiles.elements();
  }

  public boolean hasProjetFile() {
    if ((this.projectFileName != null) && (this.projectPath != null)) {
      return true;
    } else {
      return false;
    }
  }

  public String getTextForFile(String fileName) {
    CoCoFile cf = (CoCoFile)cocoFiles.get(fileName);
    return cf.getActualText();
  }

  public void setChangedTextForFile(String fileName, String text) {
    CoCoFile cf = (CoCoFile)cocoFiles.get(fileName);
    cf.setModifiedText(text);

  }

  // --------------------------------------------------------- PRIVATE

  /**
   * Sets the project filename, name and path to the desired file.
   */
  private void setProjectFileAndPath(File f) {
    // set project file name

    String name = f.getName();
    if (name.endsWith(".coco")) {
      this.projectFileName = name;
      this.projectName = name.substring(0,name.lastIndexOf("."));
    } else {
      if (name.lastIndexOf(".") > -1) {
        name = name.substring(0,name.lastIndexOf("."));
      }
      this.projectFileName = name + ".coco";
      this.projectName = name;
    }
    //System.out.println(this.projectFileName);
    //System.out.println(this.projectName);

    String path = f.getAbsolutePath();
    if (path.lastIndexOf(FS) > -1) {
      this.projectPath = path.substring(0,path.lastIndexOf(FS)+1);
    } else {
      this.projectPath = path;
    }
    //System.out.println(this.projectPath);
    Controller.getInstance().changeProjectNameTo(this.projectName);
  }


  /**
   * Adds a component file to the project.
   */
  public CoCoFile addEntityFileToProject(File f) {
    if (!this.isInProject(f.getName())) {
      String name = f.getName();
      String path = f.getPath();
      path = path.substring(0,path.lastIndexOf(FS));
      CoCoFile cf = new CoCoFile(name, path, "");
      cf.setModifiedText(cf.getOriginalText());
      cf.setModifiedText();
      //System.out.println("Adding : "+name);
      this.cocoFiles.put(name, cf);
      return cf;
    } else {
      System.out.println("Error: A file with this name does already exist in tis project!");
      return null;
    }
  }


  public boolean isInProject(String name){
    Enumeration files = this.getCoCoFiles();
    String fname = null;
    while (files.hasMoreElements()) {
      fname = ((CoCoFile)files.nextElement()).getFilename();
      if (fname.equals(name)) return true;
    }
    return false;
  }

  public CoCoFile addNewComponentToProject(String compName) {
    CoCoFile cFile = new CoCoFile(compName+".cm");
    String text = "component " + compName + " { }";
    cFile.setModifiedText(text);
    //System.out.println("Adding : "+compName+".cm");
    this.cocoFiles.put(compName+".cm", cFile);
    return cFile;
  }

  public CoCoFile addNewDataTypeToProject(String typeName) {
    CoCoFile cFile = new CoCoFile(typeName+".type");
    String text = "type " + typeName + " { }";
    cFile.setModifiedText(text);
    //System.out.println("Adding : "+typeName+".type");
    this.cocoFiles.put(typeName+".type", cFile);
    return cFile;
  }


  public CoCoFile addNewAbstractComponentToProject(String compName) {
    CoCoFile cFile = new CoCoFile(compName+".acm");
    String text = "abstract component " + compName + " { }";
    cFile.setModifiedText(text);
    //System.out.println("Adding : "+compName+".acm");
    this.cocoFiles.put(compName+".acm", cFile);
    return cFile;
  }

  public CoCoFile addNewPropertySetToProject(String propName) {
    CoCoFile cFile = new CoCoFile(propName+".prop");
    String text = "component properties " + propName + " { }";
    cFile.setModifiedText(text);
    //System.out.println("Adding : "+propName+".prop");
    this.cocoFiles.put(propName+".prop", cFile);
    return cFile;
  }

  /**
   * Loads a Project File.
   */
  private void loadProject() {
	Assert.require(this.projectFileName);
        Assert.require(this.projectPath);

        File f = new File(this.projectPath + this.projectFileName);

	try {
          // read properties
	    FileInputStream in = new FileInputStream(f);
	    Properties props = new Properties();
	    props.load(in);
	    in.close();

          // classify the properties
	    Enumeration names = props.propertyNames();
	    while (names.hasMoreElements()) {
		String name = (String)names.nextElement();
		String value = props.getProperty(name);

                String fileName = value;
                CoCoFile cf = new CoCoFile(fileName, this.projectPath, this.projectName);
                //System.out.println("Adding : "+fileName);
                cocoFiles.put(fileName, cf);
	    }
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }
}
