package coco.semantic;

import java.util.*;

import coco.parser.ast.*;
import coco.parser.symtab.*;
import coco.utils.*;


/**
 * This class tests ASTConnectors.
 */
public class ConnectorTester extends Tester {

    ASTConnector myCon;

    /**
     * Checks, if this tester can test the given node.
     * @parameter n Node to test
     * @return true: Tester can test the given node
     */
    protected boolean isCompatible(GenericNode n) {
	boolean res = false;

	if (n instanceof ASTConnector) {
	    res = true;
	}
	return(res);
    }

    /**
     * The 'real' test method. Is called, if and only if, the isCompatible method
     * returned 'true'.
     * @parameter n node to test
     * @return true: test succeeded
     */
    protected boolean testInternal(GenericNode n) {
	boolean res = true;

	ASTConnector con = (ASTConnector)n;
	myCon = con;

	ASTPortTypeReferenceList srcPorts = con.getSourcePorts();
	if (srcPorts != null) {
	    if (!testPortTypeReferenceList(srcPorts)) {
		res = false;
	    }

	}

	ASTPortTypeReferenceList tgtPorts = con.getTargetPorts();
	if (tgtPorts != null) {
	    if (!testPortTypeReferenceList(tgtPorts)) {
		res = false;
	    }
	}

	return(res);
    }

    /**
     * This mehod checks port type reference lists
     * @param l PortTypeReferenceList to check
     */
     private boolean testPortTypeReferenceList(ASTPortTypeReferenceList l) {
	boolean res = true;

	Assert.require(l);

	Enumeration e = checkEnum(l.getNodes());

	if (e != null) {

	    for ( ; e.hasMoreElements(); ) {
		ASTPortTypeReference tref = (ASTPortTypeReference)e.nextElement();

		if (!checkPortTypeReference(tref)) {
		    res = false;
		}
	    }
	}

	return(res);
     }

    /**
     * This method checks port type references used in connectors.
     * @param ref PortTypeReference to check
     */
    private boolean checkPortTypeReference(ASTPortTypeReference ref) {
	boolean res = true;

	if (ref != null) {
	    if (ref instanceof ASTDataTypeReference) {
		ASTDataTypeReference sref = (ASTDataTypeReference)ref;

		if (!checkType(sref.getTypeId(), null)) {
		    res = false;
		    reportError("unknown data type " + sref.getTypeId());
		}

	    } else {
		ASTInvocationParameterList paramList = null;

		if (ref instanceof ASTInvocationTypeReference) {
		    ASTInvocationTypeReference eref = (ASTInvocationTypeReference)ref;

		    // get all parameters
		    paramList = eref.getSignature().getInvocationParams();

		} else if (ref instanceof ASTHandlerTypeReference)  {
		    ASTHandlerTypeReference eref = (ASTHandlerTypeReference)ref;

		    // get all parameters
		    paramList = eref.getSignature().getInvocationParams();
		}

		if (paramList != null) {
		    for (Enumeration e = checkEnum(paramList.getNodes()); e.hasMoreElements(); ) {
			ASTInvocationParameter param = (ASTInvocationParameter)e.nextElement();
			ASTDataTypeIdentifier t = param.getParamType();

			// can we find the type of the data ?
			// result will be false, if the data has neither
			// a primitive type, nor a type registered in the symbol table
			if (!checkType(t, null)) {
			    res = false;
			    reportError("unknown invocation parameter type " + t);
			}
		    }
		}
	    }
	}
	return(res);
    }

    protected void reportError(String msg) {
	String ctx = "ConnectorTester";

	if (myCon != null) {
	    ctx = ctx + " (" + myCon.getConnectorID() + ")";
	}

	reportError(ctx, msg);
    }
}

