package coco.semantic;

import java.util.*;

import coco.parser.ast.*;
import coco.parser.symtab.*;
import coco.utils.*;

/**
 * This class tests ASTDataTypes.
 */
public class DataTypeTester extends Tester {

    ASTDataType myDataType;

    /**
     * Checks, if this tester can test the given node.
     * @parameter n Node to test
     * @return true: Tester can test the given node
     */
    protected boolean isCompatible(GenericNode n) {
	boolean res = false;

	if (n instanceof ASTDataType) {
	    res = true;
	}
	return(res);
    }

    /**
     * The 'real' test method. Is called, if and only if, the isCompatible method
     * returned 'true'.
     * @parameter n node to test
     * @return true: test succeeded
     */
    protected boolean testInternal(GenericNode n) {
      boolean res = true;

      ASTDataType  p = (ASTDataType)n;
      myDataType = p;

      // is basetype already known ?

      ASTIdentifier bt = p.getBasetypeIdentifier();
      if ((bt != null) && (!(isDataTypeDeclared(bt)))) {
	  res = false;
	  reportError("unknown basetype: " + bt);
      }

      // fields: types known; different names;
      // same names for fields is handled by the symtab during register process
      
      ASTFieldList fields = p.getFieldList();
      if (fields != null) {
	  for (Enumeration e = checkEnum(fields.getNodes()); e.hasMoreElements(); ) {
	      ASTField f = (ASTField)e.nextElement();
	      
	      ASTDataTypeIdentifier dtid = f.getTypeId();

	      // check only, if field type is user defined.
	      if (isUserDefinedDataType(dtid)) {
		  ASTIdentifier tid = dtid.getTypeID();
		  if (!(isDataTypeDeclared(tid))) {
		      res = false;
		      reportError("unknown data type for field " + f);
		  }
	      }
	  }
      }

      return(res);
    }
    
    protected void reportError(String msg) {
	String ctx = "DataTypeTester";

	if (myDataType != null) {
	    ctx = ctx + " (" + myDataType.getDataTypeIdentifier().toString() + ")";
	}

	reportError(ctx, msg);
    }
}

