package coco.semantic;

import java.util.*;

import coco.parser.ast.*;
import coco.parser.symtab.*;


/**
 * This class tests ASTInterfaces.
 */
public class InterfaceTester extends Tester {

    ASTInterface myIntf;

    /**
     * Checks, if this tester can test the given node.
     * @parameter n Node to test
     * @return true: Tester can test the given node
     */
    protected boolean isCompatible(GenericNode n) {
	boolean res = false;

	if (n instanceof ASTInterface) {
	    res = true;
	}
	return(res);
    }

    /**
     * The 'real' test method. Is called, if and only if, the isCompatible method
     * returned 'true'.
     * @parameter n node to test
     * @return true: test succeeded
     */
    protected boolean testInternal(GenericNode n) {
	boolean res = true;

	ASTInterface i = (ASTInterface)n;
	myIntf = i;

	// all base interfaces known ?
	// all port types known ?

	Enumeration e = checkEnum(checkList(i.getInterfaceList()).getNodes());

	if (e != null) {

	    for ( ; e.hasMoreElements(); ) {
		ASTIdentifier id = (ASTIdentifier)e.nextElement();

		if (!(isInterfaceDeclared(id))) {
		    res = false;
		    reportError("unknown interface " + id);
		}
	    }
	}

	// check ports
	// get ports from symtab service with subject = S_PORT
	Enumeration enum = checkEnum(SymtabService.getLocals(i, SymtabService.S_PORT));
	if (enum != null) {
	    for (; enum.hasMoreElements(); ) {
		ASTPort p = (ASTPort)enum.nextElement();

		if (!checkPortType(p)) {
		    res = false;
		}
	    }
	}

	return(res);
    }


    protected void reportError(String msg) {
	String ctx = "InterfaceTester";

	if (myIntf != null) {
	    ctx = ctx + " (" + myIntf.getID() + ")";
	}

	reportError(ctx, msg);
    }
}

