package coco.semantic;

import java.util.*;


public class TypeGraph {

    // singleton part

    private static TypeGraph instance;

    public static TypeGraph getInstance() {
	if (instance == null) {
	    instance = new TypeGraph();
	}
	return(instance);
    }

    // class part

    Hashtable nodes;

    private TypeGraph() {
	init();
    }

    private void init() {
	nodes = new Hashtable();
    }

    // Reset
    public void reset() {
	init();
    }

    /**
     * Adds a relation btw the given entities.
     * Both are added to the graph.
     */
    public void addRelation(String from, String to) {
	if ((from != null) && (to != null)) {

	    if (!haveRelation(from, to)) {
		TGNode f = getNode(from);
		TGNode t = getNode(to);

		f.addRelation(t);
	    }
	}
    }

    /**
     * Checks, if entity 'from' has a relationship with entity 'to'.
     */
    public boolean haveRelation(String from, String to) {
	boolean res = false;

	TGNode f = lookupNode(from);
	TGNode t = lookupNode(to);

	if ((f != null) && (t != null)) {
	    Enumeration e = f.getChildren();
	    while (e.hasMoreElements()) {
		if (e.nextElement().equals(t)) {
		    res = true;
		    break;
		}
	    }
	}
	return(res);
    }

    /**
     * Retrieves all entities, 'from' has a relationship to.
     * @return Enumeration of Strings
     */
    public Enumeration getChildren(String from) {
	Vector res = new Vector();

	TGNode f = lookupNode(from);

	if (f != null) {
	    Enumeration e = f.getChildren();
	    while(e.hasMoreElements()) {
		res.add(((TGNode)e.nextElement()).getId());
	    }
	}
	return(res.elements());
    }




    // retrieve node from list or create new node
    private TGNode getNode(String id) {
	TGNode res = lookupNode(id);

	if (res == null) {
	    res = createNode(id);
	    nodes.put(id, res);
	}

	return(res);
    }

    // lookup node in list
    private TGNode lookupNode(String id) {
	TGNode res = null;

	Object o = nodes.get(id);
	if (o != null) {
	    res = (TGNode)o;
	}

	return(res);
    }

    // create new TGNode object 
    private TGNode createNode(String id) {
	TGNode res = new TGNode(id);
	return(res);
    }





    // This is the node class in the type graph
    static class TGNode {
	String id;
	Vector children;

	public TGNode(String id) {
	    this.id = id;
	    children = new Vector();
	}
	
	public void addRelation(TGNode other) {
	    if (other != null) {
		children.add(other);
	    }
	}

	public Enumeration getChildren() {
	    return(children.elements());
	}
	
	public String getId() {
	    return(id);
	}

	public boolean equals(Object o) {
	    boolean res = false;

	    if (o instanceof TGNode) {
		TGNode n = (TGNode)o;
		res = id.equals(n.id);
	    }
	    return(res);
	}

    }
}
