package coco.tools;
import coco.project.*;
import coco.filehandling.*;
import coco.parser.*;
import coco.parser.ast.*;
import coco.parser.symtab.*;
import coco.semantic.*;

import java.util.*;
import java.io.*;

public class Checker extends CoCoToolAdapter {

    /** 
     * Retrieves the menu entry in the tools menu of the application.
     */
    public String getMenuText() {return("Checker");}

    /**
     * Retrieves the description used for help purposes.
     */
    public String getDescription() {return("Semantic checker");}

    /**
     * Retrieves a version string for this tool.
     */
    public String getVersion() {return("0.1");}

    /**
     * Starts the operation of this tool. 
     * @param project the current project to operate on.
     */
    public void execute(CoCoProject project) {
	System.err.println("\nStart checking at: " + new Date());
	System.out.println("\nStart checking at: " + new Date());

	System.err.println("Project-Name: " + project.getProjectName());
	System.out.println("Project-Name: " + project.getProjectName());

	// do initialization
	if (init(project)) {

	    // start the semantic checker
	    if (SemanticService.testAllEntities()) {
		System.out.println("Semantic checking was successful");
	    } else {
		System.out.println("Semantic checker returned errors. Output should have been provided.");
		System.err.println("Semantic checker returned errors. Output should have been provided.");
	    }
	} else {
	    System.err.println("Semantic checker: errors during initialization");
	}
    }

    /**
     * This is a private initialization method.
     * It is called prior to any action in 'execute(...)'
     */
    private boolean init(CoCoProject project) {
	boolean res = true;

	// Clean symtab
	SymtabService.removeAll();

	// Enumerate all files, that belong to the project
	// and parse them using the 'real' parser
	Enumeration files = project.getCoCoFiles();
	while(files.hasMoreElements()) {
	    CoCoFile f = (CoCoFile)files.nextElement();
	    String text = f.getActualText();

	    CoCoParser parser = ParserService.getCoCoParser(new StringReader(text));

	    try {
		ASTCompilationUnit unit = parser.compilationUnit();

		if (!parser.success()) {
		    Enumeration errors = parser.getErrors().elements();
		    
		    while (errors.hasMoreElements()) {
			System.out.println(errors.nextElement());
		    }

		    res = false;
		}
	    } catch (Exception ex) {
		ex.printStackTrace();
		res = false;
	    }
	}
	return(res);
    }
}
