package coco.tools;

import java.io.*;
import java.util.*;
import javax.swing.*;

import coco.codegen.prolog.*;
import coco.project.*;
import coco.filehandling.*;
import coco.parser.*;
import coco.parser.ast.*;
import coco.parser.symtab.*;
import coco.semantic.*;


public class PrologFactsGenerator extends CoCoToolAdapter {
   /** 
     * Retrieves the menu entry in the tools menu of the application.
     */
    public String getMenuText() {return("PrologGenerator");}

    /**
     * Retrieves the description used for help purposes.
     */
    public String getDescription() {return("Generate prolog facts");}

    /**
     * Retrieves a version string for this tool.
     */
    public String getVersion() {return("0.1");}

    /**
     * Starts the operation of this tool. 
     * @param project the current project to operate on.
     */
    public void execute(CoCoProject project) {

	try {
	    System.out.println("PrologFactsGenerator");
	    System.out.println("Project-Name: " + project.getProjectName());
	    
	    // do initialization
	    if (init(project)) {
		
		// Create an writer to write the facts to
		Writer writer = null;
		
		// choose file to write prolog facts to
		
		JFileChooser chooser = new JFileChooser();
		chooser.setFileFilter(new CoCoFileFilter("pl"));
		int returnVal = chooser.showSaveDialog(null); 
		
		if(returnVal == JFileChooser.APPROVE_OPTION) { 
		    File plFile = chooser.getSelectedFile();
		    System.out.println("Prolog facts will be written to: " + plFile);
		    
		    writer = new FileWriter(plFile);

		    // start the Prolog generation
		    PrologGenerator pg = new PrologGenerator();
		    String str = pg.getResult();

		    writer.write(str);
		    writer.flush();

		    // close file
		    if (writer instanceof FileWriter) {
			writer.close();
		    }
		}
	    } else {
		System.err.println("Prolog generator: errors during initialization");
	    }
	} catch (Exception ex) {
	    ex.printStackTrace();
	}

    }

    /**
     * This is a private initialization method.
     * It is called prior to any action in 'execute(...)'
     */
    private boolean init(CoCoProject project) {
	boolean res = true;

	// Clean symtab
	SymtabService.removeAll();

	// Enumerate all files, that belong to the project
	// and parse them using the 'real' parser
	Enumeration files = project.getCoCoFiles();
	while(files.hasMoreElements()) {
	    CoCoFile f = (CoCoFile)files.nextElement();
	    String text = f.getActualText();

	    CoCoParser parser = ParserService.getCoCoParser(new StringReader(text));

	    try {
		ASTCompilationUnit unit = parser.compilationUnit();

		if (!parser.success()) {
		    Enumeration errors = parser.getErrors().elements();
		    
		    while (errors.hasMoreElements()) {
			System.out.println(errors.nextElement());
		    }

		    res = false;
		}
	    } catch (Exception ex) {
		ex.printStackTrace();
	    }
	}
	return(res);
    }

}
