package coco.tools;
import coco.project.*;
import coco.filehandling.*;
import coco.parser.*;
import coco.parser.ast.*;
import coco.parser.symtab.*;

import java.util.*;
import java.io.*;

public class Tool1 extends CoCoToolAdapter {

    /** 
     * Retrieves the menu entry in the tools menu of the application.
     */
    public String getMenuText() {return("Tool1");}

    /**
     * Retrieves the description used for help purposes.
     */
    public String getDescription() {return("TestTool1");}

    /**
     * Retrieves a version string for this tool.
     */
    public String getVersion() {return("0.1");}

    /**
     * Starts the operation of this tool. 
     * @param project the current project to operate on.
     */
    public void execute(CoCoProject project) {
	System.out.println("Tool1");
	System.out.println("Project-Name: " + project.getProjectName());

	// do initialization
	init(project);

	SymtabService.dumpComponentSymtab(System.out);
	

    }

    /**
     * This is a private initialization method.
     * It is called prior to any action in 'execute(...)'
     */
    private void init(CoCoProject project) {
	// Clean symtab
	SymtabService.removeAll();

	// Enumerate all files, that belong to the project
	// and parse them using the 'real' parser
	Enumeration files = project.getCoCoFiles();
	while(files.hasMoreElements()) {
	    CoCoFile f = (CoCoFile)files.nextElement();
	    String text = f.getActualText();

	    CoCoParser parser = ParserService.getCoCoParser(new StringReader(text));

	    try {
		ASTCompilationUnit unit = parser.compilationUnit();

		if (!parser.success()) {
		    Enumeration errors = parser.getErrors().elements();
		    
		    while (errors.hasMoreElements()) {
			System.out.println(errors.nextElement());
		    }
		}
	    } catch (Exception ex) {
		ex.printStackTrace();
	    }
	}
    }
}
