package coco.filehandling;

import java.util.*;
import java.io.*;

import coco.utils.*;
import coco.config.GlobalConfig;
/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author
 * @version 1.0
 */

public final class RecentFilesManager {

    static private RecentFilesManager instance = null;
    private String recentFilesFile = null;

    static private final String PROP_RECENT_FILE = "Recent-File";
    static private final String PROP_RECENT_PROJECT = "Recent-Project";

    //  private Properties recentFilesList = new Properties();

    private Vector recentProjects = null;
    private Vector recentFiles = null;


    private String checkForHistoryFile() throws java.io.IOException{
	// first we search in the users home directory
	String res = createHistoryFileName(System.getProperty("user.home"));
	File f = new File(res);
	// if the file exists, return the name
	if(f.isFile() && f.canRead())
	    return res;

	// otherwise search or create it in the current directory
	res = createHistoryFileName(System.getProperty("user.dir"));
	f = new File(res);
	// if the file still doesn't exist, create a new one
	if(!f.isFile()){
	    f.createNewFile();
	}
	return res;
    }

    private Object isContained(String path, Vector v){
	Object res = null;
	String tmp = null;
	int max = v.size();
	for(int idx = 0; (idx < max) && (res == null); idx++){
	    tmp = (String)v.elementAt(idx);
	    Assert.require(tmp);
	    if(tmp.compareTo(path) == 0){
		res = tmp;
	    }
	}
	return res;
    }

    private void compactify(Vector v){
	String nFiles = GlobalConfig.getInstance().getProperty(GlobalConfig.OPTION_RECENT_FILES);
	int maxSz = Integer.parseInt(nFiles) -1 ;

	int currSz = v.size();
	if(currSz > maxSz){
	    v.setSize(maxSz);
	}


    }

    private void populateLists(Properties props){
	recentFiles.removeAllElements();
	recentProjects.removeAllElements();

	Enumeration keys = props.keys();
	String key = null;
	while(keys.hasMoreElements()){
	    key = (String)keys.nextElement();

	    if(key.startsWith(this.PROP_RECENT_FILE)){
		recentFiles.add(0, props.getProperty(key));
	    }else if(key.startsWith(this.PROP_RECENT_PROJECT)){
		recentProjects.add(0, props.getProperty(key));
	    }
	}
    }

    private Properties createPropertyRep(){
	Properties res = new Properties();
	int max = recentFiles.size();
	String tmp = null;

	for(int idx = 0; idx < max; idx++){
	    tmp = (String) recentFiles.elementAt(idx);
	    Assert.require(tmp);
	    res.setProperty(this.PROP_RECENT_FILE+Integer.toString(idx), tmp);
	}

	max = recentProjects.size();
	for(int idx = 0; idx < max; idx++){
	    tmp = (String) recentProjects.elementAt(idx);
	    Assert.require(tmp);
	    res.setProperty(this.PROP_RECENT_PROJECT+Integer.toString(idx), tmp);
	}
	return res;
    }

    private String createHistoryFileName(String basePath){
	Assert.require(basePath);
	String path = basePath;
	String pathDelim = System.getProperty("file.separator");
	Assert.require(pathDelim);
	path += pathDelim + "coco.history";
	return path;
    }


    public void readHistory() throws java.io.IOException{
	String path = checkForHistoryFile();
	Properties recentFilesList = new Properties();
	Assert.require(path);
	Assert.require(path.length());

	System.out.println("Read recent-file-list from file " + path);
	FileInputStream reader = new FileInputStream(path);
	recentFilesList.load(reader);
	populateLists(recentFilesList);

	System.out.println("Finished reading recent-file-list from file " + path);
    }


    public void writeHistory() throws java.io.IOException{
	String path = checkForHistoryFile();
	Assert.require(path);
	Assert.require(path.length());

	Properties recentFilesList = createPropertyRep();

	FileOutputStream writer = new FileOutputStream(path);
	recentFilesList.store(writer,"PECOS Composition Environment recent files");
	System.out.println("Wrote recent-file-list to file " + path);
    }

    private RecentFilesManager() {
	recentProjects = new Vector(5,2);
	recentFiles = new Vector(5,2);
    }





    static public RecentFilesManager getInstance(){
	if(instance == null){
	    instance = new RecentFilesManager();
	}
	return instance;
    }

    public void addRecentFile(String path){
	Object o = isContained(path, recentFiles);
	if(o == null)
	    recentFiles.add(0, path);
	else{
	    //      System.out.println("Already contained...");
	    recentFiles.removeElement(o);
	    recentFiles.add(0,path);
	}
	compactify(recentFiles);

    }

    public void addRecentProject(String path){
	Object o = isContained(path, recentProjects);
	if(o == null)
	    recentProjects.add(0, path);
	else{
	    recentProjects.removeElement(o);
	    recentProjects.add(0,path);
	}
	compactify(recentProjects);
    }



    public Vector getRecentFiles(){
	return recentFiles;
    }
    public Vector getRecentProjects(){
	return recentProjects;
    }


}
