package coco.fuzzyparser;

import java.io.File;

import coco.utils.Assert;
import coco.model.nodes.ErrorListTreeNode;
/**
 * berschrift:   PECOS Composition Environment
 * Beschreibung:
 * Copyright:     Copyright (c) 2001
 * Organisation:  FZI
 * @author
 * @version 1.0
 */

public class FuzzyParserService {
  static private FuzzyParser fuzzyParser = null;

/**
 * getfuzzyParser returns an instance of a fuzzy parser, that reads from a FileInputStream.
 * An assertion fails if is is null.
 * @param is - the FileInputStream to read from
 * @return  an instance of fuzzyParser
 */
  static final public FuzzyParser getfuzzyParser(java.io.FileInputStream is, String f){
    Assert.require(is);
    if(fuzzyParser == null){
     fuzzyParser = new FuzzyParser(is);
     fuzzyParser.reset();
    }else{
     fuzzyParser.ReInit(is);
    }
    fuzzyParser.setCurrentFileName(f);
    return fuzzyParser;
  }

/**
 * getfuzzyParser returns an instance of a fuzzy parser, that reads from a Reader.
 * An assertion fails if r is null.
 * @param r the Reader to read from
 * @return  an instance of fuzzyParser
 */
  static final public FuzzyParser getfuzzyParser(java.io.Reader r, String f){
    Assert.require(r);
    if(fuzzyParser == null){
     fuzzyParser = new FuzzyParser(r);
     fuzzyParser.reset();
    }else{
     fuzzyParser.ReInit(r);
    }
    fuzzyParser.setCurrentFileName(f);
    return fuzzyParser;
  }


  // reset the fuzzyParser, if allready there
  static final public void reset(){
    if (fuzzyParser != null) {
      fuzzyParser.reset();
    }
  }

  // get Errors
  static final public ErrorListTreeNode getErrors(){
    if (fuzzyParser != null) {
      return fuzzyParser.getErrors();
    } else {
      return null;
    }
  }

}
