


/*
 *	Parser specification for the composition language of
 *  the Pecos composition environment prototype
 *  T. Genssler, 2000
 *
 */


options{
	ERROR_REPORTING = true;

}


PARSER_BEGIN(FuzzyParser)

package coco.fuzzyparser;

import java.util.Enumeration;
import java.util.Hashtable;
import java.io.File;

import coco.model.nodes.*;

public class FuzzyParser{


	private static int numErrors = 0;
	private static ErrorListTreeNode errNode = null;
	private static String currentFileName = null;


public static void setCurrentFileName(String f) {
	currentFileName = f;
  }

static public void reset(){
  	errNode = new ErrorListTreeNode("Errors");
  	numErrors = 0;
  }

/** add an error tree node to the list of current errors
  *
  */

static private void addError(ParseException e){
	 //ErrorTreeNode err = new ErrorTreeNode("Error in File "+currentFileName+" at line "+token.beginLine);
	 ErrorTreeNode err = new ErrorTreeNode("Error at (" + token.beginLine + ":" + token.beginColumn +")");

	 err.setDescription(e.toString());

	 err.setBeginLine(token.beginLine);
	 err.setSourceFileName(currentFileName);
	 errNode.add(err);
}

static public ErrorListTreeNode getErrors(){

  	return errNode;
  }

static public boolean success(){
  	return numErrors == 0;
  }

  public static void main(String args[]) {
    FuzzyParser parser = null;


    System.out.println("PECOS Composition Compiler Version 0.4a (CoCo V3), (c) FZI 2001");
    if (args.length == 0) {
      System.out.println("No file given . . . Bailed out !");
      System.exit(1);
    } else if (args.length == 1) {
      //System.out.println("Reading from file " + args[0] + " . . .");

      try {
        parser = new FuzzyParser(new java.io.FileInputStream(args[0]));
      } catch (java.io.FileNotFoundException e) {
        System.out.println("Error: File " + args[0] + " not found.");
        return;
      }
    } else {

    	System.out.println("This is free software which is licensed under the GPL.");
    	System.out.println("A recent copy of the GPL can be obtained at http://www.gnu.org/copyleft/gpl.html");
			System.out.println("SYNOPSIS:");
      System.out.println("java -jar cocop.jar < inputfile");
      System.out.println("OR");
      System.out.println("java -jar cocop.jar inputfile");
      System.out.println("If no input file is given, the parser reads from stdin.");
      return;
    }

    try{
    	parser.component();
    	parser.abstractComponent();
    }catch (ParseException ex){
			System.err.println("Uncaught exception");
    	System.err.println(ex.getMessage());
    }

    System.out.println("Error list");
    System.out.println("==========");

    //Enumeration enum = errors.elements();
    //while(enum.hasMoreElements()){
	//		ParseException e = (ParseException)enum.nextElement();
	//		System.out.println(e.getMessage());
	//	}

  }

}

PARSER_END(FuzzyParser)


/*
 * Tokens to ignore in the BNF follow.
 */

SKIP :
{
  < " " >
| < "\t" >
| < "\n" >
| < "\r" >
| < "//" (~["\n"])* "\n" >
| <"/*" (~["*"])* "*" (~["/"] (~["*"])* "*")* "/">


}

// keywords of the component and composition language
TOKEN :
{

    <ABSTRACT		: "abstract">
  |	<ACTIVE	    : "active">
  | <ANY: "any">
  | <BOOL				: "bool">
  | <BYTE				: "byte">
  | <CHAR				: "char">
  | <COMPONENT 	: "component">
  | <CONNECTOR	: "connector">
	| <DOUBLE			: "double">
	| <EXTENDS		: "extends">
	| <EXTERN 		: "extern">
	| <EVENT			: "event">
	| <FALSE			: "false">
	| <FLOAT			: "float">
	| <HAS			: "has">
	| <IS					: "is">
	| <IN					: "in">
	| <INOUT			: "inout">
	| <INT				: "int">
	| <LONG				: "long">
	| <MANDATORY	: "mandatory">
	| <PACKAGE		: "package">
	| <PROPERTIES	: "properties">
	| <PROPERTY		: "property">
  | <OPTIONAL		: "optional">
	|	<OUT				: "out">
	| <SHORT			: "short">
	| <TRUE				: "true">
	| <TYPE				: "type">
	| <VOID       : "void">
  | <ROLE 			: "role">
  | <PORT			: "port">

}



TOKEN :
{
  <  ID : (["a"-"z","A"-"Z", "_"])* (["a"-"z","A"-"Z"])+ (["a"-"z","A"-"Z","0"-"9","_"])* >
| <  OCTALINT : "0" (["0"-"7"])* (["u","U","l","L"])? >
| <  DECIMALINT : ["1"-"9"] (["0"-"9"])* (["u","U","l","L"])? >
| <  HEXADECIMALINT : ("0x"|"0X") (["0"-"9","a"-"f","A"-"F"])+ (["u","U","l","L"])? >
| <  FLOATONE : ((["0"-"9"])+ "." (["0"-"9"])* | (["0"-"9"])* "." (["0"-"9"])+)
   (["e","E"] (["-","+"])? (["0"-"9"])+)? (["f","F","l","L"])? >
| <  FLOATTWO : (["0"-"9"])+ ["e","E"] (["-","+"])?
   (["0"-"9"])+ (["f","F","l","L"])? >



| <ASSGMNT	: "=">
| <L_PAREN	: "(">
| <R_PAREN	: ")">
| <L_BRACE	:"{">
| <R_BRACE	:"}">
| <L_BRACKET:"[">
| <R_BRACKET: "]">
| <SEMICOLON	:";">
| <CONNECTS : "->">
| <COLON : ":">
| <DEFINED : ":=">
| <EQ : "==">
| <NEQ : "<>">
| <LT : "<">
| <LTE :"<=">
| <GT : ">">
| <GTE : ">=">
| <COMMA : ",">
| <HOLDS : ":-">
| <DOT : ".">
| <RULETEXT : "${"((~[])+)"}$">
| <CHARACTER : "'"
   (   (~["'","\\","\n","\r"])
   | ("\\" (
             ["n","t","v","b","r","f","a","\\","?","'","\""]
            |
             "0" (["0"-"7"])*
            |
             ["1"-"9"] (["0"-"9"])*
            |
             ("0x" | "0X") (["0"-"9","a"-"f","A"-"F"])+
           )
     )
   )
   "'" >
| <  STRING : "\""
   ( ( ~["\"","\\","\n","\r"])
   | ("\\" (
             ["n","t","v","b","r","f","a","\\","?","'","\""]
            |
             "0" (["0"-"7"])*
            |
             ["1"-"9"] (["0"-"9"])*
            |
             ("0x" | "0X") (["0"-"9","a"-"f","A"-"F"])+
           )
     )
   )*
   "\"" >

| <  MULTILINE : "\"" (~["\""])* "\"" >

//| <LINE_COMMENT : "//" (~["\n"])* "\n" >
//| <COMMENT :"/*" (~["*"])* "*" (~["/"] (~["*"])* "*")* "/">


| < UNCLOSED_COMMENT : "/*"~[]>


// eat all the rest
| <UNKNOWN 	: 	~[]>

}

/* ------------------------------------------------------------------------------------------------ */

CoCoTreeNode component() :
{

	ComponentTreeNode n = null;

	String name = null;
	int beginLine = -1;
	int endLine = -1;

	InterfaceListTreeNode ifl = new InterfaceListTreeNode("Interfaces");
	PropertyTreeNode pr = null;
	PropertyListTreeNode prl = new PropertyListTreeNode("Properties");
	PortTreeNode po = null;
	PortListTreeNode pol = new PortListTreeNode("Ports");
	InstanceTreeNode in = null;
	InstanceListTreeNode inl = new InstanceListTreeNode("Instances");
	ConnectorTreeNode co = null;
	ConnectorListTreeNode col = new ConnectorListTreeNode("Connectors");

	int skipToTokens[] = {L_BRACE};
}{

	try{

	(
		[
	  	   ( <EVENT> | <ACTIVE>)

			  	{
				  beginLine = token.beginLine;
				}
		]


                <COMPONENT>
              			{
               			  if (beginLine == -1) {beginLine = token.beginLine;}
               		    	}

		name = identifier()

				{ // create component node here
				  n = new ComponentTreeNode(name);
				  n.setBeginLine(beginLine);
				  n.setName(name);
				}

		(
		          ( [<IS> ifl = interfaceList()][<HAS> propertySetReferenceList()] <L_BRACE>)

			| ( try_skipto_ex(L_BRACE, skipToTokens, true) <L_BRACE>)

		)

				{
				  if (ifl.hasChildren()) {n.add(ifl);}
				}

		(
			 LOOKAHEAD(2)
			 pr = property()

			  	{
			  	  if (pr != null) {prl.add(pr);}
			  	}

			| po = port()

				{
				  if (po != null) {pol.add(po);}
				}

			| in = instance()

				{
				  if (in != null) {inl.add(in);}
				}
			| co = connector()
				{
				  if (co != null) {col.add(co);}
				}

		)*
				{
				  if (ifl.hasChildren()) {n.add(ifl);}
				  if (prl.hasChildren()) {n.add(prl);}
				  if (pol.hasChildren()) {n.add(pol);}
				  if (inl.hasChildren()) {n.add(inl);}
				  if (col.hasChildren()) {n.add(col);}
				}

		<R_BRACE>
				{
				  n.setEndLine(token.endLine);
				}
		[<SEMICOLON>]
		<EOF>
	){
		// -- saw complete component
		//System.out.print("Saw component " + name);
                //System.out.println(", from "+Integer.toString(n.getBeginLine()) + " to " + Integer.toString(n.getEndLine()));

                return n;

                }


}catch(ParseException e){
	// stabilizing
	error_skipto(SEMICOLON, e, "Err: Could not recognize <COMPONENT>");
        return n;
        }
}


/* ------------------------- */

CoCoTreeNode abstractComponent() :
{

	AbstractComponentTreeNode n = null;

	String name = null;
	int beginLine = -1;
	int endLine = -1;

	InterfaceListTreeNode ifl = new InterfaceListTreeNode("Interfaces");
	PropertyTreeNode pr = null;
	PropertyListTreeNode prl = new PropertyListTreeNode("Properties");
	PortTreeNode po = null;
	PortListTreeNode pol = new PortListTreeNode("Ports");
	InstanceTreeNode in = null;
	InstanceListTreeNode inl = new InstanceListTreeNode("Instances");
	ConnectorTreeNode co = null;
	ConnectorListTreeNode col = new ConnectorListTreeNode("Connectors");
	RoleTreeNode ro = null;
	RoleListTreeNode rol = new RoleListTreeNode("Roles");


	int skipToTokens[] = {L_BRACE};
}{

	try{

	(

	  	   ( <ABSTRACT> )

			  	{
				  beginLine = token.beginLine;
				}



                <COMPONENT>
              			{
               			  if (beginLine == -1) {beginLine = token.beginLine;}
               		    	}

		name = identifier()

				{
				  n = new AbstractComponentTreeNode(name);
				  n.setBeginLine(beginLine);
				  n.setName(name);
				}

		(
			  ( [<EXTENDS> ifl = interfaceList()][<HAS> propertySetReferenceList()] <L_BRACE>)

			| ( try_skipto_ex(L_BRACE, skipToTokens, true) <L_BRACE>)
		)

				{
				  if (ifl.hasChildren()) {n.add(ifl);}
				}

		(
			 LOOKAHEAD(2)
			 pr = property()

			  	{
			  	  if (pr != null) {prl.add(pr);}
			  	}

			| po = port()

				{
				  if (po != null) {pol.add(po);}
				}

			| in = instance()

				{
				  if (in != null) {inl.add(in);}
				}
			| co = connector()
				{
				  if (co != null) {col.add(co);}
				}
			| ro = role()
				{
				  if (ro != null) {rol.add(ro);}
				}

		)*
				{
				  if (ifl.hasChildren()) {n.add(ifl);}
				  if (prl.hasChildren()) {n.add(prl);}
				  if (pol.hasChildren()) {n.add(pol);}
				  if (inl.hasChildren()) {n.add(inl);}
				  if (col.hasChildren()) {n.add(col);}
				  if (rol.hasChildren()) {n.add(rol);}
				}

		<R_BRACE>
				{
				  n.setEndLine(token.endLine);
				}
		[<SEMICOLON>]
		<EOF>
	){
		// -- saw complete component
		//System.out.print("Saw component " + name);
                //System.out.println(", from "+Integer.toString(n.getBeginLine()) + " to " + Integer.toString(n.getEndLine()));

                return n;

                }


}catch(ParseException e){
	// stabilizing
	error_skipto(SEMICOLON, e, "Err: Could not recognize <ABSTACT COMPONENT>");
	return n;
        }
}


/* ------------------------- */


InterfaceListTreeNode interfaceList():
{
	InterfaceTreeNode in = null;
	InterfaceListTreeNode ifl = new InterfaceListTreeNode("Interfaces");

	String name = null;
	int beginLine = -1;
	int endLine = -1;
}{
	try{

	name = identifier()

				{
				  in = new InterfaceTreeNode();
				  in.setName(name);
				  in.setBeginLine(token.beginLine);
				  ifl.add(in);
				  //System.out.println("---------- "+in.getName());
				  name = null;
				}

	(<COMMA>

	name = identifier()

				{
				  in = new InterfaceTreeNode("Interfaces");
				  in.setName(name);
				  in.setBeginLine(token.beginLine);
				  ifl.add(in);
				  //System.out.println("---------- "+in.getName());
				  name = null;
				}

	)*

	}catch(ParseException ex)
			{
				//System.out.println("asujkd");
				int tokens[] = {L_BRACE};
				error_skipto_ex(tokens, ex, "Err: <INTERFACE LIST>");
			}

	{return ifl;}
}


/* ------------------------- */


void propertySetReferenceList():
{
	String name = null;
	int beginLine = -1;
	int endLine = -1;
}{
	try{

	name = identifier()

				{
				}

	(<COMMA>

	name = identifier()

				{
				}

	)*

	}catch(ParseException ex)
			{
				//System.out.println("asujkd");
				int tokens[] = {L_BRACE};
				error_skipto_ex(tokens, ex, "Err: <INTERFACE LIST>");
			}

	{return;}
}


/* ------------------------- */

PropertySetTreeNode propertyset() :
{
	String name = null;
	int beginLine = -1;
	int endLine = -1;

	PropertyTreeNode pr = null;
	PropertySetTreeNode pl = new PropertySetTreeNode();
}

{
try{
	(
		( <PORT> | <COMPONENT> | <CONNECTOR>)

		<PROPERTIES>

		name = identifier()

				{
				  pl = new PropertySetTreeNode();
				  pl.setBeginLine(token.beginLine);
				  pl.setName(name);
				}

		<L_BRACE>

		(
			  pr = property()

			  	{
			  	  if (pr != null) {pl.add(pr);}
			  	}
		)*

		<R_BRACE>


	){
		pl.setEndLine(token.endLine);

		return(pl);
	}
	}catch(ParseException e){
		int tokens[] = {PROPERTY};
		error_skipto_ex(tokens, e, "Err: <propertyset>");

		return(pl);

	}
}

/* ------------------------- */

DatatypeTreeNode datatype() :
{
	String name = null;
	int beginLine = -1;
	int endLine = -1;

	FieldTreeNode fl = null;
	DatatypeTreeNode dt = null;
}

{
try{
	(
		<TYPE>

		name = identifier()

				{
				  dt = new DatatypeTreeNode();
				  dt.setBeginLine(token.beginLine);
				  dt.setName(name);
				}

		(
		 ( 
		  <EXTENDS>

		  /*name = */type()
		  (
		  		  (
		  		
		  			<L_BRACE>
		  
		  			(
		  				  fl = field()
		  
		  					{
		  					  if (fl != null) {dt.add(fl);}
		  					}
		  			)*
		  
		  			<R_BRACE>
		  
		  			[ <SEMICOLON> ]
		  		
		  		  )
		  		  | <SEMICOLON>
		  )

		)
		|
		(
			<L_BRACE>
			(
				  fl = field()

					{
					  if (fl != null) {dt.add(fl);}
					}
			)*
				  
		  	<R_BRACE>
		)
		)
		

	){
		dt.setEndLine(token.endLine);

		return(dt);
	}
	}catch(ParseException e){
		int tokens[] = {PROPERTY};
		error_skipto_ex(tokens, e, "Err: <data type>");

		return(dt);

	}
}

/* ------------------------- */

FieldTreeNode field() :
{
	String type = null;
        String name = null;
	int beginLine = -1;
	int endLine = -1;

	FieldTreeNode fl = null;
}
{

  try {
	(
		(   type()
		)
		

				{
				  beginLine = token.beginLine;
				}

		name = identifier()

				{
					fl = new FieldTreeNode();
					fl.setName(name);
					fl.setBeginLine(beginLine);
				}


		<SEMICOLON>



	){
		fl.setEndLine(token.endLine);
		return(fl);
	}
        }catch(ParseException e){
		int tokens[] = {PROPERTY};
		error_skipto_ex(tokens, e, "<INSTANCE>");

		return(fl);

	}


}


/* ------------------------- */

ConnectorTreeNode connector() :
{
	String name = null;
	int beginLine = -1;
	int endLine = -1;

	ConnectorTreeNode co = null;
}

{
try{
	(
		<CONNECTOR>

		(
		  name = identifier(){
		    beginLine = token.beginLine;
		  }
		  [<HAS> propertySetReferenceList()] 
		 )

				{
				  co = new ConnectorTreeNode();
				  co.setBeginLine(beginLine);
				  co.setName(name);

				  int tokens[] = {R_PAREN};
				  skipto_ex(tokens, false);
				}
		


	[ <L_BRACE>
				{
				  int tokens2[] = {R_BRACE};
				  skipto_ex(tokens2, false);
				}
	]

	[ <SEMICOLON> ]


	){
		co.setEndLine(token.endLine);

		//System.out.print("Saw connector " + name);
		//System.out.println(", from "+Integer.toString(co.getBeginLine()) + " to " + Integer.toString(co.getEndLine()));

		return(co);
	}
	}catch(ParseException e){
		int tokens[] = {PROPERTY, IN, OUT, INOUT, CONNECTOR};
		error_skipto_ex(tokens, e, "Err: <CONNECTOR>");

		return(co);

	}
}


/* ------------------------- */


PortTreeNode port() :
{
	String name = null;
	int beginLine = -1;
	int endLine = -1;

	PortTreeNode po = null;
}

{
try{
	(
		[
			<OPTIONAL>
		| <MANDATORY>
		]

		(
		 <IN>
		|<OUT>
		|<INOUT>
		)

				{
				  beginLine = token.beginLine;
				}

		type()

		name = identifier()
		[<HAS> propertySetReferenceList()] 

				{
				  po = new PortTreeNode();
				  po.setBeginLine(beginLine);
				  po.setName(name);

				  int tokens[] = {SEMICOLON, L_BRACE};
				  skipto_ex(tokens, false);
				}

		skip_balanced(L_BRACE, R_BRACE)

		[<SEMICOLON>]


	){
		po.setEndLine(token.endLine);

		//System.out.print("Saw port " + name);
		//System.out.println(", from "+Integer.toString(po.getBeginLine()) + " to " + Integer.toString(po.getEndLine()));

		return(po);
	}
	}catch(ParseException e){
		int tokens[] = {PROPERTY, IN, OUT, INOUT, CONNECTOR, R_BRACE};
		error_skipto_ex(tokens, e, "Err: Could not recognize <PORT>");

		return(po);

	}
}


/* ------------------------- */

void type() :
{
}
{
	(
		 <INT>
		|<BOOL>
		|<CHAR>
		|<FLOAT>
		|<DOUBLE>
		|<SHORT>
		|<LONG>
		|<BYTE>
		|<VOID>
		| <ANY>
		|identifier()
	)
	(<L_BRACKET> <DECIMALINT> <R_BRACKET>)*
	
	
	
	
}

/* ------------------------- */

InstanceTreeNode instance() :
{
	String type = null;
        String name = null;
	int beginLine = -1;
	int endLine = -1;

	InstanceTreeNode in = null;
}
{

  try {
	(
		type = identifier()

				{
				  beginLine = token.beginLine;
				}

		name = identifier()

	){
		in = new InstanceTreeNode();
		in.setName(name);
		in.setBeginLine(beginLine);


		skip_over(SEMICOLON);
		in.setEndLine(token.endLine);

		endLine = token.endLine;
		//System.out.print("Saw instance " + name);
		//System.out.println(", from "+Integer.toString(in.getBeginLine()) + " to " + Integer.toString(in.getEndLine()));

		return(in);
	}
        }catch(ParseException e){
		int tokens[] = {PROPERTY, IN, OUT, INOUT, CONNECTOR};
		error_skipto_ex(tokens, e, "<INSTANCE>");

		return(in);

	}


}

/* ------------------------- */

RoleTreeNode role() :
{
	String type = null;
        String name = null;
	int beginLine = -1;
	int endLine = -1;

	RoleTreeNode in = null;
}
{

  try {
	(
		( <ROLE> )

		type = identifier()

				{
				  beginLine = token.beginLine;
				}

		name = identifier()

	){
		in = new RoleTreeNode();
		in.setName(name);
		in.setBeginLine(beginLine);


		skip_over(SEMICOLON);
		in.setEndLine(token.endLine);

		endLine = token.endLine;
		//System.out.print("Saw instance " + name);
		//System.out.println(", from "+Integer.toString(in.getBeginLine()) + " to " + Integer.toString(in.getEndLine()));

		return(in);
	}
        }catch(ParseException e){
		int tokens[] = {PROPERTY, IN, OUT, INOUT, CONNECTOR};
		error_skipto_ex(tokens, e, "<ROLE>");

		return(in);

	}


}


/* ------------------------- */

PropertyTreeNode property() :
{

	String name = null;
	int[] skipToTokens={SEMICOLON};
	int beginLine = -1;
	int endLine = -1;

	PropertyTreeNode pr = null;

}{

  try {

	(

		[
					<OPTIONAL>
				| <MANDATORY>
		]
		<PROPERTY>

				{
				  beginLine = token.beginLine;
				}

		

		name = designator()

				{
				  pr = new PropertyTreeNode();
				  pr.setName(name);
				  pr.setBeginLine(beginLine);
				}

		(
			<ASSGMNT>skipto_ex(skipToTokens, true) <SEMICOLON>
		| <SEMICOLON>
		| ( try_skipto_ex(SEMICOLON, skipToTokens, true) <SEMICOLON>)
		)
	){
		pr.setEndLine(token.endLine);

		endLine = token.endLine;
		//System.out.print("Saw prop " + name);
		//System.out.println(", from "+Integer.toString(pr.getBeginLine()) + " to " + Integer.toString(pr.getEndLine()));

		return(pr);
	}
	}catch(ParseException e){
		int tokens[] = {PROPERTY, IN, OUT, INOUT, CONNECTOR};
		error_skipto_ex(tokens, e, "<PROPERTY>");

		return(pr);

	}


}


/* ------------------------- */

String designator() :
{StringBuffer res = new StringBuffer();
	}
{
	
  (<ID> dummy()){
  	res.append(token.image);
  }
  
  (
    <DOT>
    (<ID> dummy()){
    	res.append("."+token.image);
    }

  
  )*
  
  {
		return res.toString();
	}
}

/* ------------------------- */

String identifier() :
{
	}
{
  (<ID> dummy()){
		return token.image;
	}
}

/* ------------------------- */

JAVACODE

void skip_over(int kind){

	 while((token.kind != kind) && (token.kind != EOF)) {
	//	 System.out.println("-> " + token.image);

		 getNextToken();
	 };
	//System.out.println("Token after skip_over" + token.image);
}


JAVACODE

void try_skipto_ex(int expectedToken, int[] skipToTokens, boolean pushBack){

	try{
		//getNextToken();
		if(jj_ntk() != expectedToken){
			//System.out.println("Current token " + token.image);
			jj_consume_token(-1);
			throw new ParseException();
		}
	}catch(ParseException ex){
		error_skipto_ex(skipToTokens, ex, "try_skipto_ex ...");
	}
}


JAVACODE
void skipto_ex(int kind[], boolean pushBack){
	 Token t;
	 int len = kind.length;
	 boolean found = false;

	 int idx = 0;
	 boolean eof = false;
	 while(!found && !eof){
		 t = getNextToken();
		 eof = (t.kind == EOF);
		 for(idx=0; (idx<len) && (!found); idx++){
		  found = (t.kind == kind[idx]);

		 }
	 }

	 // we found one of the tokens in the given array. now push it back onto the token stream
	 // if necessary
	 if(found && pushBack){
//		Assert.require(idx <= tokenImage.length);
		idx--; // we are one position further than we want to be (post-fix increment in for loop, see above)
		String image = tokenImage[kind[idx]];
		//System.out.println(idx);
		//System.out.println("On token "+ image);
		int pushBackLen = image.length() - 2;
		if(pushBackLen > 0)
			jj_input_stream.backup(pushBackLen);
		else
			jj_input_stream.backup(1);
   }
}



JAVACODE
void skip_balanced(int startKind, int endKind){
	int sawStartKind = 1;
	if(token.kind != startKind){
	//	System.out.println("no balance!" + token.image);
		return;
	}
	//System.out.println("Balancing..." + token.image);
	do{
		getNextToken();
		if(token.kind == startKind) sawStartKind++;
		if(token.kind == endKind ) sawStartKind--;
	//	System.out.println("Token: " + token.image);
	//	System.out.println(sawStartKind);

	} while ((sawStartKind > 0) && (token.kind != EOF));

	//System.out.println("Token after balancing " + token.image);
}




JAVACODE
/**  Stabilizies the parser by skipping to the sepcified token.
   * The token itself is removed from the stream.
   * Side-effects: increases the numErros variable by one.
   *  @param kind the token to look for
   *  @param e the originally catched exception describing the actual parse problem
   */
void error_skipto(int kind, ParseException e, String errMsg) throws ParseException{

	 numErrors++;

		addError(e);
	 Token t;
	 do {
		 t = getNextToken();
		 // The above loop consumes tokens all the way upto a token of
		 // "kind".  We use a do-while loop rather than a while because the
		 // current token is the one immediately before the erroneous token
		 // (in our case the token immediately before what should have been there
	 } while ((t.kind != kind) && (t.kind != EOF));
		 	 // "if"/"while".
}


JAVACODE
/**  Stabilizies the parser by skipping to one of the specified tokens.
   * When an appropriate token is found it is pushed back onto the stream.
   * before the method returns. If no token is found the method simply
   * re-throws the original exception.
   * Side-effects: increases the numErros variable by one.
   *  @param kind[] the array of tokens to look for
   *  @param e the originally catched exception describing the actual parse problem
   */
void error_skipto_ex(int kind[], ParseException e, String errMsg) throws ParseException{

	 numErrors++;

		addError(e);
	 //System.out.println("Stabilizing ex...");
//	 ParseException e = generateParseException();  // generate the exception object.
//	 System.out.println(e.toString());  // print the error message
	 Token t;
	 int len = kind.length;
	 boolean found = false;

	 int idx = 0;
	 boolean eof = false;
	 while(!found && !eof){
		 t = getNextToken();
		 eof = (t.kind == EOF);
		 for(idx=0; (idx<len) && (!found); idx++){
		  found = (t.kind == kind[idx]);

		 }
	 }

	 // we found one of the tokens in the given array. now push it back onto the token stream
	 if(found){
//		Assert.require(idx <= tokenImage.length);
		idx--; // we are one position further than we want to be (post-fix increment in for loop, see above)
		String image = tokenImage[kind[idx]];
//		System.out.println(idx);
//		System.out.println("On token "+ image);
		int pushBackLen = image.length() - 2;
		if(pushBackLen > 0)
			jj_input_stream.backup(pushBackLen);
		else
			jj_input_stream.backup(1);
   }else {
			addError(new ParseException("Stabilizer failed"));
   }
}


JAVACODE
void dummy(){}
